@@ -19,6 +19,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 
 #ifndef __OpenBSD__
 # ifndef USE_UNIX98_PTY
@@ -92,6 +93,7 @@ extern void iax_disable_jitterbuffer(void);
 
 #define UNREGISTERED		0
 #define REGISTERED		1
+#define PENDING			2
 
 /* Packet length is 20 ms.  At 8000 Hz that's 160 samples. */
 #define VOIP_PACKET_LENGTH	20000
@@ -129,9 +131,11 @@ static char cidname[64];
 static char cidnumber[64];
 static char call_time[16];
 static char call_date[16];
+static char *dialextra = NULL;
 static int record = 0;
 static int replay = 0;
 static int nojitterbuffer = 0;
+static int commalen = 2;
 
 struct modem {
     int pid;
@@ -149,6 +153,14 @@ int logmode = S_IRUSR | S_IWUSR | S_IRGRP;
 #define LOG_ERROR   stderr
 #define LOG_INFO    stdout
 
+#ifdef SOLARIS
+#include "compat/daemon.c"
+#include "compat/timers.c"
+#include "compat/strings.c"
+#include "compat/headers.h"
+#include "sys/stropts.h"
+#endif
+
 void
 printlog(FILE *fp, char *fmt, ...)
 {
@@ -227,6 +239,28 @@ sighandler(int sig)
     cleanup(sig);
 }
 
+void
+sighandler_alarm(int sig)
+{
+    int other = 0;
+    char dtmf[2];
+    dtmf[1] = '\0';
+    while (dialextra && *dialextra != '\0' && *dialextra != ',') {
+	dtmf[0] = *dialextra;
+	printlog(LOG_INFO, "Dialing DTMF '%s'\n", dtmf);
+	iax_send_dtmf(session[0], *dialextra);
+	dialextra++;
+    }
+    while (dialextra && *dialextra == ',') {
+	other += commalen;
+	dialextra++;
+    }
+    if (other) {
+	printlog(LOG_INFO, "Dialing pause %d second(s)\n", other);
+	alarm(other);
+    } else dialextra = NULL;
+}
+
 void
 orderbytes(int16_t *buf, int len)
 {
@@ -423,10 +457,16 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 	    /* Dialing */
 	    if (modemstate != MODEM_ONHOOK && modemstate != MODEM_OFFHOOK) return -1;
 	    if (!strchr(num, '%')) {
-		printlog(LOG_INFO, "Dialing '%s'\n", num);
 		session[0] = iax_session_new();
+		char *nnum = strndup(num, 256);
+		char *nnump = strchr(nnum, ',');
+		if (nnump) {
+		    dialextra = strndup(nnump, 64);
+		    nnump[0] = '\0';
+		}
+		printlog(LOG_INFO, "Dialing '%s'\n", nnum);
 		char ich[256];
-		snprintf(ich, sizeof(ich), "%s:%s@%s/%s", regpeer, regsecret, server, num);
+		snprintf(ich, sizeof(ich), "%s:%s@%s/%s", regpeer, regsecret, server, nnum);
 		iax_call(session[0], cidnumber, cidname, ich, NULL, 0, codecreq, CODEC_SUPPORT);
 		modemstate = MODEM_CALLING;
 	    }
@@ -619,6 +659,7 @@ iaxmodem(const char *config, int nolog)
     signal(SIGINT, sighandler);
     signal(SIGTERM, sighandler);
     signal(SIGHUP, sighandler);
+    signal(SIGALRM, sighandler_alarm);
 
     int iaxnetfd;
     struct iax_event *iaxevent = 0;
@@ -628,7 +669,7 @@ iaxmodem(const char *config, int nolog)
     gettimeofday(&lastdtedata, NULL);
     lastdtedata.tv_sec -= 60;
     gettimeofday(&lastreg, NULL);
-    lastreg.tv_sec -= refreshreq;
+    lastreg.tv_sec -= (refreshreq > refresh ? (refreshreq+1) : (refresh+1));
     gettimeofday(&lastring, NULL);
     lastring.tv_sec -= 60;
     gettimeofday(&lasthangup, NULL);
@@ -714,8 +755,11 @@ iaxmodem(const char *config, int nolog)
     stty = strdup(stty);
 
 #else /* USE_UNIX98_PTY */
-
+#ifdef SOLARIS
+    amaster = open("/dev/ptmx", O_RDWR);
+#else
     amaster = posix_openpt(O_RDWR | O_NOCTTY);
+#endif
     if (amaster < 0) {
       printlog(LOG_ERROR, "Fatal error: failed to initialize UNIX98 master pty\n");
       cleanup(-1);
@@ -743,7 +787,10 @@ iaxmodem(const char *config, int nolog)
       printlog(LOG_ERROR, "Fatal error: failed to open slave pty %s\n", stty);
       cleanup(-1);
     }
-
+#ifdef SOLARIS
+    ioctl(aslave, I_PUSH, "ptem");	/* push ptem */
+    ioctl(aslave, I_PUSH, "ldterm");	/* push ldterm*/
+#endif
 #endif /* !USE_UNIX98_PTY */
 
     printlog(LOG_INFO, "Opened pty, slave device: %s\n", stty);
@@ -933,14 +980,16 @@ iaxmodem(const char *config, int nolog)
 	    memset(dspbuf, (int16_t) 0, sizeof(int16_t)*VOIP_PACKET_SIZE);
 	    t31buflen = 0;
 	    int gotlen = 0;
-	    do {
-		/*
-		 * If t31_tx returns less than VOIP_PACKET_SIZE but not zero then it means we're switching
-		 * carriers.  If it returns zero, then it means we're silent.
-		 */
-		gotlen = t31_tx(&t31_state, (int16_t *) (dspbuf + sizeof(int16_t)*t31buflen), VOIP_PACKET_SIZE - t31buflen);
-		t31buflen += gotlen;
-	    } while (t31buflen < VOIP_PACKET_SIZE && gotlen > 0 && modemstate == MODEM_CONNECTED);
+	    if (!dialextra) {
+		do {
+		    /*
+		     * If t31_tx returns less than VOIP_PACKET_SIZE but not zero then it means we're switching
+		     * carriers.  If it returns zero, then it means we're silent.
+		     */
+		    gotlen = t31_tx(&t31_state, (int16_t *) (dspbuf + sizeof(int16_t)*t31buflen), VOIP_PACKET_SIZE - t31buflen);
+		    t31buflen += gotlen;
+		} while (t31buflen < VOIP_PACKET_SIZE && gotlen > 0 && modemstate == MODEM_CONNECTED);
+	    }
 	    if (record) write(dspaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
 	    if (replay) read(dspaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
 	    if (modemstate == MODEM_CONNECTED) {	/* t31_tx can change modemstate */
@@ -1042,10 +1091,11 @@ iaxmodem(const char *config, int nolog)
 			break;
 		    case IAX_EVENT_REGREJ:
 			printlog(LOG_ERROR, "Registration failed.\n");
-			regstate = UNREGISTERED;
+			/* To prevent fast looping with registration-attempts, we leave regstate PENDING. */
+			//regstate = UNREGISTERED;
 			break;
 		    case IAX_EVENT_TIMEOUT:
-			if (regstate == UNREGISTERED) {
+			if (regstate == PENDING) {
 			    printlog(LOG_ERROR, "Registration timed out.\n");
 			}
 			break;
@@ -1093,6 +1143,8 @@ iaxmodem(const char *config, int nolog)
 			    dspaudiofd = open(dspaudiofile, O_RDONLY);
 			    iaxaudiofd = open(iaxaudiofile, O_RDONLY);
 			}
+			commalen = t31_state.at_state.p.s_regs[8];	// initialize per modem's S-register
+			sighandler_alarm(0);				// process dialextra
 			break;
 		    case IAX_EVENT_CONNECT:
 			/* incoming call detected */
@@ -1166,6 +1218,7 @@ iaxmodem(const char *config, int nolog)
 			    modemstate = MODEM_ONHOOK;
 			}
 			phonestate = PHONE_FREED;
+			gettimeofday(&lasthangup, NULL);
 			if (record) {
 			    if (dspaudiofd > 0) {
 				close(dspaudiofd);
@@ -1186,7 +1239,7 @@ iaxmodem(const char *config, int nolog)
 			/* pseudo-silence */
 			memset(dspbuf, (int16_t) 0, VOIP_PACKET_SIZE);
 			if (replay) read(iaxaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
-			if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, (int16_t *) dspbuf, VOIP_PACKET_SIZE)) {
+			if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, (int16_t *) dspbuf, VOIP_PACKET_SIZE)) {
 			    printlog(LOG_ERROR, "Error sending silence to DSP.\n");
 			}
 			if (record) write(iaxaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
@@ -1218,7 +1271,7 @@ iaxmodem(const char *config, int nolog)
 				 * syncrhonous with the remote as possible.
 				 */
 				if (replay) read(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
-				if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, fillbuf, VOIP_PACKET_SIZE)) {
+				if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, fillbuf, VOIP_PACKET_SIZE)) {
 				    printlog(LOG_ERROR, "Error sending %d units of IAX make-up audio to DSP.\n", VOIP_PACKET_SIZE);
 				}
 				if (record) write(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
@@ -1267,7 +1320,7 @@ iaxmodem(const char *config, int nolog)
 			    }
 			    memcpy(iaxbuf, audiodata, units >= VOIP_PACKET_SIZE ? VOIP_PACKET_SIZE*sizeof(int16_t) : units*sizeof(int16_t));
 			    if (replay) read(iaxaudiofd, audiodata, units*sizeof(int16_t));
-			    if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, audiodata, units)) {
+			    if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, audiodata, units)) {
 				printlog(LOG_ERROR, "Error sending %d units of IAX audio to DSP.\n", units);
 			    }
 			    if (record) write(iaxaudiofd, audiodata, units*sizeof(int16_t));
@@ -1309,13 +1362,16 @@ iaxmodem(const char *config, int nolog)
 	 * Maintain registration.  Registrations use a different
 	 * session than actual calls.
 	 */
-	if (refreshreq && (now.tv_sec > lastreg.tv_sec + refresh - 5 || (now.tv_sec == lastreg.tv_sec + refresh - 5 && now.tv_usec >= lastreg.tv_usec))) {
-	    /* refresh our registration */
+	if (refreshreq) {
 	    if (now.tv_sec > lastreg.tv_sec + refresh || (now.tv_sec == lastreg.tv_sec + refresh && now.tv_usec > lastreg.tv_usec))
 		regstate = UNREGISTERED;
-	    session[1] = iax_session_new();
-	    iax_register(session[1], server, regpeer, regsecret, refreshreq, NULL);
-	    lastreg = now;
+	    if (regstate == UNREGISTERED || now.tv_sec > lastreg.tv_sec + refresh - 5 || (now.tv_sec == lastreg.tv_sec + refresh - 5 && now.tv_usec >= lastreg.tv_usec)) {
+		/* refresh our registration */
+		session[1] = iax_session_new();
+		iax_register(session[1], server, regpeer, regsecret, refreshreq, NULL);
+		lastreg = now;
+		regstate = PENDING;
+	    }
 	}
     }
 } 
@@ -1557,6 +1613,7 @@ main(int argc, char** argv)
 {
     int ret;
     char config[256];
+    int isdaemon = 1;
     int fd;
     FILE *pidfile;
 
@@ -1572,8 +1629,17 @@ main(int argc, char** argv)
 
     /* If a config is specified, behave in non-daemon mode */
     if (argc == 2) {
-      snprintf(config, sizeof(config), "%s", argv[1]);
-      iaxmodem(config, 1);
+
+	snprintf(config, sizeof(config), "%s", argv[1]);
+
+	if (!strncmp(config, "-F", 2)) {
+
+		isdaemon = 0;
+
+	} else {
+      
+		iaxmodem(config, 1);
+	}
     }
 
     /* 
@@ -1582,7 +1648,7 @@ main(int argc, char** argv)
      */
 
     /* Detach from the console */
-    if (daemon(0, 1) != 0) {
+    if ((isdaemon) && (daemon(0, 1) != 0)) {
       printlog(LOG_ERROR, "Fatal error: daemon() failed: %s\n", strerror(errno));
       exit(-1);
     }
@@ -1593,7 +1659,7 @@ main(int argc, char** argv)
       printlog(LOG_ERROR, "Fatal error: could not open pidfile /var/run/iaxmodem.pid: %s\n", strerror(errno));
       exit(-1);
     }
-    fprintf(pidfile, "%d\n", getpid());
+    fprintf(pidfile, "%d\n", (int) getpid());
     fclose(pidfile);
 
     gothup = 1;
