@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: adsi.c,v 1.39 2007/03/26 12:57:41 steveu Exp $
+ * $Id: adsi.c,v 1.49 2007/07/29 17:56:41 steveu Exp $
  */
 
 /*! \file */
@@ -46,10 +46,11 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
+#include "spandsp/queue.h"
 #include "spandsp/dds.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/async.h"
-#include "spandsp/hdlc.h"
+#include "spandsp/crc.h"
 #include "spandsp/fsk.h"
 #include "spandsp/tone_detect.h"
 #include "spandsp/tone_generate.h"
@@ -77,13 +78,13 @@ static int adsi_tx_get_bit(void *user_data)
     s = (adsi_tx_state_t *) user_data;
     /* This is similar to the async. handling code in fsk.c, but a few special
        things are needed in the preamble, and postamble of an ADSI message. */
-    if (s->bitno < 300)
+    if (s->bit_no < s->preamble_len)
     {
         /* Alternating bit preamble */
-        bit = s->bitno & 1;
-        s->bitno++;
+        bit = s->bit_no & 1;
+        s->bit_no++;
     }
-    else if (s->bitno < 300 + s->ones_len)
+    else if (s->bit_no < s->preamble_len + s->preamble_ones_len)
     {
         /* All 1s for receiver conditioning */
         /* NB: The receiver is an async one. It needs a rest after the
@@ -91,37 +92,43 @@ static int adsi_tx_get_bit(void *user_data)
                the next start bit, and sync to the byte stream. */
         /* The length of this period varies with the circumstance */
         bit = 1;
-        s->bitno++;
+        s->bit_no++;
     }
-    else if (s->bitno < 300 + s->ones_len + 1)
+    else if (s->bit_no <= s->preamble_len + s->preamble_ones_len)
     {
-        if (s->bitpos == 0)
+        /* Push out the 8 bit async. chars, with an appropriate number of stop bits */
+        if (s->bit_pos == 0)
         {
             /* Start bit */
             bit = 0;
-            s->bitpos++;
+            s->bit_pos++;
         }
-        else if (s->bitpos < 9)
+        else if (s->bit_pos < 1 + 8)
         {
-            bit = (s->msg[s->byteno] >> (s->bitpos - 1)) & 1;
-            s->bitpos++;
+            bit = (s->msg[s->byte_no] >> (s->bit_pos - 1)) & 1;
+            s->bit_pos++;
+        }
+        else if (s->bit_pos < 1 + 8 + s->stop_bits - 1)
+        {
+            /* Stop bit */
+            bit = 1;
+            s->bit_pos++;
         }
         else
         {
             /* Stop bit */
             bit = 1;
-            s->bitpos = 0;
-            s->byteno++;
-            if (s->byteno > s->msg_len)
-                s->bitno++;
+            s->bit_pos = 0;
+            if (++s->byte_no >= s->msg_len)
+                s->bit_no++;
         }
     }
-    else if (s->bitno < 300 + s->ones_len + 5)
+    else if (s->bit_no <= s->preamble_len + s->preamble_ones_len + s->postamble_ones_len)
     {
         /* Extra stop bits beyond the last character, to meet the specs., and ensure
            all bits are out of the DSP before we shut off the FSK modem. */
         bit = 1;
-        s->bitno++;
+        s->bit_no++;
     }
     else
     {
@@ -133,6 +140,7 @@ static int adsi_tx_get_bit(void *user_data)
             s->msg_len = 0;
         }
     }
+//printf("Tx bit %d\n", bit);
     return bit;
 }
 /*- End of function --------------------------------------------------------*/
@@ -142,8 +150,8 @@ static int adsi_tdd_get_async_byte(void *user_data)
     adsi_tx_state_t *s;
     
     s = (adsi_tx_state_t *) user_data;
-    if (s->byteno < s->msg_len)
-        return s->msg[s->byteno++];
+    if (s->byte_no < s->msg_len)
+        return s->msg[s->byte_no++];
     if (s->tx_signal_on)
     {
         /* The FSK should now be switched off. */
@@ -169,7 +177,7 @@ static void adsi_rx_put_bit(void *user_data, int bit)
         case PUTBIT_CARRIER_UP:
             span_log(&s->logging, SPAN_LOG_FLOW, "Carrier up.\n");
             s->consecutive_ones = 0;
-            s->bitpos = 0;
+            s->bit_pos = 0;
             s->in_progress = 0;
             s->msg_len = 0;
             s->baudot_shift = 0;
@@ -184,12 +192,12 @@ static void adsi_rx_put_bit(void *user_data, int bit)
         return;
     }
     bit &= 1;
-    if (s->bitpos == 0)
+    if (s->bit_pos == 0)
     {
         if (bit == 0)
         {
             /* Start bit */
-            s->bitpos++;
+            s->bit_pos++;
             if (s->consecutive_ones > 10)
             {
                 /* This is a line idle condition, which means we should
@@ -203,12 +211,12 @@ static void adsi_rx_put_bit(void *user_data, int bit)
             s->consecutive_ones++;
         }
     }
-    else if (s->bitpos <= 8)
+    else if (s->bit_pos <= 8)
     {
         s->in_progress >>= 1;
         if (bit)
             s->in_progress |= 0x80;
-        s->bitpos++;
+        s->bit_pos++;
     }
     else
     {
@@ -272,7 +280,7 @@ static void adsi_rx_put_bit(void *user_data, int bit)
         {
             s->framing_errors++;
         }
-        s->bitpos = 0;
+        s->bit_pos = 0;
         s->in_progress = 0;
     }
 }
@@ -292,7 +300,7 @@ static void adsi_tdd_put_async_byte(void *user_data, int byte)
         case PUTBIT_CARRIER_UP:
             span_log(&s->logging, SPAN_LOG_FLOW, "Carrier up.\n");
             s->consecutive_ones = 0;
-            s->bitpos = 0;
+            s->bit_pos = 0;
             s->in_progress = 0;
             s->msg_len = 0;
             s->baudot_shift = 0;
@@ -333,17 +341,18 @@ static void adsi_rx_dtmf(void *user_data, const char *digits, int len)
            tolerant for a detector running continuously when on hook. */
         s->in_progress = 80000;
     }
+    /* It seems all the DTMF variants are a string of digits and letters,
+       terminated by a "#", or a "C". It appears these are unambiguous, and
+       non-conflicting. */
     for (  ;  len  &&  s->msg_len < 256;  len--)
     {
-        if (*digits == '#')
+        s->msg[s->msg_len++] = *digits;
+        if (*digits == '#'  ||  *digits == 'C')
         {
             s->put_msg(s->user_data, s->msg, s->msg_len);
             s->msg_len = 0;
         }
-        else
-        {
-            s->msg[s->msg_len++] = *digits++;
-        }
+        digits++;
     }
 }
 /*- End of function --------------------------------------------------------*/
@@ -403,18 +412,18 @@ void adsi_rx_init(adsi_rx_state_t *s,
     switch (standard)
     {
     case ADSI_STANDARD_CLASS:
-        fsk_rx_init(&(s->fskrx), &preset_fsk_specs[FSK_BELL202], FALSE, 0, adsi_rx_put_bit, s);
+        fsk_rx_init(&(s->fskrx), &preset_fsk_specs[FSK_BELL202], FALSE, adsi_rx_put_bit, s);
         break;
     case ADSI_STANDARD_CLIP:
     case ADSI_STANDARD_ACLIP:
     case ADSI_STANDARD_JCLIP:
-        fsk_rx_init(&(s->fskrx), &preset_fsk_specs[FSK_V23CH1], FALSE, 0, adsi_rx_put_bit, s);
+        fsk_rx_init(&(s->fskrx), &preset_fsk_specs[FSK_V23CH1], FALSE, adsi_rx_put_bit, s);
         break;
     case ADSI_STANDARD_CLIP_DTMF:
         dtmf_rx_init(&(s->dtmfrx), adsi_rx_dtmf, s);
         break;
     case ADSI_STANDARD_TDD:
-        fsk_rx_init(&(s->fskrx), &preset_fsk_specs[FSK_WEITBRECHT], FALSE, 0, async_rx_put_bit, &(s->asyncrx));
+        fsk_rx_init(&(s->fskrx), &preset_fsk_specs[FSK_WEITBRECHT], FALSE, async_rx_put_bit, &(s->asyncrx));
         async_rx_init(&(s->asyncrx), 5, ASYNC_PARITY_NONE, 2, TRUE, adsi_tdd_put_async_byte, s);
         break;
     }
@@ -457,6 +466,59 @@ void adsi_send_alert_tone(adsi_tx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+void adsi_tx_set_preamble(adsi_tx_state_t *s,
+                          int preamble_len,
+                          int preamble_ones_len,
+                          int postamble_ones_len,
+                          int stop_bits)
+{
+    if (preamble_len < 0)
+    {
+        if (s->standard == ADSI_STANDARD_JCLIP)
+            s->preamble_len = 0;
+        else
+            s->preamble_len = 300;
+    }
+    else
+    {
+        s->preamble_len = preamble_len;
+    }
+    if (preamble_ones_len < 0)
+    {
+        if (s->standard == ADSI_STANDARD_JCLIP)
+            s->preamble_ones_len = 75;
+        else
+            s->preamble_ones_len = 80;
+    }
+    else
+    {
+        s->preamble_ones_len = preamble_ones_len;
+    }
+    if (postamble_ones_len < 0)
+    {
+        if (s->standard == ADSI_STANDARD_JCLIP)
+            s->postamble_ones_len = 5;
+        else
+            s->postamble_ones_len = 5;
+    }
+    else
+    {
+        s->postamble_ones_len = postamble_ones_len;
+    }
+    if (stop_bits < 0)
+    {
+        if (s->standard == ADSI_STANDARD_JCLIP)
+            s->stop_bits = 4;
+        else
+            s->stop_bits = 1;
+    }
+    else
+    {
+        s->stop_bits = stop_bits;
+    }
+}
+/*- End of function --------------------------------------------------------*/
+
 int adsi_put_message(adsi_tx_state_t *s, uint8_t *msg, int len)
 {
     int i;
@@ -518,8 +580,6 @@ int adsi_put_message(adsi_tx_state_t *s, uint8_t *msg, int len)
         s->msg[i++] = (uint8_t) (crc_value & 0xFF);
         s->msg[i++] = (uint8_t) ((crc_value >> 8) & 0xFF);
         s->msg_len = i;
-
-        s->ones_len = 80;
         break;
     case ADSI_STANDARD_TDD:
         if (len > 255)
@@ -539,13 +599,12 @@ int adsi_put_message(adsi_tx_state_t *s, uint8_t *msg, int len)
             sum += s->msg[ii];
         s->msg[len] = (uint8_t) ((-sum) & 0xFF);
         s->msg_len = len + 1;
-        s->ones_len = 80;
         break;
     }    
     /* Prepare the bit sequencing */
-    s->byteno = 0;
-    s->bitpos = 0;
-    s->bitno = 0;
+    s->byte_no = 0;
+    s->bit_pos = 0;
+    s->bit_no = 0;
     return len;
 }
 /*- End of function --------------------------------------------------------*/
@@ -564,6 +623,7 @@ void adsi_tx_init(adsi_tx_state_t *s, int standard)
                              0,
                              FALSE);
     s->standard = standard;
+    adsi_tx_set_preamble(s, -1, -1, -1, -1);
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
     start_tx(s);
 }
@@ -826,21 +886,33 @@ int adsi_next_field(adsi_rx_state_t *s, const uint8_t *msg, int msg_len, int pos
             return -2;
         break;
     case ADSI_STANDARD_CLIP_DTMF:
-        if (pos >= msg_len)
-            return -1;
-        if (pos < 0)
-            pos = 0;
-        *field_type = msg[pos++];
-        *field_body = msg + pos;
-        i = pos;
-        while (i < msg_len  &&  msg[i] != '#')
-            i++;
-        *field_len = i - pos;
-        pos = i;
-        if (msg[pos] == '#')
-            pos++;
         if (pos > msg_len)
-            return -2;
+            return -1;
+        if (pos <= 0)
+        {
+            pos = 1;
+            *field_type = msg[msg_len - 1];
+            *field_len = 0;
+            *field_body = NULL;
+        }
+        else
+        {
+            /* Remove bias on the pos value */
+            pos--;
+            *field_type = msg[pos++];
+            *field_body = msg + pos;
+            i = pos;
+            while (i < msg_len  &&  msg[i] >= '0'  &&  msg[i] <= '9')
+                i++;
+            *field_len = i - pos;
+            pos = i;
+            if (msg[pos] == '#'  ||  msg[pos] == 'C')
+                pos++;
+            if (pos > msg_len)
+                return -2;
+            /* Bias the pos value, so we don't return 0 inappropriately */
+            pos++;
+        }
         break;
     case ADSI_STANDARD_TDD:
         if (pos >= msg_len)
@@ -921,15 +993,20 @@ int adsi_add_field(adsi_tx_state_t *s, uint8_t *msg, int len, uint8_t field_type
         }
         break;
     case ADSI_STANDARD_CLIP_DTMF:
-        if (len < 0)
+        if (len <= 0)
         {
-            len = 0;
+            /* Initialise a new message. The field type is actually the message type. */
+            msg[0] = field_type;
+            len = 1;
         }
         else
         {
+            /* Save and reuse the terminator/message type */
+            len--;
+            x = msg[len];
             msg[len] = field_type;
             memcpy(msg + len + 1, field_body, field_len);
-            msg[len + field_len + 1] = '#';
+            msg[len + field_len + 1] = x;
             len += (field_len + 2);
         }
         break;
