@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dtmf.c,v 1.10 2006/10/24 13:45:25 steveu Exp $
+ * $Id: dtmf.c,v 1.21 2007/06/26 15:54:12 steveu Exp $
  */
  
 /*! \file dtmf.h */
@@ -33,15 +33,22 @@
 
 #include <inttypes.h>
 #include <stdlib.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <string.h>
 #include <stdio.h>
 #include <time.h>
 #include <fcntl.h>
 
 #include "spandsp/telephony.h"
+#include "spandsp/queue.h"
 #include "spandsp/tone_detect.h"
 #include "spandsp/tone_generate.h"
+#include "spandsp/super_tone_rx.h"
 #include "spandsp/dtmf.h"
 
 #if !defined(M_PI)
@@ -51,18 +58,27 @@
 
 //#define USE_3DNOW
 
-#define ms_to_samples(t)            (((t)*SAMPLE_RATE)/1000)
-
-#define DTMF_DURATION               ms_to_samples(70)
-#define DTMF_PAUSE                  ms_to_samples(80)
-#define DTMF_CYCLE                  (DTMF_DURATION + DTMF_PAUSE)
+#define DEFAULT_DTMF_TX_LEVEL       -10
+#define DEFAULT_DTMF_TX_ON_TIME     50
+#define DEFAULT_DTMF_TX_OFF_TIME    55
 
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+#define DTMF_THRESHOLD              1.0e5f
+#define DTMF_NORMAL_TWIST           6.3f    /* 8dB */
+#define DTMF_REVERSE_TWIST          2.5f    /* 4dB */
+#define DTMF_RELATIVE_PEAK_ROW      6.3f    /* 8dB */
+#define DTMF_RELATIVE_PEAK_COL      6.3f    /* 8dB */
+#define DTMF_TO_TOTAL_ENERGY        42.0f
+#define DTMF_POWER_OFFSET           (90.30f - 48.32f)
+#else
 #define DTMF_THRESHOLD              8.0e7f
 #define DTMF_NORMAL_TWIST           6.3f    /* 8dB */
 #define DTMF_REVERSE_TWIST          2.5f    /* 4dB */
 #define DTMF_RELATIVE_PEAK_ROW      6.3f    /* 8dB */
 #define DTMF_RELATIVE_PEAK_COL      6.3f    /* 8dB */
 #define DTMF_TO_TOTAL_ENERGY        42.0f
+#define DTMF_POWER_OFFSET           90.30f
+#endif
 
 static const float dtmf_row[] =
 {
@@ -163,15 +179,20 @@ static __inline__ void _dtmf_goertzel_update(goertzel_state_t *s,
     s[2].v3 = vv[10];
     s[3].v3 = vv[11];
 }
-#endif
 /*- End of function --------------------------------------------------------*/
+#endif
 
 int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
 {
     float row_energy[4];
     float col_energy[4];
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+    int famp;
+    int v1;
+#else
     float famp;
     float v1;
+#endif
     int i;
     int j;
     int sample;
@@ -212,33 +233,70 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
                 s->z440_2 = s->z440_1;
                 s->z440_1 = v1;
             }
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+            famp >>= 8;
+            s->energy += famp*famp;
+            /* With GCC 2.95, the following unrolled code seems to take about 35%
+               (rough estimate) as long as a neat little 0-3 loop */
+            v1 = s->row_out[0].v2;
+            s->row_out[0].v2 = s->row_out[0].v3;
+            s->row_out[0].v3 = ((s->row_out[0].fac*s->row_out[0].v2) >> 12) - v1 + famp;
+
+            v1 = s->col_out[0].v2;
+            s->col_out[0].v2 = s->col_out[0].v3;
+            s->col_out[0].v3 = ((s->col_out[0].fac*s->col_out[0].v2) >> 12) - v1 + famp;
+
+            v1 = s->row_out[1].v2;
+            s->row_out[1].v2 = s->row_out[1].v3;
+            s->row_out[1].v3 = ((s->row_out[1].fac*s->row_out[1].v2) >> 12) - v1 + famp;
+
+            v1 = s->col_out[1].v2;
+            s->col_out[1].v2 = s->col_out[1].v3;
+            s->col_out[1].v3 = ((s->col_out[1].fac*s->col_out[1].v2) >> 12) - v1 + famp;
+
+            v1 = s->row_out[2].v2;
+            s->row_out[2].v2 = s->row_out[2].v3;
+            s->row_out[2].v3 = ((s->row_out[2].fac*s->row_out[2].v2) >> 12) - v1 + famp;
+
+            v1 = s->col_out[2].v2;
+            s->col_out[2].v2 = s->col_out[2].v3;
+            s->col_out[2].v3 = ((s->col_out[2].fac*s->col_out[2].v2) >> 12) - v1 + famp;
+
+            v1 = s->row_out[3].v2;
+            s->row_out[3].v2 = s->row_out[3].v3;
+            s->row_out[3].v3 = ((s->row_out[3].fac*s->row_out[3].v2) >> 12) - v1 + famp;
+
+            v1 = s->col_out[3].v2;
+            s->col_out[3].v2 = s->col_out[3].v3;
+            s->col_out[3].v3 = ((s->col_out[3].fac*s->col_out[3].v2) >> 12) - v1 + famp;
+#else
             s->energy += famp*famp;
             /* With GCC 2.95, the following unrolled code seems to take about 35%
                (rough estimate) as long as a neat little 0-3 loop */
             v1 = s->row_out[0].v2;
             s->row_out[0].v2 = s->row_out[0].v3;
             s->row_out[0].v3 = s->row_out[0].fac*s->row_out[0].v2 - v1 + famp;
-    
+
             v1 = s->col_out[0].v2;
             s->col_out[0].v2 = s->col_out[0].v3;
             s->col_out[0].v3 = s->col_out[0].fac*s->col_out[0].v2 - v1 + famp;
-    
+
             v1 = s->row_out[1].v2;
             s->row_out[1].v2 = s->row_out[1].v3;
             s->row_out[1].v3 = s->row_out[1].fac*s->row_out[1].v2 - v1 + famp;
-    
+
             v1 = s->col_out[1].v2;
             s->col_out[1].v2 = s->col_out[1].v3;
             s->col_out[1].v3 = s->col_out[1].fac*s->col_out[1].v2 - v1 + famp;
-    
+
             v1 = s->row_out[2].v2;
             s->row_out[2].v2 = s->row_out[2].v3;
             s->row_out[2].v3 = s->row_out[2].fac*s->row_out[2].v2 - v1 + famp;
-    
+
             v1 = s->col_out[2].v2;
             s->col_out[2].v2 = s->col_out[2].v3;
             s->col_out[2].v3 = s->col_out[2].fac*s->col_out[2].v2 - v1 + famp;
-    
+
             v1 = s->row_out[3].v2;
             s->row_out[3].v2 = s->row_out[3].v3;
             s->row_out[3].v3 = s->row_out[3].fac*s->row_out[3].v2 - v1 + famp;
@@ -246,6 +304,7 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
             v1 = s->col_out[3].v2;
             s->col_out[3].v2 = s->col_out[3].v3;
             s->col_out[3].v3 = s->col_out[3].fac*s->col_out[3].v2 - v1 + famp;
+#endif
         }
 #endif
         s->current_sample += (limit - sample);
@@ -293,6 +352,7 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
                 &&
                 (row_energy[best_row] + col_energy[best_col]) > DTMF_TO_TOTAL_ENERGY*s->energy)
             {
+                /* Got a hit */
                 hit = dtmf_positions[(best_row << 2) + best_col];
             }
         }
@@ -332,7 +392,10 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
                 {
                     /* Avoid reporting multiple no digit conditions on flaky hits */
                     if (s->in_digit  ||  hit)
-                        s->realtime_callback(s->realtime_callback_data, hit);
+                    {
+                        i = (s->in_digit  &&  !hit)  ?  -99  :  rintf(log10f(s->energy)*10.0f - 20.08f - DTMF_POWER_OFFSET + DBM0_MAX_POWER);
+                        s->realtime_callback(s->realtime_callback_data, hit, i);
+                    }
                 }
                 else
                 {
@@ -364,7 +427,7 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
             goertzel_reset(&s->row_out[i]);
             goertzel_reset(&s->col_out[i]);
         }
-        s->energy = 0.0;
+        s->energy = 0.0f;
         s->current_sample = 0;
     }
     if (s->current_digits  &&  s->callback)
@@ -377,6 +440,16 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
 }
 /*- End of function --------------------------------------------------------*/
 
+int dtmf_rx_status(dtmf_rx_state_t *s)
+{
+    if (s->in_digit)
+        return s->in_digit;
+    if (s->last_hit)
+        return 'x';
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 size_t dtmf_rx_get(dtmf_rx_state_t *s, char *buf, int max)
 {
     if (max > s->current_digits)
@@ -393,7 +466,7 @@ size_t dtmf_rx_get(dtmf_rx_state_t *s, char *buf, int max)
 /*- End of function --------------------------------------------------------*/
 
 void dtmf_rx_set_realtime_callback(dtmf_rx_state_t *s,
-                                   void (*callback)(void *user_data, int signal),
+                                   tone_report_func_t callback,
                                    void *user_data)
 {
     s->realtime_callback = callback;
@@ -408,21 +481,21 @@ void dtmf_rx_parms(dtmf_rx_state_t *s,
 {
     if (filter_dialtone >= 0)
     {
-        s->z350_1 = 0.0;
-        s->z350_2 = 0.0;
-        s->z440_1 = 0.0;
-        s->z440_2 = 0.0;
+        s->z350_1 = 0.0f;
+        s->z350_2 = 0.0f;
+        s->z440_1 = 0.0f;
+        s->z440_2 = 0.0f;
         s->filter_dialtone = filter_dialtone;
     }
     if (twist >= 0)
-        s->normal_twist = powf(10.0, twist/10.0);
+        s->normal_twist = powf(10.0f, twist/10.0f);
     if (reverse_twist >= 0)
-        s->reverse_twist = powf(10.0, reverse_twist/10.0);
+        s->reverse_twist = powf(10.0f, reverse_twist/10.0f);
 }
 /*- End of function --------------------------------------------------------*/
 
 dtmf_rx_state_t *dtmf_rx_init(dtmf_rx_state_t *s,
-                              void (*callback)(void *user_data, const char *digits, int len),
+                              dtmf_rx_callback_t callback,
                               void *user_data)
 {
     int i;
@@ -453,7 +526,7 @@ dtmf_rx_state_t *dtmf_rx_init(dtmf_rx_state_t *s,
         goertzel_init(&s->row_out[i], &dtmf_detect_row[i]);
         goertzel_init(&s->col_out[i], &dtmf_detect_col[i]);
     }
-    s->energy = 0.0;
+    s->energy = 0.0f;
     s->current_sample = 0;
     s->lost_digits = 0;
     s->current_digits = 0;
@@ -471,15 +544,15 @@ static void dtmf_tx_initialise(void)
         return;
     for (row = 0;  row < 4;  row++)
     {
-    	for (col = 0;  col < 4;  col++)
+        for (col = 0;  col < 4;  col++)
         {
-    	    make_tone_gen_descriptor(&dtmf_digit_tones[row*4 + col],
+            make_tone_gen_descriptor(&dtmf_digit_tones[row*4 + col],
                                      (int) dtmf_row[row],
-                                     -10,
+                                     DEFAULT_DTMF_TX_LEVEL,
                                      (int) dtmf_col[col],
-                                     -10,
-                                     50,
-                                     55,
+                                     DEFAULT_DTMF_TX_LEVEL,
+                                     DEFAULT_DTMF_TX_ON_TIME,
+                                     DEFAULT_DTMF_TX_OFF_TIME,
                                      0,
                                      0,
                                      FALSE);
@@ -492,8 +565,8 @@ static void dtmf_tx_initialise(void)
 int dtmf_tx(dtmf_tx_state_t *s, int16_t amp[], int max_samples)
 {
     int len;
-    size_t dig;
-    char *cp;
+    const char *cp;
+    int digit;
 
     len = 0;
     if (s->tones.current_section >= 0)
@@ -501,21 +574,14 @@ int dtmf_tx(dtmf_tx_state_t *s, int16_t amp[], int max_samples)
         /* Deal with the fragment left over from last time */
         len = tone_gen(&(s->tones), amp, max_samples);
     }
-    dig = 0;
-    while (dig < s->current_digits  &&  len < max_samples)
+    while (len < max_samples  &&  (digit = queue_read_byte(&s->queue)) >= 0)
     {
         /* Step to the next digit */
-        if ((cp = strchr(dtmf_positions, s->digits[dig++])) == NULL)
+        if ((cp = strchr(dtmf_positions, digit)) == NULL)
             continue;
         tone_gen_init(&(s->tones), &(s->tone_descriptors[cp - dtmf_positions]));
         len += tone_gen(&(s->tones), amp + len, max_samples - len);
     }
-    if (dig)
-    {
-        /* Shift out the consumed digits */
-        s->current_digits -= dig;
-        memmove(s->digits, s->digits + dig, s->current_digits);
-    }
     return len;
 }
 /*- End of function --------------------------------------------------------*/
@@ -523,24 +589,18 @@ int dtmf_tx(dtmf_tx_state_t *s, int16_t amp[], int max_samples)
 size_t dtmf_tx_put(dtmf_tx_state_t *s, const char *digits)
 {
     size_t len;
+    size_t space;
 
     /* This returns the number of characters that would not fit in the buffer.
        The buffer will only be loaded if the whole string of digits will fit,
        in which case zero is returned. */
-    if ((len = strlen(digits)) > 0)
-    {
-        if (s->current_digits + len <= MAX_DTMF_DIGITS)
-        {
-            memcpy(s->digits + s->current_digits, digits, len);
-            s->current_digits += len;
-            len = 0;
-        }
-        else
-        {
-            len = MAX_DTMF_DIGITS - s->current_digits;
-        }
-    }
-    return len;
+    if ((len = strlen(digits)) == 0)
+        return 0;
+    if ((space = queue_free_space(&s->queue)) < len)
+        return len - space;
+    if (queue_write(&s->queue, (const uint8_t *) digits, len) >= 0)
+        return 0;
+    return -1;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -551,7 +611,7 @@ dtmf_tx_state_t *dtmf_tx_init(dtmf_tx_state_t *s)
     s->tone_descriptors = dtmf_digit_tones;
     tone_gen_init(&(s->tones), &dtmf_digit_tones[0]);
     s->current_sample = 0;
-    s->current_digits = 0;
+    queue_init(&s->queue, MAX_DTMF_DIGITS, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC);
     s->tones.current_section = -1;
     return s;
 }
