@@ -26,7 +26,7 @@
  * implementation of the LPC-10 2400 bps Voice Coder. They do not
  * exert copyright claims on their code, and it may be freely used.
  *
- * $Id: lpc10_voicing.c,v 1.4 2006/10/24 13:21:59 steveu Exp $
+ * $Id: lpc10_voicing.c,v 1.7 2006/11/30 15:41:47 steveu Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,7 +37,12 @@
 #include <stdio.h>
 #include <inttypes.h>
 #include <memory.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/dc_restore.h"
@@ -287,15 +292,27 @@ void lpc10_voicing(lpc10_encode_state_t *s,
     /* Update linear discriminant function history each frame: */
     if (half == 0)
     {
-        s->voice[0] = s->voice[2];
-        s->voice[1] = s->voice[3];
-        s->voice[2] = s->voice[4];
-        s->voice[3] = s->voice[5];
+        s->voice[0][0] = s->voice[1][0];
+        s->voice[0][1] = s->voice[1][1];
+        s->voice[1][0] = s->voice[2][0];
+        s->voice[1][1] = s->voice[2][1];
         s->maxmin = *maxamd / max(*minamd, 1.0f);
     }
-    /* Calculate voicing parameters twice per frame: */
-    vparms(vwin, &inbuf[inbuf_offset], &lpbuf[lpbuf_offset], buflim, 
-           half, &s->dither, mintau, &zc, &lbe, &fbe, &qs, &rc1, &ar_b, &ar_f);
+    /* Calculate voicing parameters twice per frame */
+    vparms(vwin,
+           &inbuf[inbuf_offset],
+           &lpbuf[lpbuf_offset],
+           buflim, 
+           half,
+           &s->dither,
+           mintau,
+           &zc,
+           &lbe,
+           &fbe,
+           &qs,
+           &rc1,
+           &ar_b,
+           &ar_f);
     /* Estimate signal-to-noise ratio to select the appropriate VDC vector. */
     /* The SNR is estimated as the running average of the ratio of the */
     /* running average full-band voiced energy to the running average */
@@ -321,12 +338,12 @@ void lpc10_voicing(lpc10_encode_state_t *s,
     value[6] = ar_b;
     value[7] = ar_f;
     /* Evaluation of linear discriminant function: */
-    s->voice[4 + half] = vdc[snrl*10 + 9];
+    s->voice[2][half] = vdc[snrl*10 + 9];
     for (i = 0;  i < 8;  i++)
-        s->voice[4 + half] += vdc[snrl*10 + i]*value[i];
+        s->voice[2][half] += vdc[snrl*10 + i]*value[i];
     /* Classify as voiced if discriminant > 0, otherwise unvoiced */
     /* Voicing decision for current half-frame:  1 = Voiced; 0 = Unvoiced */
-    s->voibuf[6 + half] = (s->voice[4 + half] > 0.0f)  ?  1  :  0;
+    s->voibuf[3][half] = (s->voice[2][half] > 0.0f)  ?  1  :  0;
     /* Skip voicing decision smoothing in first half-frame: */
     /* Give a value to VSTATE, so that trace statements below will print */
     /* a consistent value from one call to the next when HALF .EQ. 1. */
@@ -376,60 +393,60 @@ void lpc10_voicing(lpc10_encode_state_t *s,
         /* P and 1F but not after 1F. */
         ot = ((obound[0] & 2) != 0  ||  obound[1] == 1)  &&  (obound[2] & 1) == 0;
         /* Multi-way dispatch on voicing decision history: */
-        vstate = (s->voibuf[2] << 3) + (s->voibuf[3] << 2) + (s->voibuf[4] << 1) + s->voibuf[5];
+        vstate = (s->voibuf[1][0] << 3) + (s->voibuf[1][1] << 2) + (s->voibuf[2][0] << 1) + s->voibuf[2][1];
         switch (vstate + 1)
         {
         case 2:
-            if (ot  &&  s->voibuf[6] == 1)
-                s->voibuf[4] = 1;
+            if (ot  &&  s->voibuf[3][0] == 1)
+                s->voibuf[2][0] = 1;
             break;
         case 3:
-            if (s->voibuf[6] == 0  ||  s->voice[2] < -s->voice[3])
-                s->voibuf[4] = 0;
+            if (s->voibuf[3][0] == 0  ||  s->voice[1][0] < -s->voice[1][1])
+                s->voibuf[2][0] = 0;
             else
-                s->voibuf[5] = 1;
+                s->voibuf[2][1] = 1;
             break;
         case 5:
-            s->voibuf[3] = 0;
+            s->voibuf[1][1] = 0;
             break;
         case 6:
-            if (s->voice[1] < -s->voice[2])
-                s->voibuf[3] = 0;
+            if (s->voice[0][1] < -s->voice[1][0])
+                s->voibuf[1][1] = 0;
             else
-                s->voibuf[4] = 1;
+                s->voibuf[2][0] = 1;
             break;
         case 7:
-            if (s->voibuf[0] == 1  ||  s->voibuf[6] == 1  ||  s->voice[3] > s->voice[0])
-                s->voibuf[5] = 1;
+            if (s->voibuf[0][0] == 1  ||  s->voibuf[3][0] == 1  ||  s->voice[1][1] > s->voice[0][0])
+                s->voibuf[2][1] = 1;
             else
-                s->voibuf[2] = 1;
+                s->voibuf[1][0] = 1;
             break;
         case 8:
             if (ot)
-                s->voibuf[3] = 0;
+                s->voibuf[1][1] = 0;
             break;
         case 9:
             if (ot)
-                s->voibuf[3] = 1;
+                s->voibuf[1][1] = 1;
             break;
         case 11:
-            if (s->voice[2] < -s->voice[1])
-                s->voibuf[4] = 0;
+            if (s->voice[1][9] < -s->voice[0][1])
+                s->voibuf[2][0] = 0;
             else
-                s->voibuf[3] = 1;
+                s->voibuf[1][1] = 1;
             break;
         case 12:
-            s->voibuf[3] = 1;
+            s->voibuf[1][1] = 1;
             break;
         case 14:
-            if (s->voibuf[6] == 0  &&  s->voice[3] < -s->voice[2])
-                s->voibuf[5] = 0;
+            if (s->voibuf[3][0] == 0  &&  s->voice[1][1] < -s->voice[1][0])
+                s->voibuf[2][1] = 0;
             else
-                s->voibuf[4] = 1;
+                s->voibuf[2][0] = 1;
             break;
         case 15:
-            if (ot  &&  s->voibuf[6] == 0)
-                s->voibuf[4] = 0;
+            if (ot  &&  s->voibuf[3][0] == 0)
+                s->voibuf[2][0] = 0;
             break;
         }
     }
@@ -440,7 +457,7 @@ void lpc10_voicing(lpc10_encode_state_t *s,
     /* filters.) */
     /* During voiced half-frames, update the low-pass (LBVE) and all-pass */
     /* (FBVE) voiced energy estimates. */
-    if (s->voibuf[6 + half] == 0)
+    if (s->voibuf[3][half] == 0)
     {
         r1 = (s->sfbue*63 + (min(fbe, s->ofbue*3) << 3))/64.0f;
         s->sfbue = lrintf(r1);
@@ -461,7 +478,7 @@ void lpc10_voicing(lpc10_encode_state_t *s,
     /* NOTE: The divisor is a function of REF, the expected energies. */
     /* Computing MIN */
     /* Computing MAX */
-    r2 = sqrt((float) (s->lbue*s->lbve))*64/3000;
+    r2 = sqrtf((float) (s->lbue*s->lbve))*64/3000;
     r1 = max(r2, 1.0f);
     s->dither = min(r1, 20.0f);
     /* Voicing decisions are returned in VOIBUF. */
