@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: hdlc.h,v 1.30 2007/04/08 08:16:18 steveu Exp $
+ * $Id: hdlc.h,v 1.35 2007/07/29 17:56:41 steveu Exp $
  */
 
 /*! \file */
@@ -39,7 +39,6 @@ HDLC may not be a DSP function, but is needed to accompany several DSP component
 \section hdlc_page_sec_2 How does it work?
 */
 
-
 #if !defined(_SPANDSP_HDLC_H_)
 #define _SPANDSP_HDLC_H_
 
@@ -48,7 +47,7 @@ HDLC may not be a DSP function, but is needed to accompany several DSP component
 */
 #define HDLC_MAXFRAME_LEN       400	
 
-typedef void (*hdlc_frame_handler_t)(void *user_data, int ok, const uint8_t *pkt, int len);
+typedef void (*hdlc_frame_handler_t)(void *user_data, const uint8_t *pkt, int len, int ok);
 typedef void (*hdlc_underflow_handler_t)(void *user_data);
 
 /*!
@@ -58,6 +57,8 @@ typedef struct
 {
     /*! 2 for CRC-16, 4 for CRC-32 */
     int crc_bytes;
+    /*! \brief Maximum permitted frame length. */
+    size_t max_frame_len;
     /*! \brief The callback routine called to process each good received frame. */
     hdlc_frame_handler_t frame_handler;
     /*! \brief An opaque parameter passed to the callback routine. */
@@ -78,11 +79,16 @@ typedef struct
     unsigned int byte_in_progress;
     /*! \brief The current number of bits in byte_in_progress. */
     int num_bits;
-	
+    int octet_counting_mode;
+    /*! \brief Octet count, to achieve the functionality needed for things
+               like MTP. */
+    int octet_count;
+    int octet_count_report_interval;
+
     /*! \brief Buffer for a frame in progress. */
     uint8_t buffer[HDLC_MAXFRAME_LEN + 4];
     /*! \brief Length of a frame in progress. */
-    int len;
+    size_t len;
 
     /*! \brief The number of bytes of good frames received (CRC not included). */
     unsigned long int rx_bytes;
@@ -130,7 +136,7 @@ typedef struct
     /*! \brief TRUE if frame creation works in progressive mode. */
     int progressive;
     /*! \brief Maximum permitted frame length. */
-    int max_frame_len;
+    size_t max_frame_len;
 
     /*! \brief The stuffed bit stream being created. */
     uint32_t octets_in_progress;
@@ -140,13 +146,15 @@ typedef struct
     int idle_octet;
     /*! \brief The number of flag octets to send for a timed burst of flags. */
     int flag_octets;
+    /*! \brief The number of abort octets to send for a timed burst of aborts. */
+    int abort_octets;
     /*! \brief TRUE if the next underflow of timed flag octets should be reported */
     int report_flag_underflow;
 
     /*! \brief The current message being transmitted, with its CRC attached. */
     uint8_t buffer[HDLC_MAXFRAME_LEN + 4];
     /*! \brief The length of the message in the buffer. */
-    int len;
+    size_t len;
     /*! \brief The current send position within the buffer. */
     int pos;
     /*! \brief The running CRC, as data fills the frame buffer. */
@@ -166,54 +174,6 @@ extern "C"
 {
 #endif
 
-/*! \brief Calculate the ITU/CCITT CRC-32 value in buffer.
-    \param buf The buffer containing the data.
-    \param len The length of the frame.
-    \param crc The initial CRC value. This is usually 0xFFFFFFFF, or 0 for a new block (it depends on
-           the application). It is previous returned CRC value for the continuation of a block.
-    \return The CRC value.
-*/
-uint32_t crc_itu32_calc(const uint8_t *buf, int len, uint32_t crc);
-
-/*! \brief Append an ITU/CCITT CRC-32 value to a frame.
-    \param buf The buffer containing the frame. This must be at least 2 bytes longer than
-               the frame it contains, to allow room for the CRC value.
-    \param len The length of the frame.
-    \return The new length of the frame.
-*/
-int crc_itu32_append(uint8_t *buf, int len);
-
-/*! \brief Check the ITU/CCITT CRC-32 value in a frame.
-    \param buf The buffer containing the frame.
-    \param len The length of the frame.
-    \return TRUE if the CRC is OK, else FALSE.
-*/
-int crc_itu32_check(const uint8_t *buf, int len);
-
-/*! \brief Calculate the ITU/CCITT CRC-16 value in buffer.
-    \param buf The buffer containing the data.
-    \param len The length of the frame.
-    \param crc The initial CRC value. This is usually 0xFFFF, or 0 for a new block (it depends on
-           the application). It is previous returned CRC value for the continuation of a block.
-    \return The CRC value.
-*/
-uint16_t crc_itu16_calc(const uint8_t *buf, int len, uint16_t crc);
-
-/*! \brief Append an ITU/CCITT CRC-16 value to a frame.
-    \param buf The buffer containing the frame. This must be at least 2 bytes longer than
-               the frame it contains, to allow room for the CRC value.
-    \param len The length of the frame.
-    \return The new length of the frame.
-*/
-int crc_itu16_append(uint8_t *buf, int len);
-
-/*! \brief Check the ITU/CCITT CRC-16 value in a frame.
-    \param buf The buffer containing the frame.
-    \param len The length of the frame.
-    \return TRUE if the CRC is OK, else FALSE.
-*/
-int crc_itu16_check(const uint8_t *buf, int len);
-
 /*! \brief Initialise an HDLC receiver context.
     \param s A pointer to an HDLC receiver context.
     \param crc32 TRUE to use ITU CRC32. FALSE to use ITU CRC16.
@@ -232,6 +192,18 @@ hdlc_rx_state_t *hdlc_rx_init(hdlc_rx_state_t *s,
                               hdlc_frame_handler_t handler,
                               void *user_data);
 
+/*! \brief Set the maximum frame length for an HDLC receiver context.
+    \param s A pointer to an HDLC receiver context.
+    \param max_len The maximum length.
+*/
+void hdlc_rx_set_max_frame_len(hdlc_rx_state_t *s, size_t max_len);
+
+/*! \brief Set the octet counting report interval.
+    \param s A pointer to an HDLC receiver context.
+    \param interval The interval, in octets.
+*/
+void hdlc_rx_set_octet_counting_report_interval(hdlc_rx_state_t *s, int interval);
+
 /*! \brief Get the current receive statistics.
     \param s A pointer to an HDLC receiver context.
     \param t A pointer to the buffer for the statistics.
@@ -240,11 +212,25 @@ hdlc_rx_state_t *hdlc_rx_init(hdlc_rx_state_t *s,
 int hdlc_rx_get_stats(hdlc_rx_state_t *s,
                       hdlc_rx_stats_t *t);
 
-/* Use either the bit-by-bit or byte-by-byte routines. Do not mix them is a
-   single instance of HDLC */
+/*! \brief Put a single bit of data to an HDLC receiver.
+    \param s A pointer to an HDLC receiver context.
+    \param new_bit The bit.
+*/
 void hdlc_rx_put_bit(hdlc_rx_state_t *s, int new_bit);
+
+/*! \brief Put a byte of data to an HDLC receiver.
+    \param s A pointer to an HDLC receiver context.
+    \param new_byte The byte of data.
+*/
 void hdlc_rx_put_byte(hdlc_rx_state_t *s, int new_byte);
 
+/*! \brief Put a series of bytes of data to an HDLC receiver.
+    \param s A pointer to an HDLC receiver context.
+    \param buf The buffer of data.
+    \param len The length of the data in the buffer.
+*/
+void hdlc_rx_put(hdlc_rx_state_t *s, const uint8_t buf[], int len);
+
 /*! \brief Initialise an HDLC transmitter context.
     \param s A pointer to an HDLC transmitter context.
     \param crc32 TRUE to use ITU CRC32. FALSE to use ITU CRC16.
@@ -265,7 +251,7 @@ hdlc_tx_state_t *hdlc_tx_init(hdlc_tx_state_t *s,
     \param s A pointer to an HDLC transmitter context.
     \param max_len The maximum length.
 */
-void hdlc_tx_set_max_frame_len(hdlc_tx_state_t *s, int max_len);
+void hdlc_tx_set_max_frame_len(hdlc_tx_state_t *s, size_t max_len);
 
 /*! \brief Transmit a frame.
     \param s A pointer to an HDLC transmitter context.
@@ -273,7 +259,7 @@ void hdlc_tx_set_max_frame_len(hdlc_tx_state_t *s, int max_len);
     \param len The length of the frame to be transmitted.
     \return 0 if the frame was accepted for transmission, else -1.
 */
-int hdlc_tx_frame(hdlc_tx_state_t *s, const uint8_t *frame, int len);
+int hdlc_tx_frame(hdlc_tx_state_t *s, const uint8_t *frame, size_t len);
 
 /*! \brief Corrupt the frame currently being transmitted, by giving it the wrong CRC.
     \param s A pointer to an HDLC transmitter context.
@@ -281,14 +267,20 @@ int hdlc_tx_frame(hdlc_tx_state_t *s, const uint8_t *frame, int len);
 */
 int hdlc_tx_corrupt_frame(hdlc_tx_state_t *s);
 
-/*! \brief Transmit a specified quantity of flag octets as a preamble.
+/*! \brief Transmit a specified quantity of flag octets, typically as a preamble.
     \param s A pointer to an HDLC transmitter context.
-    \param len The length of the required preamble, in flag octets. If len is zero this
+    \param len The length of the required period of flags, in flag octets. If len is zero this
            requests that HDLC transmission be terminated when the buffers have fully
            drained.
-    \return 0 if the preamble was accepted for transmission, else -1.
+    \return 0 if the flags were accepted for transmission, else -1.
 */
-int hdlc_tx_preamble(hdlc_tx_state_t *s, int len);
+int hdlc_tx_flags(hdlc_tx_state_t *s, int len);
+
+/*! \brief Send an abort.
+    \param s A pointer to an HDLC transmitter context.
+    \return 0 if the frame was aborted, else -1.
+*/
+int hdlc_tx_abort(hdlc_tx_state_t *s);
 
 /*! \brief Get the next bit for transmission.
     \param s A pointer to an HDLC transmitter context.
@@ -302,6 +294,14 @@ int hdlc_tx_get_bit(hdlc_tx_state_t *s);
 */
 int hdlc_tx_get_byte(hdlc_tx_state_t *s);
 
+/*! \brief Get the next sequence of bytes for transmission.
+    \param s A pointer to an HDLC transmitter context.
+    \param buf The buffer for the data.
+    \param len The number of bytes to get.
+    \return The number of bytes actually got.
+*/
+int hdlc_tx_get(hdlc_tx_state_t *s, uint8_t buf[], size_t max_len);
+
 #if defined(__cplusplus)
 }
 #endif
