@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t30.h,v 1.72 2007/05/01 14:34:51 steveu Exp $
+ * $Id: t30.h,v 1.77 2007/08/02 13:55:48 steveu Exp $
  */
 
 /*! \file */
@@ -277,6 +277,18 @@ enum
     T30_MODEM_DONE
 };
 
+enum
+{
+    T30_FRONT_END_SEND_STEP_COMPLETE = 0,
+    T30_FRONT_END_SEND_COMPLETE,
+    /*! The current receive has completed. This is only needed to report an
+        unexpected end of the receive operation, as might happen with T.38
+        dying. */
+    T30_FRONT_END_RECEIVE_COMPLETE,
+    T30_FRONT_END_SIGNAL_PRESENT,
+    T30_FRONT_END_SIGNAL_ABSENT
+};
+
 enum
 {
     T30_SUPPORT_V27TER = 0x01,
@@ -486,11 +498,12 @@ struct t30_state_s
     int local_interrupt_pending;
     int crp_enabled;
     int line_encoding;
-    int min_row_bits;
+    uint8_t min_scan_time_code;
     int x_resolution;
     int y_resolution;
     int image_width;
     int retries;
+    /*! \brief TRUE if error correcting mode is used. */
     int error_correcting_mode;
     int ppr_count;
     int receiver_not_ready_count;
@@ -543,6 +556,8 @@ typedef struct
     int error_correcting_mode;
     /*! \brief The number of pages transferred so far. */
     int pages_transferred;
+    /*! \brief The number of pages in the file (<0 if not known). */
+    int pages_in_file;
     /*! \brief The number of horizontal pixels in the most recent page. */
     int width;
     /*! \brief The number of vertical pixels in the most recent page. */
@@ -810,17 +825,11 @@ void t30_set_ecm_capability(t30_state_t *s, int enabled);
     \param state TRUE to enable interrupt request, else FALSE. */
 void t30_local_interrupt_request(t30_state_t *s, int state);
 
-/*! Inform the T.30 engine the current transmission has completed.
-    \brief Inform the T.30 engine the current transmission has completed.
-    \param s The T.30 context. */
-void t30_send_complete(void *user_data);
-
-/*! Inform the T.30 engine the current receive has completed. This is
-    only needed to report an unexpected end of the receive operation, as
-    might happen with T.38 dying.
-    \brief Inform the T.30 engine the current receive has completed.
-    \param s The T.30 context. */
-void t30_receive_complete(void *user_data);
+/*! Inform the T.30 engine of a status change in the front end (end of tx, rx signal change, etc.).
+    \brief Inform the T.30 engine of a status change in the front end (end of tx, rx signal change, etc.).
+    \param user_data The T.30 context.
+    \param status The type of status change which occured. */
+void t30_front_end_status(void *user_data, int status);
 
 /*! Get a bit of received non-ECM image data.
     \brief Get a bit of received non-ECM image data.
@@ -864,10 +873,10 @@ void t30_non_ecm_put_chunk(void *user_data, const uint8_t buf[], int len);
 /*! Process a received HDLC frame.
     \brief Process a received HDLC frame.
     \param s The T.30 context.
-    \param ok TRUE if the frame was received without error.
     \param msg The HDLC message.
-    \param int The length of the message, in octets. */
-void t30_hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len);
+    \param int The length of the message, in octets.
+    \param ok TRUE if the frame was received without error. */
+void t30_hdlc_accept(void *user_data, const uint8_t *msg, int len, int ok);
 
 /*! Report the passage of time to the T.30 engine.
     \brief Report the passage of time to the T.30 engine.
