@@ -1,4 +1,3 @@
-#define ENABLE_V17
 /*
  * SpanDSP - a series of DSP components for telephony
  *
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t31.h,v 1.42 2007/04/30 13:13:36 steveu Exp $
+ * $Id: t31.h,v 1.43 2007/05/15 13:22:43 steveu Exp $
  */
 
 /*! \file */
@@ -107,14 +106,12 @@ struct t31_state_s
                messages. */
     fsk_rx_state_t v21rx;
 
-#if defined(ENABLE_V17)
     /*! \brief A V.17 modem context used when sending FAXes at 7200bps, 9600bps
                12000bps or 14400bps*/
     v17_tx_state_t v17tx;
     /*! \brief A V.29 modem context used when receiving FAXes at 7200bps, 9600bps
                12000bps or 14400bps*/
     v17_rx_state_t v17rx;
-#endif
 
     /*! \brief A V.29 modem context used when sending FAXes at 7200bps or
                9600bps */
@@ -135,7 +132,6 @@ struct t31_state_s
 
     /*! \brief Rx power meter, use to detect silence */
     power_meter_t rx_power;
-    int32_t fsk_min_power;
     int16_t last_sample;
     int32_t silence_threshold_power;
     
