@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t4.h,v 1.32 2007/04/08 08:16:18 steveu Exp $
+ * $Id: t4.h,v 1.34 2007/08/02 13:55:48 steveu Exp $
  */
 
 /*! \file */
@@ -41,17 +41,26 @@ for FAX transmission.
 \section t4_page_sec_1 How does it work?
 */
 
-#define T4_COMPRESSION_ITU_T4_1D    1
-#define T4_COMPRESSION_ITU_T4_2D    2
-#define T4_COMPRESSION_ITU_T6       3
+enum
+{
+    T4_COMPRESSION_ITU_T4_1D = 1,
+    T4_COMPRESSION_ITU_T4_2D = 2,
+    T4_COMPRESSION_ITU_T6 = 3
+};
 
-#define T4_X_RESOLUTION_R4          4019
-#define T4_X_RESOLUTION_R8          8037
-#define T4_X_RESOLUTION_R16         16074
+enum
+{
+    T4_X_RESOLUTION_R4 = 4019,
+    T4_X_RESOLUTION_R8 = 8037,
+    T4_X_RESOLUTION_R16 = 16074
+};
 
-#define T4_Y_RESOLUTION_STANDARD    3850
-#define T4_Y_RESOLUTION_FINE        7700
-#define T4_Y_RESOLUTION_SUPERFINE   15400
+enum
+{
+    T4_Y_RESOLUTION_STANDARD = 3850,
+    T4_Y_RESOLUTION_FINE = 7700,
+    T4_Y_RESOLUTION_SUPERFINE = 15400
+};
 
 /*!
     T.4 FAX compression/decompression descriptor. This defines the working state
@@ -96,6 +105,7 @@ typedef struct
     int             stop_page;
 
     int             pages_transferred;
+    int             pages_in_file;
     /*! Column-to-column (X) resolution in pixels per metre. */
     int             x_resolution;
     /*! Row-to-row (Y) resolution in pixels per metre. */
@@ -182,6 +192,8 @@ typedef struct
 {
     /*! \brief The number of pages transferred so far. */
     int pages_transferred;
+    /*! \brief The number of pages in the file (<0 if unknown). */
+    int pages_in_file;
     /*! \brief The number of horizontal pixels in the most recent page. */
     int width;
     /*! \brief The number of vertical pixels in the most recent page. */
@@ -431,6 +443,11 @@ int t4_tx_get_x_resolution(t4_state_t *s);
     \return The number of columns. */
 int t4_tx_get_image_width(t4_state_t *s);
 
+/*! \brief Get the number of pages in the file.
+    \param s The T.4 context.
+    \return The number of pages, or -1 if there is an error. */
+int t4_tx_get_pages_in_file(t4_state_t *s);
+
 /*! Get the current image transfer statistics. 
     \brief Get the current transfer statistics.
     \param s The T.4 context.
