@@ -99,6 +99,8 @@ typedef struct
     int16_t last_sample;
     /*! \brief >0 if a signal above the minimum is present. It may or may not be a V.27ter signal. */
     int signal_present;
+    /*! \brief Whether or not a carrier drop was detected and the signal delivery is pending. */
+    int carrier_drop_pending;
     /*! \brief TRUE if the previous trained values are to be reused. */
     int old_train;
 
