@@ -1,4 +1,3 @@
-#define ENABLE_V17
 /*
  * SpanDSP - a series of DSP components for telephony
  *
@@ -26,7 +25,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t31.c,v 1.88 2007/04/30 13:13:36 steveu Exp $
+ * $Id: t31.c,v 1.92 2007/07/29 17:56:41 steveu Exp $
  */
 
 /*! \file */
@@ -70,10 +69,8 @@
 #include "spandsp/v29tx.h"
 #include "spandsp/v27ter_rx.h"
 #include "spandsp/v27ter_tx.h"
-#if defined(ENABLE_V17)
 #include "spandsp/v17rx.h"
 #include "spandsp/v17tx.h"
-#endif
 #include "spandsp/t4.h"
 #include "spandsp/t30.h"
 #include "spandsp/t38_core.h"
@@ -86,7 +83,6 @@
 #define DATA_TX_COUNT               1
 #define DATA_END_TX_COUNT           1
 #define DEFAULT_DTE_TIMEOUT         5
-#define DEFAULT_MIN_POWER           8000
 
 typedef const char *(*at_cmd_service_t)(t31_state_t *s, const char *cmd);
 
@@ -126,10 +122,8 @@ enum
 };
 
 static int restart_modem(t31_state_t *s, int new_modem);
-static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len);
-#if defined(ENABLE_V17)
+static void hdlc_accept(void *user_data, const uint8_t *msg, int len, int ok);
 static int early_v17_rx(void *user_data, const int16_t amp[], int len);
-#endif
 static int early_v27ter_rx(void *user_data, const int16_t amp[], int len);
 static int early_v29_rx(void *user_data, const int16_t amp[], int len);
 static int dummy_rx(void *s, const int16_t amp[], int len);
@@ -251,7 +245,7 @@ static int process_rx_data(t38_core_state_t *s, void *user_data, int data_type,
         /* Don't deal with zero length frames. Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_OK
            packets, when they have sent no data for the body of the frame. */
         if (t->current_rx_type == T31_V21_RX  &&  t->tx_out_bytes > 0  &&  !t->missing_data)
-            hdlc_accept((void *) t, TRUE, t->hdlc_rx_buf, t->hdlc_rx_len);
+            hdlc_accept((void *) t, t->hdlc_rx_buf, t->hdlc_rx_len, TRUE);
         t->hdlc_rx_len = 0;
         t->missing_data = FALSE;
         break;
@@ -271,8 +265,8 @@ static int process_rx_data(t38_core_state_t *s, void *user_data, int data_type,
             /* Don't deal with zero length frames. Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_OK
                packets, when they have sent no data for the body of the frame. */
             if (t->tx_out_bytes > 0)
-                hdlc_accept((void *) t, TRUE, t->hdlc_rx_buf, t->hdlc_rx_len);
-            hdlc_accept((void *) t, TRUE, NULL, PUTBIT_CARRIER_DOWN);
+                hdlc_accept((void *) t, t->hdlc_rx_buf, t->hdlc_rx_len, TRUE);
+            hdlc_accept((void *) t, NULL, PUTBIT_CARRIER_DOWN, TRUE);
         }
         t->tx_out_bytes = 0;
         t->missing_data = FALSE;
@@ -284,7 +278,7 @@ static int process_rx_data(t38_core_state_t *s, void *user_data, int data_type,
             span_log(&t->logging, SPAN_LOG_WARNING, "There is data in a T38_FIELD_HDLC_FCS_BAD_SIG_END!\n");
         span_log(&t->logging, SPAN_LOG_FLOW, "Type %s - CRC bad, sig end (%s)\n", t30_frametype(t->tx_data[2]), (t->missing_data)  ?  "missing octets"  :  "clean");
         if (t->current_rx_type == T31_V21_RX)
-            hdlc_accept((void *) t, TRUE, NULL, PUTBIT_CARRIER_DOWN);
+            hdlc_accept((void *) t, NULL, PUTBIT_CARRIER_DOWN, TRUE);
         t->hdlc_rx_len = 0;
         t->missing_data = FALSE;
         break;
@@ -295,7 +289,7 @@ static int process_rx_data(t38_core_state_t *s, void *user_data, int data_type,
            i.e. they send T38_FIELD_HDLC_FCS_OK, and then T38_FIELD_HDLC_SIG_END when the carrier actually drops.
            The other is because the HDLC signal drops unexpectedly - i.e. not just after a final frame. */
         if (t->current_rx_type == T31_V21_RX)
-            hdlc_accept((void *) t, TRUE, NULL, PUTBIT_CARRIER_DOWN);
+            hdlc_accept((void *) t, NULL, PUTBIT_CARRIER_DOWN, TRUE);
         t->hdlc_rx_len = 0;
         t->missing_data = FALSE;
         break;
@@ -453,11 +447,9 @@ static int t31_modem_control_handler(at_state_t *s, void *user_data, int op, con
     {
     case AT_MODEM_CONTROL_ANSWER:
         t->call_samples = 0;
-        t->fsk_min_power = DEFAULT_MIN_POWER;
         break;
     case AT_MODEM_CONTROL_CALL:
         t->call_samples = 0;
-        t->fsk_min_power = DEFAULT_MIN_POWER;
         break;
     case AT_MODEM_CONTROL_ONHOOK:
         if (t->tx_holding)
@@ -616,7 +608,7 @@ static void hdlc_tx_underflow(void *user_data)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
+static void hdlc_accept(void *user_data, const uint8_t *msg, int len, int ok)
 {
     uint8_t buf[256];
     t31_state_t *s;
@@ -677,7 +669,7 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
                 s->modem = T31_V21_RX;
                 s->at_state.transmit = FALSE;
             }
-            if (s->modem != T31_V21_RX)
+            if (s->modem == T31_V17_RX || s->modem == T31_V27TER_RX || s->modem == T31_V29_RX)
             {
                 /* V.21 has been detected while expecting a different carrier.
                    If +FAR=0 then result +FCERROR and return to command-mode.
@@ -718,10 +710,6 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
                     }
                 }
             }
-            /* To avoid detecting echo as signalling in the future we update
-               the minimum power threshold relative to the current power
-               level for future V.21 receptions. */
-            s->fsk_min_power = s->rx_power.reading / 2;
             break;
         case PUTBIT_ABORT:
             /* Just ignore these */
@@ -732,6 +720,20 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
         }
         return;
     }
+    if (!s->rx_message_received)
+    {
+        if (s->at_state.dte_is_waiting)
+        {
+            /* Report CONNECT as soon as possible to avoid a timeout. */
+            at_put_response_code(&s->at_state, AT_RESPONSE_CODE_CONNECT);
+            s->rx_message_received = TRUE;
+        }
+        else
+        {
+            buf[0] = AT_RESPONSE_CODE_CONNECT;
+            queue_write_msg(s->rx_queue, buf, 1);
+        }
+    }
     /* If OK is pending then we just ignore whatever comes in */
     if (!s->at_state.ok_is_pending)
     {
@@ -784,7 +786,8 @@ static void t31_v21_rx(t31_state_t *s)
     s->hdlc_final = FALSE;
     s->hdlc_tx_len = 0;
     s->dled = FALSE;
-    fsk_rx_init(&(s->v21rx), &preset_fsk_specs[FSK_V21CH2], TRUE, s->fsk_min_power, (put_bit_func_t) hdlc_rx_put_bit, &(s->hdlcrx));
+    fsk_rx_init(&(s->v21rx), &preset_fsk_specs[FSK_V21CH2], TRUE, (put_bit_func_t) hdlc_rx_put_bit, &(s->hdlcrx));
+    fsk_rx_signal_cutoff(&(s->v21rx), -39.09);
     s->at_state.transmit = TRUE;
 }
 /*- End of function --------------------------------------------------------*/
@@ -890,7 +893,7 @@ static int restart_modem(t31_state_t *s, int new_modem)
         {
             hdlc_tx_init(&(s->hdlctx), FALSE, 2, FALSE, hdlc_tx_underflow, s);
             /* The spec says 1s +-15% of preamble. So, the minimum is 32 octets. */
-            hdlc_tx_preamble(&(s->hdlctx), 32);
+            hdlc_tx_flags(&(s->hdlctx), 32);
             fsk_tx_init(&(s->v21tx), &preset_fsk_specs[FSK_V21CH2], (get_bit_func_t) hdlc_tx_get_bit, &(s->hdlctx));
             s->tx_handler = (span_tx_handler_t *) &fsk_tx;
             s->tx_user_data = &(s->v21tx);
@@ -912,7 +915,6 @@ static int restart_modem(t31_state_t *s, int new_modem)
             t31_v21_rx(s);
         }
         break;
-#if defined(ENABLE_V17)
     case T31_V17_TX:
         if (s->t38_mode)
         {
@@ -956,7 +958,6 @@ static int restart_modem(t31_state_t *s, int new_modem)
         }
         s->at_state.transmit = FALSE;
         break;
-#endif
     case T31_V27TER_TX:
         if (s->t38_mode)
         {
@@ -1285,7 +1286,6 @@ static int process_class1_cmd(at_state_t *t, void *user_data, int direction, int
             s->short_train = FALSE;
             s->bit_rate = 9600;
             break;
-#if defined(ENABLE_V17)
         case 73:
             new_modem = (new_transmit)  ?  T31_V17_TX  :  T31_V17_RX;
             s->short_train = FALSE;
@@ -1326,7 +1326,6 @@ static int process_class1_cmd(at_state_t *t, void *user_data, int direction, int
             s->short_train = TRUE;
             s->bit_rate = 14400;
             break;
-#endif
         default:
             return -1;
         }
@@ -1369,6 +1368,12 @@ int t31_at_rx(t31_state_t *s, const char *t, int len)
         /* Data from the DTE in this state returns us to command mode */
         if (len)
         {
+            if (s->at_state.rx_signal_present)
+            {
+                s->at_state.rx_data[s->at_state.rx_data_bytes++] = DLE;
+                s->at_state.rx_data[s->at_state.rx_data_bytes++] = ETX;
+                s->at_state.at_tx_handler(&s->at_state, s->at_state.at_tx_user_data, s->at_state.rx_data, s->at_state.rx_data_bytes);
+            }
             s->at_state.rx_data_bytes = 0;
             s->at_state.transmit = FALSE;
             s->modem = T31_SILENCE_TX;
@@ -1438,7 +1443,6 @@ static int cng_rx(void *user_data, const int16_t amp[], int len)
 }
 /*- End of function --------------------------------------------------------*/
 
-#if defined(ENABLE_V17)
 static int early_v17_rx(void *user_data, const int16_t amp[], int len)
 {
     t31_state_t *s;
@@ -1468,7 +1472,6 @@ static int early_v17_rx(void *user_data, const int16_t amp[], int len)
     return len;
 }
 /*- End of function --------------------------------------------------------*/
-#endif
 
 static int early_v27ter_rx(void *user_data, const int16_t amp[], int len)
 {
@@ -1546,7 +1549,7 @@ int t31_rx(t31_state_t *s, int16_t amp[], int len)
         /* Clean up any DC influence. */
         power = power_meter_update(&(s->rx_power), amp[i] - s->last_sample);
         s->last_sample = amp[i];
-        if (power > (s->silence_threshold_power + s->fsk_min_power) / 2)
+        if (power > s->silence_threshold_power)
         {
             s->silence_heard = 0;
         }
@@ -1630,9 +1633,7 @@ int t31_tx(t31_state_t *s, int16_t amp[], int max_len)
                     t31_set_at_rx_mode(s, AT_MODE_HDLC);
                     break;
                 case T31_V21_TX:
-#if defined(ENABLE_V17)
                 case T31_V17_TX:
-#endif
                 case T31_V27TER_TX:
                 case T31_V29_TX:
                     s->modem = -1;
@@ -1677,10 +1678,8 @@ t31_state_t *t31_init(t31_state_t *s,
 
     s->modem_control_handler = modem_control_handler;
     s->modem_control_user_data = modem_control_user_data;
-#if defined(ENABLE_V17)
     v17_rx_init(&(s->v17rx), 14400, non_ecm_put_bit, s);
     v17_tx_init(&(s->v17tx), 14400, FALSE, non_ecm_get_bit, s);
-#endif
     v29_rx_init(&(s->v29rx), 9600, non_ecm_put_bit, s);
     v29_rx_signal_cutoff(&(s->v29rx), -45.5);
     v29_tx_init(&(s->v29tx), 9600, FALSE, non_ecm_get_bit, s);
@@ -1688,9 +1687,8 @@ t31_state_t *t31_init(t31_state_t *s,
     v27ter_tx_init(&(s->v27ter_tx), 4800, FALSE, non_ecm_get_bit, s);
     silence_gen_init(&(s->silence_gen), 0);
     power_meter_init(&(s->rx_power), 4);
-    s->fsk_min_power = DEFAULT_MIN_POWER;
     s->last_sample = 0;
-    s->silence_threshold_power = power_meter_level_dbm0(-43);
+    s->silence_threshold_power = power_meter_level_dbm0(-36);
     s->at_state.rx_signal_present = FALSE;
     s->at_state.rx_trained = FALSE;
 
