@@ -1,4 +1,3 @@
-#define ENABLE_V17
 /*
  * SpanDSP - a series of DSP components for telephony
  *
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t38_gateway.c,v 1.77 2007/04/13 12:12:07 steveu Exp $
+ * $Id: t38_gateway.c,v 1.84 2007/07/29 17:56:41 steveu Exp $
  */
 
 /*! \file */
@@ -56,6 +55,7 @@
 #include "spandsp/complex.h"
 #include "spandsp/tone_generate.h"
 #include "spandsp/async.h"
+#include "spandsp/crc.h"
 #include "spandsp/hdlc.h"
 #include "spandsp/silence_gen.h"
 #include "spandsp/fsk.h"
@@ -63,10 +63,8 @@
 #include "spandsp/v29tx.h"
 #include "spandsp/v27ter_rx.h"
 #include "spandsp/v27ter_tx.h"
-#if defined(ENABLE_V17)
 #include "spandsp/v17rx.h"
 #include "spandsp/v17tx.h"
-#endif
 #include "spandsp/t4.h"
 
 #include "spandsp/t30_fcf.h"
@@ -241,7 +239,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         break;
     case T38_IND_V21_PREAMBLE:
         hdlc_tx_init(&s->hdlctx, FALSE, 2, TRUE, hdlc_underflow_handler, s);
-        hdlc_tx_preamble(&s->hdlctx, 32);
+        hdlc_tx_flags(&s->hdlctx, 32);
         s->hdlc_len[s->hdlc_in] = 0;
         fsk_tx_init(&(s->v21tx), &preset_fsk_specs[FSK_V21CH2], (get_bit_func_t) hdlc_tx_get_bit, &s->hdlctx);
         /* Impose a minimum silence */
@@ -253,7 +251,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         break;
     case T38_IND_V27TER_2400_TRAINING:
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
-        hdlc_tx_preamble(&s->hdlctx, 60);
+        hdlc_tx_flags(&s->hdlctx, 60);
         v27ter_tx_restart(&(s->v27ter_tx), 2400, s->use_tep);
         v27ter_tx_set_get_bit(&(s->v27ter_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &(silence_gen);
@@ -262,7 +260,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v27ter_tx);
         break;
     case T38_IND_V27TER_4800_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 120);
+        hdlc_tx_flags(&s->hdlctx, 120);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v27ter_tx_restart(&(s->v27ter_tx), 4800, s->use_tep);
         v27ter_tx_set_get_bit(&(s->v27ter_tx), get_bit_func, get_bit_user_data);
@@ -272,7 +270,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v27ter_tx);
         break;
     case T38_IND_V29_7200_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 180);
+        hdlc_tx_flags(&s->hdlctx, 180);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v29_tx_restart(&(s->v29tx), 7200, s->use_tep);
         v29_tx_set_get_bit(&(s->v29tx), get_bit_func, get_bit_user_data);
@@ -282,7 +280,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v29tx);
         break;
     case T38_IND_V29_9600_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 240);
+        hdlc_tx_flags(&s->hdlctx, 240);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v29_tx_restart(&(s->v29tx), 9600, s->use_tep);
         v29_tx_set_get_bit(&(s->v29tx), get_bit_func, get_bit_user_data);
@@ -291,9 +289,8 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_handler = (span_tx_handler_t *) &(v29_tx);
         s->next_tx_user_data = &(s->v29tx);
         break;
-#if defined(ENABLE_V17)
     case T38_IND_V17_7200_SHORT_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 180);
+        hdlc_tx_flags(&s->hdlctx, 180);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 7200, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -303,7 +300,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_7200_LONG_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 180);
+        hdlc_tx_flags(&s->hdlctx, 180);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 7200, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -313,7 +310,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_9600_SHORT_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 240);
+        hdlc_tx_flags(&s->hdlctx, 240);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 9600, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -323,7 +320,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_9600_LONG_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 240);
+        hdlc_tx_flags(&s->hdlctx, 240);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 9600, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -333,7 +330,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_12000_SHORT_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 300);
+        hdlc_tx_flags(&s->hdlctx, 300);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 12000, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -343,7 +340,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_12000_LONG_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 300);
+        hdlc_tx_flags(&s->hdlctx, 300);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 12000, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -353,7 +350,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_14400_SHORT_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 360);
+        hdlc_tx_flags(&s->hdlctx, 360);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 14400, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -363,7 +360,7 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_user_data = &(s->v17tx);
         break;
     case T38_IND_V17_14400_LONG_TRAINING:
-        hdlc_tx_preamble(&s->hdlctx, 360);
+        hdlc_tx_flags(&s->hdlctx, 360);
         silence_gen_alter(&s->silence_gen, ms_to_samples(75));
         v17_tx_restart(&(s->v17tx), 14400, s->use_tep, s->short_train);
         v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
@@ -372,7 +369,6 @@ static int set_next_tx_type(t38_gateway_state_t *s)
         s->next_tx_handler = (span_tx_handler_t *) &(v17_tx);
         s->next_tx_user_data = &(s->v17tx);
         break;
-#endif
     case T38_IND_V8_ANSAM:
         break;
     case T38_IND_V8_SIGNAL:
@@ -466,12 +462,15 @@ static void edit_control_messages(t38_gateway_state_t *s, uint8_t *buf, int len,
             case DISBIT3:
             case (DISBIT4 | DISBIT3):
                 /* V.27ter and V.29 */
+                if (!(s->supported_modems & T30_SUPPORT_V29))
+                    buf[4] &= ~DISBIT3;
                 break;
             case (DISBIT6 | DISBIT4 | DISBIT3):
                 /* V.27ter, V.29 and V.17 */
-#if !defined(ENABLE_V17)
-                buf[4] &= ~DISBIT6;
-#endif
+                if (!(s->supported_modems & T30_SUPPORT_V17))
+                    buf[4] &= ~DISBIT6;
+                if (!(s->supported_modems & T30_SUPPORT_V29))
+                    buf[4] &= ~DISBIT3;
                 break;
             case (DISBIT5 | DISBIT4):
             case (DISBIT6 | DISBIT4):
@@ -558,7 +557,6 @@ static void monitor_control_messages(t38_gateway_state_t *s, uint8_t *buf, int l
                 s->fast_modem = T38_V29_RX;
                 s->short_train = FALSE;
                 break;
-#if defined(ENABLE_V17)
             case DISBIT6:
                 s->fast_bit_rate = 14400;
                 s->fast_modem = T38_V17_RX;
@@ -579,7 +577,6 @@ static void monitor_control_messages(t38_gateway_state_t *s, uint8_t *buf, int l
                 s->fast_modem = T38_V17_RX;
                 s->short_train = FALSE;
                 break;
-#endif
             case (DISBIT5 | DISBIT3):
             case (DISBIT5 | DISBIT4 | DISBIT3):
             case (DISBIT6 | DISBIT5):
@@ -777,8 +774,9 @@ static int process_rx_data(t38_core_state_t *t, void *user_data, int data_type,
             /* The sender has incorrectly included data in this message. It is unclear what we should do
                with it, to maximise tolerance of buggy implementations. */
         }
-        /* Don't deal with repeats of the same field type. Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_OK
-           packets, when they have sent no data for the body of the frame. */
+        /* Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_OK messages, in IFP packets with
+           incrementing sequence numbers, which are actually repeats. They get through to this point because
+           of the incrementing sequence numbers. We need to filter them here in a context sensitive manner. */
         if (t->current_rx_data_type != data_type
             ||
             t->current_rx_field_type != field_type)
@@ -803,8 +801,9 @@ static int process_rx_data(t38_core_state_t *t, void *user_data, int data_type,
             /* The sender has incorrectly included data in this message. We can safely ignore it, as the
                bad FCS means we will throw away the whole message, anyway. */
         }
-        /* Don't deal with repeats of the same field type. Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_BAD
-           packets, when they have sent no data for the body of the frame. */
+        /* Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_BAD messages, in IFP packets with
+           incrementing sequence numbers, which are actually repeats. They get through to this point because
+           of the incrementing sequence numbers. We need to filter them here in a context sensitive manner. */
         if (t->current_rx_data_type != data_type
             ||
             t->current_rx_field_type != field_type)
@@ -830,8 +829,9 @@ static int process_rx_data(t38_core_state_t *t, void *user_data, int data_type,
             /* The sender has incorrectly included data in this message. It is unclear what we should do
                with it, to maximise tolerance of buggy implementations. */
         }
-        /* Don't deal with repeats of the same field type. Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_OK_SIG_END
-           packets, when they have sent no data for the body of the frame. */
+        /* Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_OK_SIG_END messages, in IFP packets with
+           incrementing sequence numbers, which are actually repeats. They get through to this point because
+           of the incrementing sequence numbers. We need to filter them here in a context sensitive manner. */
         if (t->current_rx_data_type != data_type
             ||
             t->current_rx_field_type != field_type)
@@ -859,8 +859,9 @@ static int process_rx_data(t38_core_state_t *t, void *user_data, int data_type,
             /* The sender has incorrectly included data in this message. We can safely ignore it, as the
                bad FCS means we will throw away the whole message, anyway. */
         }
-        /* Don't deal with repeats of the same field type. Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_BAD_SIG_END
-           packets, when they have sent no data for the body of the frame. */
+        /* Some T.38 implementations send multiple T38_FIELD_HDLC_FCS_BAD_SIG_END messages, in IFP packets with
+           incrementing sequence numbers, which are actually repeats. They get through to this point because
+           of the incrementing sequence numbers. We need to filter them here in a context sensitive manner. */
         if (t->current_rx_data_type != data_type
             ||
             t->current_rx_field_type != field_type)
@@ -888,6 +889,9 @@ static int process_rx_data(t38_core_state_t *t, void *user_data, int data_type,
             /* The sender has incorrectly included data in this message, but there seems nothing meaningful
                it could be. There could not be an FCS good/bad report beyond this. */
         }
+        /* Some T.38 implementations send multiple T38_FIELD_HDLC_SIG_END messages, in IFP packets with
+           incrementing sequence numbers, which are actually repeats. They get through to this point because
+           of the incrementing sequence numbers. We need to filter them here in a context sensitive manner. */
         if (t->current_rx_data_type != data_type
             ||
             t->current_rx_field_type != field_type)
@@ -925,6 +929,9 @@ static int process_rx_data(t38_core_state_t *t, void *user_data, int data_type,
         s->corrupt_the_frame_from_t38 = FALSE;
         break;
     case T38_FIELD_T4_NON_ECM_SIG_END:
+        /* Some T.38 implementations send multiple T38_FIELD_T4_NON_ECM_SIG_END messages, in IFP packets with
+           incrementing sequence numbers, which are actually repeats. They get through to this point because
+           of the incrementing sequence numbers. We need to filter them here in a context sensitive manner. */
         if (t->current_rx_data_type != data_type
             ||
             t->current_rx_field_type != field_type)
@@ -997,7 +1004,6 @@ static void announce_training(t38_gateway_state_t *s)
     ind = T38_IND_NO_SIGNAL;
     switch (s->fast_rx_active)
     {
-#if defined(ENABLE_V17)
     case T38_V17_RX:
         switch (s->fast_bit_rate)
         {
@@ -1024,7 +1030,6 @@ static void announce_training(t38_gateway_state_t *s)
             break;
         }
         break;
-#endif
     case T38_V27TER_RX:
         switch (s->fast_bit_rate)
         {
@@ -1436,13 +1441,6 @@ static void t38_hdlc_rx_put_bit(hdlc_rx_state_t *t, int new_bit)
 }
 /*- End of function --------------------------------------------------------*/
 
-static int dummy_rx(void *s, const int16_t amp[], int len)
-{
-    return 0;
-}
-/*- End of function --------------------------------------------------------*/
-
-#if defined(ENABLE_V17)
 static int early_v17_rx(void *user_data, const int16_t amp[], int len)
 {
     t38_gateway_state_t *s;
@@ -1470,7 +1468,6 @@ static int early_v17_rx(void *user_data, const int16_t amp[], int len)
     return len;
 }
 /*- End of function --------------------------------------------------------*/
-#endif
 
 static int early_v27ter_rx(void *user_data, const int16_t amp[], int len)
 {
@@ -1560,7 +1557,10 @@ static int restart_rx_modem(t38_gateway_state_t *s)
     s->rx_trained = FALSE;
     /* Default to the transmit data being V.21, unless a faster modem pops up trained. */
     s->current_tx_data_type = T38_DATA_V21;
-    fsk_rx_init(&(s->v21rx), &preset_fsk_specs[FSK_V21CH2], TRUE, 0, (put_bit_func_t) t38_hdlc_rx_put_bit, &(s->hdlcrx));
+    fsk_rx_init(&(s->v21rx), &preset_fsk_specs[FSK_V21CH2], TRUE, (put_bit_func_t) t38_hdlc_rx_put_bit, &(s->hdlcrx));
+#if 0
+    fsk_rx_signal_cutoff(&(s->v21rx), -45.5);
+#endif
     if (s->short_train  &&  s->ecm_mode)
     {
         put_bit_func = (put_bit_func_t) t38_hdlc_rx_put_bit;
@@ -1575,7 +1575,6 @@ static int restart_rx_modem(t38_gateway_state_t *s)
     s->octets_per_data_packet = 1;
     switch (s->fast_modem)
     {
-#if defined(ENABLE_V17)
     case T38_V17_RX:
         v17_rx_restart(&(s->v17rx), s->fast_bit_rate, 2);
         v17_rx_set_put_bit(&(s->v17rx), put_bit_func, put_bit_user_data);
@@ -1583,7 +1582,6 @@ static int restart_rx_modem(t38_gateway_state_t *s)
         s->rx_user_data = s;
         s->fast_rx_active = T38_V17_RX;
         break;
-#endif
     case T38_V27TER_RX:
         v27ter_rx_restart(&(s->v27ter_rx), s->fast_bit_rate, FALSE);
         v27ter_rx_set_put_bit(&(s->v27ter_rx), put_bit_func, put_bit_user_data);
@@ -1652,6 +1650,12 @@ void t38_gateway_set_transmit_on_idle(t38_gateway_state_t *s, int transmit_on_id
 void t38_gateway_set_supported_modems(t38_gateway_state_t *s, int supported_modems)
 {
     s->supported_modems = supported_modems;
+    if ((s->supported_modems & T30_SUPPORT_V17))
+        s->t38.fastest_image_data_rate = 14400;
+    else if ((s->supported_modems & T30_SUPPORT_V29))
+        s->t38.fastest_image_data_rate = 9600;
+    else
+        s->t38.fastest_image_data_rate = 4800;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -1671,11 +1675,12 @@ t38_gateway_state_t *t38_gateway_init(t38_gateway_state_t *s,
     memset(s, 0, sizeof(*s));
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
     span_log_set_protocol(&s->logging, "T.38G");
-#if defined(ENABLE_V17)
     v17_rx_init(&(s->v17rx), 14400, non_ecm_put_bit, s);
     v17_tx_init(&(s->v17tx), 14400, FALSE, non_ecm_get_bit, s);
-#endif
     v29_rx_init(&(s->v29rx), 9600, non_ecm_put_bit, s);
+#if 0
+    v29_rx_signal_cutoff(&(s->v29rx), -45.5);
+#endif
     v29_tx_init(&(s->v29tx), 9600, FALSE, non_ecm_get_bit, s);
     v27ter_rx_init(&(s->v27ter_rx), 4800, non_ecm_put_bit, s);
     v27ter_tx_init(&(s->v27ter_tx), 4800, FALSE, non_ecm_get_bit, s);
@@ -1692,17 +1697,7 @@ t38_gateway_state_t *t38_gateway_init(t38_gateway_state_t *s,
                   (void *) s,
                   tx_packet_handler,
                   tx_packet_user_data);
-#if defined(ENABLE_V17)
-    s->t38.fastest_image_data_rate = 14400;
-#else
-    s->t38.fastest_image_data_rate = 9600;
-#endif
-    t38_gateway_set_supported_modems(s,
-#if defined(ENABLE_V17)
-                                     T30_SUPPORT_V27TER | T30_SUPPORT_V29 | T30_SUPPORT_V17);
-#else    
-                                     T30_SUPPORT_V27TER | T30_SUPPORT_V29);
-#endif
+    t38_gateway_set_supported_modems(s, T30_SUPPORT_V27TER | T30_SUPPORT_V29);
     t38_gateway_set_nsx_suppression(s, TRUE);
     s->ecm_allowed = FALSE;
     restart_rx_modem(s);
