@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t4.c,v 1.71 2007/04/05 16:32:08 steveu Exp $
+ * $Id: t4.c,v 1.74 2007/05/29 12:55:03 steveu Exp $
  */
 
 /*
@@ -673,6 +673,7 @@ int t4_rx_end_page(t4_state_t *s)
        right. For multi-page documents we will need to come back and fill in
        the right answer when we know it. */
     TIFFSetField(s->tiff_file, TIFFTAG_PAGENUMBER, s->pages_transferred++, 1);
+    s->pages_in_file = s->pages_transferred;
     if (s->output_compression == COMPRESSION_CCITT_T4)
     {
         if (s->bad_rows)
@@ -1101,6 +1102,9 @@ int t4_rx_init(t4_state_t *s, const char *file, int output_encoding)
     s->bytes_per_row = 0;
 
     s->pages_transferred = 0;
+    s->pages_in_file = 0;
+    s->start_page = 0;
+    s->stop_page = INT_MAX;
 
     s->image_buffer = NULL;
     s->image_buffer_size = 0;
@@ -1719,6 +1723,7 @@ int t4_tx_init(t4_state_t *s, const char *file, int start_page, int stop_page)
     s->rows_to_next_1d_row = s->max_rows_to_next_1d_row - 1;
 
     s->pages_transferred = s->start_page;
+    s->pages_in_file = -1;
     if ((s->row_buf = malloc(s->bytes_per_row)) == NULL)
         return -1;
     if ((s->ref_row_buf = malloc(s->bytes_per_row)) == NULL)
@@ -1878,12 +1883,20 @@ int t4_tx_start_page(t4_state_t *s)
 int t4_tx_more_pages(t4_state_t *s)
 {
     span_log(&s->logging, SPAN_LOG_FLOW, "Checking for the existance of page %d\n", s->pages_transferred + 1);
-    if (s->pages_transferred > s->stop_page)
+    if (s->pages_transferred >= s->stop_page)
         return -1;
     if (s->tiff_file == NULL)
         return -1;
-    if (!TIFFSetDirectory(s->tiff_file, (tdir_t) s->pages_transferred + 1))
-        return -1;
+    if (s->pages_in_file >= 0)
+    {
+        if (s->pages_transferred + 1 >= s->pages_in_file)
+            return -1;
+    }
+    else
+    {
+        if (!TIFFSetDirectory(s->tiff_file, (tdir_t) s->pages_transferred + 1))
+            return -1;
+    }
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -2034,9 +2047,25 @@ int t4_tx_get_image_width(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+int t4_tx_get_pages_in_file(t4_state_t *s)
+{
+    int max;
+
+    max = 0;
+    while (TIFFSetDirectory(s->tiff_file, (tdir_t) max))
+    	max++;
+    s->pages_in_file = max;
+    /* Back to the previous page */
+    if (!TIFFSetDirectory(s->tiff_file, (tdir_t) s->pages_transferred))
+        return -1;
+    return max;
+}
+/*- End of function --------------------------------------------------------*/
+
 void t4_get_transfer_statistics(t4_state_t *s, t4_stats_t *t)
 {
-    t->pages_transferred = s->pages_transferred;
+    t->pages_transferred = s->pages_transferred - s->start_page;
+    t->pages_in_file = s->pages_in_file;
     t->width = s->image_width;
     t->length = s->image_length;
     t->bad_rows = s->bad_rows;
