@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tone_detect.c,v 1.31 2007/03/03 10:40:33 steveu Exp $
+ * $Id: tone_detect.c,v 1.32 2007/06/22 12:40:35 steveu Exp $
  */
  
 /*! \file tone_detect.h */
@@ -55,7 +55,11 @@
 
 void make_goertzel_descriptor(goertzel_descriptor_t *t, float freq, int samples)
 {
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+    t->fac = 4096.0f*2.0f*cosf(2.0f*M_PI*(freq/(float) SAMPLE_RATE));
+#else
     t->fac = 2.0f*cosf(2.0f*M_PI*(freq/(float) SAMPLE_RATE));
+#endif
     t->samples = samples;
 }
 /*- End of function --------------------------------------------------------*/
@@ -88,7 +92,11 @@ int goertzel_update(goertzel_state_t *s,
                     int samples)
 {
     int i;
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+    int32_t v1;
+#else
     float v1;
+#endif
 
     if (samples > s->samples - s->current_sample)
         samples = s->samples - s->current_sample;
@@ -96,7 +104,11 @@ int goertzel_update(goertzel_state_t *s,
     {
         v1 = s->v2;
         s->v2 = s->v3;
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+        s->v3 = ((s->fac*s->v2) >> 12) - v1 + (amp[i] >> 8);
+#else
         s->v3 = s->fac*s->v2 - v1 + amp[i];
+#endif
     }
     s->current_sample += samples;
     return samples;
@@ -105,16 +117,28 @@ int goertzel_update(goertzel_state_t *s,
 
 float goertzel_result(goertzel_state_t *s)
 {
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+    int32_t v1;
+#else
     float v1;
+#endif
 
     /* Push a zero through the process to finish things off. */
     v1 = s->v2;
     s->v2 = s->v3;
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+    s->v3 = ((s->fac*s->v2) >> 12) - v1;
+#else
     s->v3 = s->fac*s->v2 - v1;
+#endif
     /* Now calculate the non-recursive side of the filter. */
     /* The result here is not scaled down to allow for the magnification
        effect of the filter (the usual DFT magnification effect). */
+#if defined(USE_FIXED_POINT_EXPERIMENTAL)
+    return s->v3*s->v3 + s->v2*s->v2 - ((s->v2*s->v3) >> 12)*s->fac;
+#else
     return s->v3*s->v3 + s->v2*s->v2 - s->v2*s->v3*s->fac;
+#endif
 }
 /*- End of function --------------------------------------------------------*/
 /*- End of file ------------------------------------------------------------*/
