@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v22bis_tx.c,v 1.32 2006/10/24 13:45:27 steveu Exp $
+ * $Id: v22bis_tx.c,v 1.36 2007/05/07 13:11:46 steveu Exp $
  */
 
 /*! \file */
@@ -37,7 +37,12 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
@@ -270,444 +275,444 @@ const complexf_t v22bis_constellation[16] =
 static const float pulseshaper[PULSESHAPER_COEFF_SETS][V22BIS_TX_FILTER_STEPS] =
 {
     {
-        -0.0047287346,          /* Filter 0 */
-        -0.0083947197,
-        -0.0087380763,
-         0.0088053673,
-         0.5108981827,
-         0.5108981827,
-         0.0088053673,
-        -0.0087380763,
-        -0.0083947197
+        -0.0047287346f,         /* Filter 0 */
+        -0.0083947197f,
+        -0.0087380763f,
+         0.0088053673f,
+         0.5108981827f,
+         0.5108981827f,
+         0.0088053673f,
+        -0.0087380763f,
+        -0.0083947197f
     },
     {
-        -0.0044638629,          /* Filter 1 */
-        -0.0089241700,
-        -0.0111288952,
-         0.0023412184,
-         0.5623914901,
-         0.4599551720,
-         0.0144817755,
-        -0.0063186648,
-        -0.0077293609
+        -0.0044638629f,         /* Filter 1 */
+        -0.0089241700f,
+        -0.0111288952f,
+         0.0023412184f,
+         0.5623914901f,
+         0.4599551720f,
+         0.0144817755f,
+        -0.0063186648f,
+        -0.0077293609f
     },
     {
-        -0.0041048584,          /* Filter 2 */
-        -0.0093040596,
-        -0.0134459768,
-        -0.0048558766,
-         0.6141017035,
-         0.4098822897,
-         0.0193317049,
-        -0.0039145680,
-        -0.0069438567
+        -0.0041048584f,         /* Filter 2 */
+        -0.0093040596f,
+        -0.0134459768f,
+        -0.0048558766f,
+         0.6141017035f,
+         0.4098822897f,
+         0.0193317049f,
+        -0.0039145680f,
+        -0.0069438567f
     },
     {
-        -0.0036565006,          /* Filter 3 */
-        -0.0095231635,
-        -0.0156437084,
-        -0.0127148737,
-         0.6656848457,
-         0.3609830295,
-         0.0233320755,
-        -0.0015677363,
-        -0.0060557371
+        -0.0036565006f,         /* Filter 3 */
+        -0.0095231635f,
+        -0.0156437084f,
+        -0.0127148737f,
+         0.6656848457f,
+         0.3609830295f,
+         0.0233320755f,
+        -0.0015677363f,
+        -0.0060557371f
     },
     {
-        -0.0031253709,          /* Filter 4 */
-        -0.0095729633,
-        -0.0176768181,
-        -0.0211485021,
-         0.7167894869,
-         0.3135419896,
-         0.0264748749,
-         0.0006824956,
-        -0.0050839319
+        -0.0031253709f,         /* Filter 4 */
+        -0.0095729633f,
+        -0.0176768181f,
+        -0.0211485021f,
+         0.7167894869f,
+         0.3135419896f,
+         0.0264748749f,
+         0.0006824956f,
+        -0.0050839319f
     },
     {
-        -0.0025197700,          /* Filter 5 */
-        -0.0094478866,
-        -0.0195012095,
-        -0.0300535107,
-         0.7670600056,
-         0.2678225635,
-         0.0287663895,
-         0.0027999985,
-        -0.0040483891
+        -0.0025197700f,         /* Filter 5 */
+        -0.0094478866f,
+        -0.0195012095f,
+        -0.0300535107f,
+         0.7670600056f,
+         0.2678225635f,
+         0.0287663895f,
+         0.0027999985f,
+        -0.0040483891f
     },
     {
-        -0.0018496023,          /* Filter 6 */
-        -0.0091454978,
-        -0.0210748106,
-        -0.0393111426,
-         0.8161399423,
-         0.2240649005,
-         0.0302262769,
-         0.0047523617,
-        -0.0029696854
+        -0.0018496023f,         /* Filter 6 */
+        -0.0091454978f,
+        -0.0210748106f,
+        -0.0393111426f,
+         0.8161399423f,
+         0.2240649005f,
+         0.0302262769f,
+         0.0047523617f,
+        -0.0029696854f
     },
     {
-        -0.0011262266,          /* Filter 7 */
-        -0.0086666380,
-        -0.0223584207,
-        -0.0487878398,
-         0.8636754069,
-         0.1824841563,
-         0.0308864956,
-         0.0065113237,
-        -0.0018686358
+        -0.0011262266f,         /* Filter 7 */
+        -0.0086666380f,
+        -0.0223584207f,
+        -0.0487878398f,
+         0.8636754069f,
+         0.1824841563f,
+         0.0308864956f,
+         0.0065113237f,
+        -0.0018686358f
     },
     {
-        -0.0003622774,          /* Filter 8 */
-        -0.0080155088,
-        -0.0233165437,
-        -0.0583361774,
-         0.9093185032,
-         0.1432690480,
-         0.0307901140,
-         0.0080531155,
-        -0.0007659096
+        -0.0003622774f,         /* Filter 8 */
+        -0.0080155088f,
+        -0.0233165437f,
+        -0.0583361774f,
+         0.9093185032f,
+         0.1432690480f,
+         0.0307901140f,
+         0.0080531155f,
+        -0.0007659096f
     },
     {
-         0.0004285425,          /* Filter 9 */
-        -0.0071996967,
-        -0.0239181901,
-        -0.0677960213,
-         0.9527307304,
-         0.1065807242,
-         0.0299900191,
-         0.0093587151,
-         0.0003183408
+         0.0004285425f,         /* Filter 9 */
+        -0.0071996967f,
+        -0.0239181901f,
+        -0.0677960213f,
+         0.9527307304f,
+         0.1065807242f,
+         0.0299900191f,
+         0.0093587151f,
+         0.0003183408f
     },
     {
-         0.0012316933,          /* Filter 10 */
-        -0.0062301368,
-        -0.0241376359,
-        -0.0769959031,
-         0.9935863233,
-         0.0725519600,
-         0.0285475474,
-         0.0104140102,
-         0.0013648323
+         0.0012316933f,         /* Filter 10 */
+        -0.0062301368f,
+        -0.0241376359f,
+        -0.0769959031f,
+         0.9935863233f,
+         0.0725519600f,
+         0.0285475474f,
+         0.0104140102f,
+         0.0013648323f
     },
     {
-         0.0020320508,          /* Filter 11 */
-        -0.0051210137,
-        -0.0239551212,
-        -0.0857546018,
-         1.0315754934,
-         0.0412866769,
-         0.0265310587,
-         0.0112098711,
-         0.0023554782
+         0.0020320508f,         /* Filter 11 */
+        -0.0051210137f,
+        -0.0239551212f,
+        -0.0857546018f,
+         1.0315754934f,
+         0.0412866769f,
+         0.0265310587f,
+         0.0112098711f,
+         0.0023554782f
     },
     {
-         0.0028141763,          /* Filter 12 */
-        -0.0038896008,
-        -0.0233574765,
-        -0.0938829156,
-         1.0664075323,
-         0.0128597894,
-         0.0240144782,
-         0.0117421344,
-         0.0032736852
+         0.0028141763f,         /* Filter 12 */
+        -0.0038896008f,
+        -0.0233574765f,
+        -0.0938829156f,
+         1.0664075323f,
+         0.0128597894f,
+         0.0240144782f,
+         0.0117421344f,
+         0.0032736852f
     },
     {
-         0.0035625973,          /* Filter 13 */
-        -0.0025560369,
-        -0.0223386620,
-        -0.1011856112,
-         1.0978137424,
-        -0.0126826277,
-         0.0210758250,
-         0.0120115019,
-         0.0041046165
+         0.0035625973f,         /* Filter 13 */
+        -0.0025560369f,
+        -0.0223386620f,
+        -0.1011856112f,
+         1.0978137424f,
+        -0.0126826277f,
+         0.0210758250f,
+         0.0120115019f,
+         0.0041046165f
     },
     {
-         0.0042620971,          /* Filter 14 */
-        -0.0011430452,
-        -0.0209002083,
-        -0.1074635265,
-         1.1255501596,
-        -0.0353228594,
-         0.0177957527,
-         0.0120233576,
-         0.0048354157
+         0.0042620971f,         /* Filter 14 */
+        -0.0011430452f,
+        -0.0209002083f,
+        -0.1074635265f,
+         1.1255501596f,
+        -0.0353228594f,
+         0.0177957527f,
+         0.0120233576f,
+         0.0048354157f
     },
     {
-         0.0048980053,          /* Filter 15 */
-         0.0003244045,
-        -0.0190515462,
-        -0.1125158080,
-         1.1494000377,
-        -0.0550707703,
-         0.0142561191,
-         0.0117875105,
-         0.0054553898
+         0.0048980053f,         /* Filter 15 */
+         0.0003244045f,
+        -0.0190515462f,
+        -0.1125158080f,
+         1.1494000377f,
+        -0.0550707703f,
+         0.0142561191f,
+         0.0117875105f,
+         0.0054553898f
     },
     {
-         0.0054564857,          /* Filter 16 */
-         0.0018194852,
-        -0.0168102168,
-        -0.1161422557,
-         1.1691760633,
-        -0.0719627404,
-         0.0105386068,
-         0.0113178688,
-         0.0059561483
+         0.0054564857f,         /* Filter 16 */
+         0.0018194852f,
+        -0.0168102168f,
+        -0.1161422557f,
+         1.1691760633f,
+        -0.0719627404f,
+         0.0105386068f,
+         0.0113178688f,
+         0.0059561483f
     },
     {
-         0.0059248154,          /* Filter 17 */
-         0.0033139475,
-        -0.0142019526,
-        -0.1181457502,
-         1.1847222770,
-        -0.0860603350,
-         0.0067234125,
-         0.0106320540,
-         0.0063316975
+         0.0059248154f,         /* Filter 17 */
+         0.0033139475f,
+        -0.0142019526f,
+        -0.1181457502f,
+         1.1847222770f,
+        -0.0860603350f,
+         0.0067234125f,
+         0.0106320540f,
+         0.0063316975f
     },
     {
-         0.0062916504,          /* Filter 18 */
-         0.0047785946,
-        -0.0112606234,
-        -0.1183347329,
-         1.1959156771,
-        -0.0974487311,
-         0.0028880206,
-         0.0097509621,
-         0.0065784888
+         0.0062916504f,         /* Filter 18 */
+         0.0047785946f,
+        -0.0112606234f,
+        -0.1183347329f,
+         1.1959156771f,
+        -0.0974487311f,
+         0.0028880206f,
+         0.0097509621f,
+         0.0065784888f
     },
     {
-         0.0065472715,          /* Filter 19 */
-         0.0061837898,
-        -0.0080280420,
-        -0.1165257094,
-         1.2026674866,
-        -0.1062349247,
-        -0.0008939235,
-         0.0086982833,
-         0.0066954225
+         0.0065472715f,         /* Filter 19 */
+         0.0061837898f,
+        -0.0080280420f,
+        -0.1165257094f,
+         1.2026674866f,
+        -0.1062349247f,
+        -0.0008939235f,
+         0.0086982833f,
+         0.0066954225f
     },
     {
-         0.0066838062,          /* Filter 20 */
-         0.0074999881,
-        -0.0045536271,
-        -0.1125457458,
-         1.2049240699,
-        -0.1125457458,
-        -0.0045536271,
-         0.0074999881,
-         0.0066838062
+         0.0066838062f,         /* Filter 20 */
+         0.0074999881f,
+        -0.0045536271f,
+        -0.1125457458f,
+         1.2049240699f,
+        -0.1125457458f,
+        -0.0045536271f,
+         0.0074999881f,
+         0.0066838062f
     },
     {
-         0.0066954225,          /* Filter 21 */
-         0.0086982833,
-        -0.0008939235,
-        -0.1062349247,
-         1.2026674866,
-        -0.1165257094,
-        -0.0080280420,
-         0.0061837898,
-         0.0065472715
+         0.0066954225f,         /* Filter 21 */
+         0.0086982833f,
+        -0.0008939235f,
+        -0.1062349247f,
+         1.2026674866f,
+        -0.1165257094f,
+        -0.0080280420f,
+         0.0061837898f,
+         0.0065472715f
     },
     {
-         0.0065784888,          /* Filter 22 */
-         0.0097509621,
-         0.0028880206,
-        -0.0974487311,
-         1.1959156771,
-        -0.1183347329,
-        -0.0112606234,
-         0.0047785946,
-         0.0062916504
+         0.0065784888f,         /* Filter 22 */
+         0.0097509621f,
+         0.0028880206f,
+        -0.0974487311f,
+         1.1959156771f,
+        -0.1183347329f,
+        -0.0112606234f,
+         0.0047785946f,
+         0.0062916504f
     },
     {
-         0.0063316975,          /* Filter 23 */
-         0.0106320540,
-         0.0067234125,
-        -0.0860603350,
-         1.1847222770,
-        -0.1181457502,
-        -0.0142019526,
-         0.0033139475,
-         0.0059248154
+         0.0063316975f,         /* Filter 23 */
+         0.0106320540f,
+         0.0067234125f,
+        -0.0860603350f,
+         1.1847222770f,
+        -0.1181457502f,
+        -0.0142019526f,
+         0.0033139475f,
+         0.0059248154f
     },
     {
-         0.0059561483,          /* Filter 24 */
-         0.0113178688,
-         0.0105386068,
-        -0.0719627404,
-         1.1691760633,
-        -0.1161422557,
-        -0.0168102168,
-         0.0018194852,
-         0.0054564857
+         0.0059561483f,         /* Filter 24 */
+         0.0113178688f,
+         0.0105386068f,
+        -0.0719627404f,
+         1.1691760633f,
+        -0.1161422557f,
+        -0.0168102168f,
+         0.0018194852f,
+         0.0054564857f
     },
     {
-         0.0054553898,          /* Filter 25 */
-         0.0117875105,
-         0.0142561191,
-        -0.0550707703,
-         1.1494000377,
-        -0.1125158080,
-        -0.0190515462,
-         0.0003244045,
-         0.0048980053
+         0.0054553898f,         /* Filter 25 */
+         0.0117875105f,
+         0.0142561191f,
+        -0.0550707703f,
+         1.1494000377f,
+        -0.1125158080f,
+        -0.0190515462f,
+         0.0003244045f,
+         0.0048980053f
     },
     {
-         0.0048354157,          /* Filter 26 */
-         0.0120233576,
-         0.0177957527,
-        -0.0353228594,
-         1.1255501596,
-        -0.1074635265,
-        -0.0209002083,
-        -0.0011430452,
-         0.0042620971
+         0.0048354157f,         /* Filter 26 */
+         0.0120233576f,
+         0.0177957527f,
+        -0.0353228594f,
+         1.1255501596f,
+        -0.1074635265f,
+        -0.0209002083f,
+        -0.0011430452f,
+         0.0042620971f
     },
     {
-         0.0041046165,          /* Filter 27 */
-         0.0120115019,
-         0.0210758250,
-        -0.0126826277,
-         1.0978137424,
-        -0.1011856112,
-        -0.0223386620,
-        -0.0025560369,
-         0.0035625973
+         0.0041046165f,         /* Filter 27 */
+         0.0120115019f,
+         0.0210758250f,
+        -0.0126826277f,
+         1.0978137424f,
+        -0.1011856112f,
+        -0.0223386620f,
+        -0.0025560369f,
+         0.0035625973f
     },
     {
-         0.0032736852,          /* Filter 28 */
-         0.0117421344,
-         0.0240144782,
-         0.0128597894,
-         1.0664075323,
-        -0.0938829156,
-        -0.0233574765,
-        -0.0038896008,
-         0.0028141763
+         0.0032736852f,         /* Filter 28 */
+         0.0117421344f,
+         0.0240144782f,
+         0.0128597894f,
+         1.0664075323f,
+        -0.0938829156f,
+        -0.0233574765f,
+        -0.0038896008f,
+         0.0028141763f
     },
     {
-         0.0023554782,          /* Filter 29 */
-         0.0112098711,
-         0.0265310587,
-         0.0412866769,
-         1.0315754934,
-        -0.0857546018,
-        -0.0239551212,
-        -0.0051210137,
-         0.0020320508
+         0.0023554782f,         /* Filter 29 */
+         0.0112098711f,
+         0.0265310587f,
+         0.0412866769f,
+         1.0315754934f,
+        -0.0857546018f,
+        -0.0239551212f,
+        -0.0051210137f,
+         0.0020320508f
     },
     {
-         0.0013648323,          /* Filter 30 */
-         0.0104140102,
-         0.0285475474,
-         0.0725519600,
-         0.9935863233,
-        -0.0769959031,
-        -0.0241376359,
-        -0.0062301368,
-         0.0012316933
+         0.0013648323f,         /* Filter 30 */
+         0.0104140102f,
+         0.0285475474f,
+         0.0725519600f,
+         0.9935863233f,
+        -0.0769959031f,
+        -0.0241376359f,
+        -0.0062301368f,
+         0.0012316933f
     },
     {
-         0.0003183408,          /* Filter 31 */
-         0.0093587151,
-         0.0299900191,
-         0.1065807242,
-         0.9527307304,
-        -0.0677960213,
-        -0.0239181901,
-        -0.0071996967,
-         0.0004285425
+         0.0003183408f,         /* Filter 31 */
+         0.0093587151f,
+         0.0299900191f,
+         0.1065807242f,
+         0.9527307304f,
+        -0.0677960213f,
+        -0.0239181901f,
+        -0.0071996967f,
+         0.0004285425f
     },
     {
-        -0.0007659096,          /* Filter 32 */
-         0.0080531155,
-         0.0307901140,
-         0.1432690480,
-         0.9093185032,
-        -0.0583361774,
-        -0.0233165437,
-        -0.0080155088,
-        -0.0003622774
+        -0.0007659096f,         /* Filter 32 */
+         0.0080531155f,
+         0.0307901140f,
+         0.1432690480f,
+         0.9093185032f,
+        -0.0583361774f,
+        -0.0233165437f,
+        -0.0080155088f,
+        -0.0003622774f
     },
     {
-        -0.0018686358,          /* Filter 33 */
-         0.0065113237,
-         0.0308864956,
-         0.1824841563,
-         0.8636754069,
-        -0.0487878398,
-        -0.0223584207,
-        -0.0086666380,
-        -0.0011262266
+        -0.0018686358f,         /* Filter 33 */
+         0.0065113237f,
+         0.0308864956f,
+         0.1824841563f,
+         0.8636754069f,
+        -0.0487878398f,
+        -0.0223584207f,
+        -0.0086666380f,
+        -0.0011262266f
     },
     {
-        -0.0029696854,          /* Filter 34 */
-         0.0047523617,
-         0.0302262769,
-         0.2240649005,
-         0.8161399423,
-        -0.0393111426,
-        -0.0210748106,
-        -0.0091454978,
-        -0.0018496023
+        -0.0029696854f,         /* Filter 34 */
+         0.0047523617f,
+         0.0302262769f,
+         0.2240649005f,
+         0.8161399423f,
+        -0.0393111426f,
+        -0.0210748106f,
+        -0.0091454978f,
+        -0.0018496023f
     },
     {
-        -0.0040483891,          /* Filter 35 */
-         0.0027999985,
-         0.0287663895,
-         0.2678225635,
-         0.7670600056,
-        -0.0300535107,
-        -0.0195012095,
-        -0.0094478866,
-        -0.0025197700
+        -0.0040483891f,         /* Filter 35 */
+         0.0027999985f,
+         0.0287663895f,
+         0.2678225635f,
+         0.7670600056f,
+        -0.0300535107f,
+        -0.0195012095f,
+        -0.0094478866f,
+        -0.0025197700f
     },
     {
-        -0.0050839319,          /* Filter 36 */
-         0.0006824956,
-         0.0264748749,
-         0.3135419896,
-         0.7167894869,
-        -0.0211485021,
-        -0.0176768181,
-        -0.0095729633,
-        -0.0031253709
+        -0.0050839319f,         /* Filter 36 */
+         0.0006824956f,
+         0.0264748749f,
+         0.3135419896f,
+         0.7167894869f,
+        -0.0211485021f,
+        -0.0176768181f,
+        -0.0095729633f,
+        -0.0031253709f
     },
     {
-        -0.0060557371,          /* Filter 37 */
-        -0.0015677363,
-         0.0233320755,
-         0.3609830295,
-         0.6656848457,
-        -0.0127148737,
-        -0.0156437084,
-        -0.0095231635,
-        -0.0036565006
+        -0.0060557371f,         /* Filter 37 */
+        -0.0015677363f,
+         0.0233320755f,
+         0.3609830295f,
+         0.6656848457f,
+        -0.0127148737f,
+        -0.0156437084f,
+        -0.0095231635f,
+        -0.0036565006f
     },
     {
-        -0.0069438567,          /* Filter 38 */
-        -0.0039145680,
-         0.0193317049,
-         0.4098822897,
-         0.6141017035,
-        -0.0048558766,
-        -0.0134459768,
-        -0.0093040596,
-        -0.0041048584
+        -0.0069438567f,         /* Filter 38 */
+        -0.0039145680f,
+         0.0193317049f,
+         0.4098822897f,
+         0.6141017035f,
+        -0.0048558766f,
+        -0.0134459768f,
+        -0.0093040596f,
+        -0.0041048584f
     },
     {
-        -0.0077293609,          /* Filter 39 */
-        -0.0063186648,
-         0.0144817755,
-         0.4599551720,
-         0.5623914901,
-         0.0023412184,
-        -0.0111288952,
-        -0.0089241700,
-        -0.0044638629
+        -0.0077293609f,         /* Filter 39 */
+        -0.0063186648f,
+         0.0144817755f,
+         0.4599551720f,
+         0.5623914901f,
+         0.0023412184f,
+        -0.0111288952f,
+        -0.0089241700f,
+        -0.0044638629f
     },
 };
 
@@ -978,8 +983,7 @@ int v22bis_tx(v22bis_state_t *s, int16_t amp[], int len)
                 s->tx_rrc_filter_step = 0;
         }
         /* Root raised cosine pulse shaping at baseband */
-        x.re = 0.0f;
-        x.im = 0.0f;
+        x = complex_setf(0.0f, 0.0f);
         for (i = 0;  i < V22BIS_TX_FILTER_STEPS;  i++)
         {
             x.re += pulseshaper[39 - s->tx_baud_phase][i]*s->tx_rrc_filter[i + s->tx_rrc_filter_step].re;
@@ -994,7 +998,7 @@ int v22bis_tx(v22bis_state_t *s, int16_t amp[], int len)
             famp += dds_modf(&(s->guard_phase), s->guard_phase_rate, s->guard_level, 0);
         }
         /* Don't bother saturating. We should never clip. */
-        amp[sample] = (int16_t) lrintf(famp);
+        amp[sample] = (int16_t) rintf(famp);
     }
     return sample;
 }
