@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_rx.c,v 1.76 2007/04/10 16:12:20 steveu Exp $
+ * $Id: v27ter_rx.c,v 1.78 2007/06/29 12:56:28 steveu Exp $
  */
 
 /*! \file */
@@ -1181,6 +1181,7 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
     int i;
     int j;
     int step;
+    int16_t x;
     complexf_t z;
     complexf_t zz;
     complexf_t samplex;
@@ -1198,13 +1199,14 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
             /* There should be no DC in the signal, but sometimes there is.
                We need to measure the power with the DC blocked, but not using
                a slow to respond DC blocker. Use the most elementary HPF. */
-            power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
-            s->last_sample = amp[i];
+            x = amp[i] >> 1;
+            power = power_meter_update(&(s->power), x - s->last_sample);
+            s->last_sample = x;
             //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_current_dbm0(&(s->power)));
             if (s->signal_present)
             {
                 /* Look for power below turnoff threshold to turn the carrier off */
-                if (power < s->carrier_off_power)
+                if (s->carrier_drop_pending || power < s->carrier_off_power)
                 {
                     if (--s->signal_present <= 0)
                     {
@@ -1214,6 +1216,12 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                         s->put_bit(s->user_data, PUTBIT_CARRIER_DOWN);
                         continue;
                     }
+                    else
+                    {
+                        /* Carrier has dropped, but the put_bit is
+                           pending the signal_present delay. */
+                        s->carrier_drop_pending = 1;
+                    }
                 }
             }
             else
@@ -1222,47 +1230,47 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                 if (power < s->carrier_on_power)
                     continue;
                 s->signal_present = 1;
+                s->carrier_drop_pending = 0;
                 s->put_bit(s->user_data, PUTBIT_CARRIER_UP);
             }
-            if (s->training_stage != TRAINING_STAGE_PARKED)
-            {
-                /* Only spend effort processing this data if the modem is not
-                   parked, after training failure. */
-                z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
+            if (s->training_stage == TRAINING_STAGE_PARKED)
+                continue;
+            /* Only spend effort processing this data if the modem is not
+               parked, after training failure. */
+            z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
         
-                /* Put things into the equalization buffer at T/2 rate. The Gardner algorithm
-                   will fiddle the step to align this with the symbols. */
-                if ((s->eq_put_step -= PULSESHAPER_4800_COEFF_SETS) <= 0)
+            /* Put things into the equalization buffer at T/2 rate. The Gardner algorithm
+               will fiddle the step to align this with the symbols. */
+            if ((s->eq_put_step -= PULSESHAPER_4800_COEFF_SETS) <= 0)
+            {
+                if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
                 {
-                    if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
-                    {
-                        /* Only AGC during the initial training */
-                        s->agc_scaling = (1.0f/PULSESHAPER_4800_GAIN)*1.414f/sqrtf(power);
-                    }
-                    /* Pulse shape while still at the carrier frequency, using a quadrature
-                       pair of filters. This results in a properly bandpass filtered complex
-                       signal, which can be brought directly to bandband by complex mixing.
-                       No further filtering, to remove mixer harmonics, is needed. */
-                    step = -s->eq_put_step;
-                    if (step > PULSESHAPER_4800_COEFF_SETS - 1)
-                        step = PULSESHAPER_4800_COEFF_SETS - 1;
-                    s->eq_put_step += PULSESHAPER_4800_COEFF_SETS*5/2;
-                    zz.re = pulseshaper_4800[step][0].re*s->rrc_filter[s->rrc_filter_step];
-                    zz.im = pulseshaper_4800[step][0].im*s->rrc_filter[s->rrc_filter_step];
-                    for (j = 1;  j < V27TER_RX_4800_FILTER_STEPS;  j++)
-                    {
-                        zz.re += pulseshaper_4800[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
-                        zz.im += pulseshaper_4800[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
-                    }
-                    samplex.re = zz.re*s->agc_scaling;
-                    samplex.im = zz.im*s->agc_scaling;
-                    /* Shift to baseband - since this is done in a full complex form, the
-                       result is clean, and requires no further filtering, apart from the
-                       equalizer. */
-                    zz.re = samplex.re*z.re - samplex.im*z.im;
-                    zz.im = -samplex.re*z.im - samplex.im*z.re;
-                    process_half_baud(s, &zz);
+                    /* Only AGC during the initial training */
+                    s->agc_scaling = (1.0f/PULSESHAPER_4800_GAIN)*1.414f/sqrtf(power);
                 }
+                /* Pulse shape while still at the carrier frequency, using a quadrature
+                   pair of filters. This results in a properly bandpass filtered complex
+                   signal, which can be brought directly to bandband by complex mixing.
+                   No further filtering, to remove mixer harmonics, is needed. */
+                step = -s->eq_put_step;
+                if (step > PULSESHAPER_4800_COEFF_SETS - 1)
+                    step = PULSESHAPER_4800_COEFF_SETS - 1;
+                s->eq_put_step += PULSESHAPER_4800_COEFF_SETS*5/2;
+                zz.re = pulseshaper_4800[step][0].re*s->rrc_filter[s->rrc_filter_step];
+                zz.im = pulseshaper_4800[step][0].im*s->rrc_filter[s->rrc_filter_step];
+                for (j = 1;  j < V27TER_RX_4800_FILTER_STEPS;  j++)
+                {
+                    zz.re += pulseshaper_4800[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
+                    zz.im += pulseshaper_4800[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
+                }
+                samplex.re = zz.re*s->agc_scaling;
+                samplex.im = zz.im*s->agc_scaling;
+                /* Shift to baseband - since this is done in a full complex form, the
+                   result is clean, and requires no further filtering, apart from the
+                   equalizer. */
+                zz.re = samplex.re*z.re - samplex.im*z.im;
+                zz.im = -samplex.re*z.im - samplex.im*z.re;
+                process_half_baud(s, &zz);
             }
         }
     }
@@ -1278,13 +1286,14 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
             /* There should be no DC in the signal, but sometimes there is.
                We need to measure the power with the DC blocked, but not using
                a slow to respond DC blocker. Use the most elementary HPF. */
-            power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
-            s->last_sample = amp[i];
+            x = amp[i] >> 1;
+            power = power_meter_update(&(s->power), x - s->last_sample);
+            s->last_sample = x;
             //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_current_dbm0(&(s->power)));
             if (s->signal_present)
             {
                 /* Look for power below turnoff threshold to turn the carrier off */
-                if (power < s->carrier_off_power)
+                if (s->carrier_drop_pending || power < s->carrier_off_power)
                 {
                     if (--s->signal_present <= 0)
                     {
@@ -1294,6 +1303,12 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                         s->put_bit(s->user_data, PUTBIT_CARRIER_DOWN);
                         continue;
                     }
+                    else
+                    {
+                        /* Carrier has dropped, but the put_bit is
+                           pending the signal_present delay. */
+                        s->carrier_drop_pending = 1;
+                    }
                 }
             }
             else
@@ -1302,47 +1317,47 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                 if (power < s->carrier_on_power)
                     continue;
                 s->signal_present = 1;
+                s->carrier_drop_pending = 0;
                 s->put_bit(s->user_data, PUTBIT_CARRIER_UP);
             }
-            if (s->training_stage != TRAINING_STAGE_PARKED)
-            {
-                /* Only spend effort processing this data if the modem is not
-                   parked, after training failure. */
-                z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
+            if (s->training_stage == TRAINING_STAGE_PARKED)
+                continue;
+            /* Only spend effort processing this data if the modem is not
+               parked, after training failure. */
+            z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
         
-                /* Put things into the equalization buffer at T/2 rate. The Gardner algorithm
-                   will fiddle the step to align this with the symbols. */
-                if ((s->eq_put_step -= PULSESHAPER_2400_COEFF_SETS) <= 0)
+            /* Put things into the equalization buffer at T/2 rate. The Gardner algorithm
+               will fiddle the step to align this with the symbols. */
+            if ((s->eq_put_step -= PULSESHAPER_2400_COEFF_SETS) <= 0)
+            {
+                if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
                 {
-                    if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
-                    {
-                        /* Only AGC during the initial training */
-                        s->agc_scaling = (1.0f/PULSESHAPER_2400_GAIN)*1.414f/sqrtf(power);
-                    }
-                    /* Pulse shape while still at the carrier frequency, using a quadrature
-                       pair of filters. This results in a properly bandpass filtered complex
-                       signal, which can be brought directly to bandband by complex mixing.
-                       No further filtering, to remove mixer harmonics, is needed. */
-                    step = -s->eq_put_step;
-                    if (step > PULSESHAPER_2400_COEFF_SETS - 1)
-                        step = PULSESHAPER_2400_COEFF_SETS - 1;
-                    s->eq_put_step += PULSESHAPER_2400_COEFF_SETS*20/(3*2);
-                    zz.re = pulseshaper_2400[step][0].re*s->rrc_filter[s->rrc_filter_step];
-                    zz.im = pulseshaper_2400[step][0].im*s->rrc_filter[s->rrc_filter_step];
-                    for (j = 1;  j < V27TER_RX_2400_FILTER_STEPS;  j++)
-                    {
-                        zz.re += pulseshaper_2400[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
-                        zz.im += pulseshaper_2400[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
-                    }
-                    samplex.re = zz.re*s->agc_scaling;
-                    samplex.im = zz.im*s->agc_scaling;
-                    /* Shift to baseband - since this is done in a full complex form, the
-                       result is clean, and requires no further filtering apart from the
-                       equalizer. */
-                    zz.re = samplex.re*z.re - samplex.im*z.im;
-                    zz.im = -samplex.re*z.im - samplex.im*z.re;
-                    process_half_baud(s, &zz);
+                    /* Only AGC during the initial training */
+                    s->agc_scaling = (1.0f/PULSESHAPER_2400_GAIN)*1.414f/sqrtf(power);
                 }
+                /* Pulse shape while still at the carrier frequency, using a quadrature
+                   pair of filters. This results in a properly bandpass filtered complex
+                   signal, which can be brought directly to bandband by complex mixing.
+                   No further filtering, to remove mixer harmonics, is needed. */
+                step = -s->eq_put_step;
+                if (step > PULSESHAPER_2400_COEFF_SETS - 1)
+                    step = PULSESHAPER_2400_COEFF_SETS - 1;
+                s->eq_put_step += PULSESHAPER_2400_COEFF_SETS*20/(3*2);
+                zz.re = pulseshaper_2400[step][0].re*s->rrc_filter[s->rrc_filter_step];
+                zz.im = pulseshaper_2400[step][0].im*s->rrc_filter[s->rrc_filter_step];
+                for (j = 1;  j < V27TER_RX_2400_FILTER_STEPS;  j++)
+                {
+                    zz.re += pulseshaper_2400[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
+                    zz.im += pulseshaper_2400[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
+                }
+                samplex.re = zz.re*s->agc_scaling;
+                samplex.im = zz.im*s->agc_scaling;
+                /* Shift to baseband - since this is done in a full complex form, the
+                   result is clean, and requires no further filtering apart from the
+                   equalizer. */
+                zz.re = samplex.re*z.re - samplex.im*z.im;
+                zz.im = -samplex.re*z.im - samplex.im*z.re;
+                process_half_baud(s, &zz);
             }
         }
     }
@@ -1374,6 +1389,7 @@ int v27ter_rx_restart(v27ter_rx_state_t *s, int rate, int old_train)
     s->training_count = 0;
     s->training_error = 0.0f;
     s->signal_present = 0;
+    s->carrier_drop_pending = 0;
 
     s->carrier_phase = 0;
     //s->carrier_track_i = 100000.0f;
