@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29rx.c,v 1.102 2007/04/10 16:12:20 steveu Exp $
+ * $Id: v29rx.c,v 1.104 2007/06/29 12:56:28 steveu Exp $
  */
 
 /*! \file */
@@ -1917,7 +1917,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
                 ||
                 s->carrier_phase_rate > dds_phase_ratef(CARRIER_NOMINAL_FREQ + 20.0f))
             {
-               span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
+               span_log(&s->logging, SPAN_LOG_FLOW, "Training failed 1 (sequence failed)\n");
                /* Park this modem */
                s->training_stage = TRAINING_STAGE_PARKED;
                s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
@@ -1940,7 +1940,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
         {
             /* This is bogus. There are not this many bauds in this section
                of a real training sequence. */
-            span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
+            span_log(&s->logging, SPAN_LOG_FLOW, "Training failed 2 (sequence failed)\n");
             /* Park this modem */
             s->training_stage = TRAINING_STAGE_PARKED;
             s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
@@ -2044,6 +2044,7 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
     int i;
     int j;
     int step;
+    int16_t x;
     complexf_t z;
     complexf_t zz;
     complexf_t sample;
@@ -2060,12 +2061,13 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
         /* There should be no DC in the signal, but sometimes there is.
            We need to measure the power with the DC blocked, but not using
            a slow to respond DC blocker. Use the most elementary HPF. */
-        power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
-        s->last_sample = amp[i];
+        x = amp[i] >> 1;
+        power = power_meter_update(&(s->power), x - s->last_sample);
+        s->last_sample = x;
         if (s->signal_present)
         {
             /* Look for power below turn-off threshold to turn the carrier off */
-            if (power < s->carrier_off_power)
+            if (s->carrier_drop_pending || power < s->carrier_off_power)
             {
                 if (--s->signal_present <= 0)
                 {
@@ -2075,6 +2077,12 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
                     s->put_bit(s->user_data, PUTBIT_CARRIER_DOWN);
                     continue;
                 }
+                else
+                {
+                    /* Carrier has dropped, but the put_bit is
+                       pending the signal_present delay. */
+                    s->carrier_drop_pending = 1;
+                }
             }
         }
         else
@@ -2083,64 +2091,64 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
             if (power < s->carrier_on_power)
                 continue;
             s->signal_present = 1;
+            s->carrier_drop_pending = 0;
             s->put_bit(s->user_data, PUTBIT_CARRIER_UP);
         }
-        if (s->training_stage != TRAINING_STAGE_PARKED)
+        if (s->training_stage == TRAINING_STAGE_PARKED)
+            continue;
+        /* Only spend effort processing this data if the modem is not
+           parked, after training failure. */
+        s->eq_put_step -= PULSESHAPER_COEFF_SETS;
+        step = -s->eq_put_step;
+        if (step > PULSESHAPER_COEFF_SETS - 1)
+            step = PULSESHAPER_COEFF_SETS - 1;
+        if (step < 0)
+            step += PULSESHAPER_COEFF_SETS;
+        zz.re = pulseshaper[step][0].re*s->rrc_filter[s->rrc_filter_step];
+        for (j = 1;  j < V29_RX_FILTER_STEPS;  j++)
+            zz.re += pulseshaper[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
+        sample.re = zz.re*s->agc_scaling;
+
+        /* Symbol timing synchronisation band edge filters */
+        /* Low Nyquist band edge filter */
+        v = s->symbol_sync_low[0]*SYNC_LOW_BAND_EDGE_COEFF_0 + s->symbol_sync_low[1]*SYNC_LOW_BAND_EDGE_COEFF_1 + sample.re;
+        s->symbol_sync_low[1] = s->symbol_sync_low[0];
+        s->symbol_sync_low[0] = v;
+        /* High Nyquist band edge filter */
+        v = s->symbol_sync_high[0]*SYNC_HIGH_BAND_EDGE_COEFF_0 + s->symbol_sync_high[1]*SYNC_HIGH_BAND_EDGE_COEFF_1 + sample.re;
+        s->symbol_sync_high[1] = s->symbol_sync_high[0];
+        s->symbol_sync_high[0] = v;
+
+        z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
+
+        /* Put things into the equalization buffer at T/2 rate. The symbol synchronisation
+           will fiddle the step to align this with the symbols. */
+        if (s->eq_put_step <= 0)
         {
-            /* Only spend effort processing this data if the modem is not
-               parked, after training failure. */
-            s->eq_put_step -= PULSESHAPER_COEFF_SETS;
+            if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
+            {
+                /* Only AGC during the initial training */
+                s->agc_scaling = (1.0f/PULSESHAPER_GAIN)*5.0f*0.72f/sqrtf(power);
+            }
+            /* Pulse shape while still at the carrier frequency, using a quadrature
+               pair of filters. This results in a properly bandpass filtered complex
+               signal, which can be brought directly to bandband by complex mixing.
+               No further filtering, to remove mixer harmonics, is needed. */
             step = -s->eq_put_step;
             if (step > PULSESHAPER_COEFF_SETS - 1)
                 step = PULSESHAPER_COEFF_SETS - 1;
-            if (step < 0)
-                step += PULSESHAPER_COEFF_SETS;
-            zz.re = pulseshaper[step][0].re*s->rrc_filter[s->rrc_filter_step];
+            zz.im = pulseshaper[step][0].im*s->rrc_filter[s->rrc_filter_step];
             for (j = 1;  j < V29_RX_FILTER_STEPS;  j++)
-                zz.re += pulseshaper[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
+                zz.im += pulseshaper[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
             sample.re = zz.re*s->agc_scaling;
-
-            /* Symbol timing synchronisation band edge filters */
-            /* Low Nyquist band edge filter */
-            v = s->symbol_sync_low[0]*SYNC_LOW_BAND_EDGE_COEFF_0 + s->symbol_sync_low[1]*SYNC_LOW_BAND_EDGE_COEFF_1 + sample.re;
-            s->symbol_sync_low[1] = s->symbol_sync_low[0];
-            s->symbol_sync_low[0] = v;
-            /* High Nyquist band edge filter */
-            v = s->symbol_sync_high[0]*SYNC_HIGH_BAND_EDGE_COEFF_0 + s->symbol_sync_high[1]*SYNC_HIGH_BAND_EDGE_COEFF_1 + sample.re;
-            s->symbol_sync_high[1] = s->symbol_sync_high[0];
-            s->symbol_sync_high[0] = v;
-
-            z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
-
-            /* Put things into the equalization buffer at T/2 rate. The symbol synchronisation
-               will fiddle the step to align this with the symbols. */
-            if (s->eq_put_step <= 0)
-            {
-                if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
-                {
-                    /* Only AGC during the initial training */
-                    s->agc_scaling = (1.0f/PULSESHAPER_GAIN)*5.0f*0.72f/sqrtf(power);
-                }
-                /* Pulse shape while still at the carrier frequency, using a quadrature
-                   pair of filters. This results in a properly bandpass filtered complex
-                   signal, which can be brought directly to bandband by complex mixing.
-                   No further filtering, to remove mixer harmonics, is needed. */
-                step = -s->eq_put_step;
-                if (step > PULSESHAPER_COEFF_SETS - 1)
-                    step = PULSESHAPER_COEFF_SETS - 1;
-                zz.im = pulseshaper[step][0].im*s->rrc_filter[s->rrc_filter_step];
-                for (j = 1;  j < V29_RX_FILTER_STEPS;  j++)
-                    zz.im += pulseshaper[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
-                sample.re = zz.re*s->agc_scaling;
-                sample.im = zz.im*s->agc_scaling;
-                s->eq_put_step += PULSESHAPER_COEFF_SETS*10/(3*2);
-                /* Shift to baseband - since this is done in a full complex form, the
-                   result is clean, and requires no further filtering, apart from the
-                   equalizer. */
-                zz.re = sample.re*z.re - sample.im*z.im;
-                zz.im = -sample.re*z.im - sample.im*z.re;
-                process_half_baud(s, &zz);
-            }
+            sample.im = zz.im*s->agc_scaling;
+            s->eq_put_step += PULSESHAPER_COEFF_SETS*10/(3*2);
+            /* Shift to baseband - since this is done in a full complex form, the
+               result is clean, and requires no further filtering, apart from the
+               equalizer. */
+            zz.re = sample.re*z.re - sample.im*z.im;
+            zz.im = -sample.re*z.im - sample.im*z.re;
+            process_half_baud(s, &zz);
         }
     }
     return 0;
@@ -2180,6 +2188,7 @@ int v29_rx_restart(v29_rx_state_t *s, int rate, int old_train)
     s->training_stage = TRAINING_STAGE_SYMBOL_ACQUISITION;
     s->training_count = 0;
     s->signal_present = 0;
+    s->carrier_drop_pending = 0;
     s->old_train = old_train;
 
     s->carrier_phase = 0;
