@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v8.c,v 1.21 2007/04/30 13:13:36 steveu Exp $
+ * $Id: v8.c,v 1.22 2007/06/25 12:55:22 steveu Exp $
  */
  
 /*! \file */
@@ -50,9 +50,9 @@
 #include "spandsp/tone_detect.h"
 #include "spandsp/tone_generate.h"
 #include "spandsp/super_tone_rx.h"
-#include "spandsp/modem_connect_tones.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/fsk.h"
+#include "spandsp/modem_connect_tones.h"
 #include "spandsp/v8.h"
 
 #define ms_to_samples(t)    (((t)*SAMPLE_RATE)/1000)
@@ -486,9 +486,9 @@ static void put_bit(void *user_data, int bit)
 static void v8_decode_init(v8_state_t *s)
 {
     if (s->caller)
-        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH2], FALSE, 0, put_bit, s);
+        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH2], FALSE, put_bit, s);
     else
-        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH1], FALSE, 0, put_bit, s);
+        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH1], FALSE, put_bit, s);
     s->preamble_type = 0;
     s->bit_stream = 0;
     s->cm_jm_count = 0;
