@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: adsi_tests.c,v 1.27 2006/10/24 13:45:28 steveu Exp $
+ * $Id: adsi_tests.c,v 1.32 2007/06/08 13:49:38 steveu Exp $
  */
 
 /*! \page adsi_tests_page ADSI tests
@@ -45,7 +45,12 @@ tests, these tests do not include line modelling.
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
@@ -171,7 +176,25 @@ int adsi_create_message(adsi_tx_state_t *s, uint8_t *msg)
         len = adsi_add_field(s, msg, len, JCLIP_DIALED_NUM_DES, (uint8_t *) "215", 3);
         break;
     case ADSI_STANDARD_CLIP_DTMF:
-        len = adsi_add_field(s, msg, 0, CLIP_DTMF_CALLER_NUMBER, (uint8_t *) "12345678", 8);
+        if (cycle > 2)
+            cycle = 0;
+        switch (cycle)
+        {
+        case 0:
+            len = adsi_add_field(s, msg, -1, CLIP_DTMF_C_TERMINATED, NULL, 0);
+            len = adsi_add_field(s, msg, len, CLIP_DTMF_C_CALLER_NUMBER, (uint8_t *) "12345678", 8);
+            len = adsi_add_field(s, msg, len, CLIP_DTMF_C_ABSENCE, (uint8_t *) "10", 2);
+            len = adsi_add_field(s, msg, len, CLIP_DTMF_C_REDIRECT_NUMBER, (uint8_t *) "87654321", 8);
+            break;
+        case 1:
+            len = adsi_add_field(s, msg, -1, CLIP_DTMF_HASH_TERMINATED, NULL, 0);
+            len = adsi_add_field(s, msg, len, CLIP_DTMF_HASH_CALLER_NUMBER, (uint8_t *) "12345678", 8);
+            break;
+        case 2:
+            len = adsi_add_field(s, msg, -1, CLIP_DTMF_HASH_TERMINATED, NULL, 0);
+            len = adsi_add_field(s, msg, len, CLIP_DTMF_HASH_ABSENCE, (uint8_t *) "1", 1);
+            break;
+        }
         break;
     case ADSI_STANDARD_TDD:
         t = "The quick Brown Fox Jumps Over The Lazy dog 0123456789!@#$%^&*()";
@@ -390,11 +413,30 @@ static void put_adsi_msg(void *user_data, const uint8_t *msg, int len)
                 case ADSI_STANDARD_CLIP_DTMF:
                     switch (message_type)
                     {
-                    case CLIP_DTMF_CALLER_NUMBER:
-                        printf("Caller's number");
+                    case CLIP_DTMF_HASH_TERMINATED:
+                        switch (field_type)
+                        {
+                        case CLIP_DTMF_HASH_CALLER_NUMBER:
+                            printf("Caller's number");
+                            break;
+                        case CLIP_DTMF_HASH_ABSENCE:
+                            printf("Caller's number absent: private (1), overseas (2) or not available (3)");
+                            break;
+                        }
                         break;
-                    case CLIP_DTMF_ABSENCE1:
-                        printf("Caller's number absent: private (1), overseas (2) or not available (3)");
+                    case CLIP_DTMF_C_TERMINATED:
+                        switch (field_type)
+                        {
+                        case CLIP_DTMF_C_CALLER_NUMBER:
+                            printf("Caller's number");
+                            break;
+                        case CLIP_DTMF_C_REDIRECT_NUMBER:
+                            printf("Redirect number");
+                            break;
+                        case CLIP_DTMF_C_ABSENCE:
+                            printf("Caller's number absent");
+                            break;
+                        }
                         break;
                     }
                     break;
@@ -476,11 +518,11 @@ static void put_adsi_msg(void *user_data, const uint8_t *msg, int len)
                 case ADSI_STANDARD_CLIP_DTMF:
                     switch (message_type)
                     {
-                    case CLIP_DTMF_CALLER_NUMBER:
-                        printf("Caller's number");
+                    case CLIP_DTMF_HASH_TERMINATED:
+                        printf("# terminated");
                         break;
-                    case CLIP_DTMF_ABSENCE1:
-                        printf("Caller's number absent");
+                    case CLIP_DTMF_C_TERMINATED:
+                        printf("C terminated");
                         break;
                     default:
                         printf("Unknown");
@@ -517,13 +559,17 @@ int main(int argc, char *argv[])
     int outframes;
     int len;
     int i;
-    int j;
     int push;
     int log_audio;
+    int short_preamble;
+    int test_standard;
+    int first_standard;
+    int last_standard;
 
     log_audio = FALSE;
     decode_test_file = NULL;
-    current_standard = ADSI_STANDARD_CLASS;
+    test_standard = -1;
+    short_preamble = FALSE;
     for (i = 1;  i < argc;  i++)
     {
         if (strcmp(argv[i], "-d") == 0)
@@ -532,23 +578,28 @@ int main(int argc, char *argv[])
             decode_test_file = argv[i];
             continue;
         }
+        if (strcmp(argv[i], "-p") == 0)
+        {
+            short_preamble = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-s") == 0)
         {
             i++;
             if (strcasecmp("CLASS", argv[i]) == 0)
-                current_standard = ADSI_STANDARD_CLASS;
+                test_standard = ADSI_STANDARD_CLASS;
             else if (strcasecmp("CLIP", argv[i]) == 0)
-                current_standard = ADSI_STANDARD_CLIP;
+                test_standard = ADSI_STANDARD_CLIP;
             else if (strcasecmp("A-CLIP", argv[i]) == 0)
-                current_standard = ADSI_STANDARD_ACLIP;
+                test_standard = ADSI_STANDARD_ACLIP;
             else if (strcasecmp("J-CLIP", argv[i]) == 0)
-                current_standard = ADSI_STANDARD_JCLIP;
+                test_standard = ADSI_STANDARD_JCLIP;
             else if (strcasecmp("CLIP-DTMF", argv[i]) == 0)
-                current_standard = ADSI_STANDARD_CLIP_DTMF;
+                test_standard = ADSI_STANDARD_CLIP_DTMF;
             else if (strcasecmp("TDD", argv[i]) == 0)
-                current_standard = ADSI_STANDARD_TDD;
+                test_standard = ADSI_STANDARD_TDD;
             else
-                current_standard = atoi(argv[i]);
+                test_standard = atoi(argv[i]);
             continue;
         }
         if (strcmp(argv[i], "-l") == 0)
@@ -593,6 +644,10 @@ int main(int argc, char *argv[])
             fprintf(stderr, "    Cannot open wave file '%s'\n", decode_test_file);
             exit(2);
         }
+        if (test_standard < 0)
+            current_standard = ADSI_STANDARD_CLASS;
+        else
+            current_standard = test_standard;
 
         adsi_rx_init(&rx_adsi, current_standard, put_adsi_msg, NULL);
         span_log_set_level(&rx_adsi.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW);
@@ -635,12 +690,23 @@ int main(int argc, char *argv[])
         }
         /* Go through all the standards */
         /* This assumes standard 0 is NULL, and TDD is the last in the list */
-        for (j = 1;  j <= ADSI_STANDARD_TDD;  j++)
+        if (test_standard < 0)
         {
-            printf("Testing %s\n", adsi_standard_to_str(j));
-            current_standard = j;
-            adsi_tx_init(&tx_adsi, j);
-            adsi_rx_init(&rx_adsi, j, put_adsi_msg, NULL);
+            first_standard = ADSI_STANDARD_CLASS;
+            last_standard = ADSI_STANDARD_TDD;
+        }
+        else
+        {
+            first_standard =
+            last_standard = test_standard;
+        }
+        for (current_standard = first_standard;  current_standard <= last_standard;  current_standard++)
+        {
+            printf("Testing %s\n", adsi_standard_to_str(current_standard));
+            adsi_tx_init(&tx_adsi, current_standard);
+            if (short_preamble)
+                adsi_tx_set_preamble(&tx_adsi, 50, 20, 5, -1);
+            adsi_rx_init(&rx_adsi, current_standard, put_adsi_msg, NULL);
 
             /* Fake an OK condition for the first message test */
             good_message_received = TRUE;
@@ -660,7 +726,7 @@ int main(int argc, char *argv[])
                     {
                         if (!good_message_received)
                         {
-                            printf("No message received %s (%d)\n", adsi_standard_to_str(j), i);
+                            printf("No message received %s (%d)\n", adsi_standard_to_str(current_standard), i);
                             exit(2);
                         }
                         good_message_received = FALSE;
