@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: bit_operations_tests.c,v 1.5 2006/10/24 13:22:02 steveu Exp $
+ * $Id: bit_operations_tests.c,v 1.9 2007/02/23 12:59:45 steveu Exp $
  */
 
 /*! \page bit_operations_tests_page Bit operations tests
@@ -42,13 +42,21 @@ their operation with very dumb brute force versions of the same functionality.
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
 
 #include "spandsp.h"
 
+uint8_t from[1000000];
+uint8_t to[1000000];
+
 static __inline__ int top_bit_dumb(unsigned int data)
 {
     int i;
@@ -186,14 +194,14 @@ int main(int argc, char *argv[])
         bx = top_bit(x);
         if (ax != bx)
         {
-            printf("Test failed: top bit mismatch - %d %d\n", ax, bx);
+            printf("Test failed: top bit mismatch 0x%" PRIx32 " -> %u %u\n", x, ax, bx);
             exit(2);
         }
         ax = bottom_bit_dumb(x);
         bx = bottom_bit(x);
         if (ax != bx)
         {
-            printf("Test failed: bottom bit mismatch - %d %d\n", ax, bx);
+            printf("Test failed: bottom bit mismatch 0x%" PRIx32 " -> %u %u\n", x, ax, bx);
             exit(2);
         }
         x = rand();
@@ -208,14 +216,25 @@ int main(int argc, char *argv[])
             exit(2);
         }
     }
+    for (i = 0;  i < 1000000;  i++)
+        from[i] = rand();
+    bit_reverse(to, from, 1000000);
+    for (i = 0;  i < 1000000;  i++)
+    {
+        if (bit_reverse8_dumb(from[i]) != to[i])
+        {
+            printf("Test failed: bit reverse - at %d, %02x %02x %02x\n", i, from[i], bit_reverse8(from[i]), to[i]);
+            exit(2);
+        }
+    }
     for (i = 0;  i < 256;  i++)
     {
-        x = i | (i << 8) | (i << 16) | (i << 24);
+        x = i | (((i + 1) & 0xFF) << 8) | (((i + 2) & 0xFF) << 16) | (((i + 3) & 0xFF) << 24);
         ax32 = bit_reverse_4bytes_dumb(x);
         bx32 = bit_reverse_4bytes(x);
         if (ax32 != bx32)
         {
-            printf("Test failed: bit reverse 4 bytes - %x %x %x\n", x, ax32, bx32);
+            printf("Test failed: bit reverse 4 bytes - %" PRIx32 " %" PRIx32 " %" PRIx32 "\n", x, ax32, bx32);
             exit(2);
         }
     }
@@ -235,7 +254,7 @@ int main(int argc, char *argv[])
         bx32 = bit_reverse32(i);
         if (ax32 != bx32)
         {
-            printf("Test failed: bit reverse 32 - %x %x %x\n", i, ax32, bx32);
+            printf("Test failed: bit reverse 32 - %d %" PRIx32 " %" PRIx32 "\n", i, ax32, bx32);
             exit(2);
         }
     }
@@ -256,13 +275,13 @@ int main(int argc, char *argv[])
         ax32 = most_significant_one32(1 << i);
         if (ax32 != (1 << i))
         {
-            printf("Test failed: most significant one 32 - %x %x %x\n", i, ax32, (1 << i));
+            printf("Test failed: most significant one 32 - %x %" PRIx32 " %x\n", i, ax32, (1 << i));
             exit(2);
         }
         ax32 = least_significant_one32(1 << i);
         if (ax32 != (1 << i))
         {
-            printf("Test failed: least significant one 32 - %x %x %x\n", i, ax32, (1 << i));
+            printf("Test failed: least significant one 32 - %x %" PRIx32 " %x\n", i, ax32, (1 << i));
             exit(2);
         }
     }
