@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fsk_tests.c,v 1.33 2007/04/10 16:12:20 steveu Exp $
+ * $Id: fsk_tests.c,v 1.36 2007/06/28 13:10:59 steveu Exp $
  */
 
 /*! \page fsk_tests_page FSK modem tests
@@ -70,6 +70,7 @@ These tests allow either:
 char *decode_test_file = NULL;
 both_ways_line_model_state_t *model;
 int rx_bits = 0;
+int cutoff_test_carrier = FALSE;
 
 static void put_bit(void *user_data, int bit)
 {
@@ -101,6 +102,36 @@ static void put_bit(void *user_data, int bit)
 }
 /*- End of function --------------------------------------------------------*/
 
+static void cutoff_test_put_bit(void *user_data, int bit)
+{
+    if (bit < 0)
+    {
+        /* Special conditions */
+        switch (bit)
+        {
+        case PUTBIT_TRAINING_FAILED:
+            printf("Training failed\n");
+            break;
+        case PUTBIT_TRAINING_SUCCEEDED:
+            printf("Training succeeded\n");
+            break;
+        case PUTBIT_CARRIER_UP:
+            //printf("Carrier up\n");
+            cutoff_test_carrier = TRUE;
+            break;
+        case PUTBIT_CARRIER_DOWN:
+            //printf("Carrier down\n");
+            cutoff_test_carrier = FALSE;
+            break;
+        default:
+            printf("Eh!\n");
+            break;
+        }
+        return;
+    }
+}
+/*- End of function --------------------------------------------------------*/
+
 static void reporter(void *user_data, int reason, bert_results_t *results)
 {
     int channel;
@@ -166,6 +197,7 @@ int main(int argc, char *argv[])
     AFfilesetup filesetup;
     int outframes;    
     int i;
+    int j;
     int samples;
     int test_bps;
     int noise_level;
@@ -177,6 +209,10 @@ int main(int argc, char *argv[])
     int log_audio;
     int channel_codec;
     int rbs_pattern;
+    int on_at;
+    int off_at;
+    tone_gen_descriptor_t tone_desc;
+    tone_gen_state_t tone_tx;
 
     channel_codec = MUNGE_CODEC_NONE;
     rbs_pattern = 0;
@@ -230,6 +266,11 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (modem_under_test_1 >= 0)
+        printf("Modem channel 1 is '%s'\n", preset_fsk_specs[modem_under_test_1].name);
+    if (modem_under_test_2 >= 0)
+        printf("Modem channel 2 is '%s'\n", preset_fsk_specs[modem_under_test_2].name);
+
     filesetup = AF_NULL_FILESETUP;
     outhandle = AF_NULL_FILEHANDLE;
 
@@ -244,10 +285,23 @@ int main(int argc, char *argv[])
         afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
         afInitFileFormat(filesetup, AF_FILE_WAVE);
         afInitChannels(filesetup, AF_DEFAULT_TRACK, 2);
+        if ((outhandle = afOpenFile(OUTPUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
+        {
+            fprintf(stderr, "    Cannot create wave file '%s'\n", OUTPUT_FILE_NAME);
+            exit(2);
+        }
     }
     noise_level = -200;
     bits_per_test = 0;
     inhandle = NULL;
+
+    memset(caller_amp, 0, sizeof(*caller_amp));
+    memset(answerer_amp, 0, sizeof(*answerer_amp));
+    memset(caller_model_amp, 0, sizeof(*caller_model_amp));
+    memset(answerer_model_amp, 0, sizeof(*answerer_model_amp));
+    power_meter_init(&caller_meter, 7);
+    power_meter_init(&answerer_meter, 7);
+
     if (decode_test_file)
     {
         if ((inhandle = afOpenFile(decode_test_file, "r", NULL)) == AF_NULL_FILEHANDLE)
@@ -255,21 +309,99 @@ int main(int argc, char *argv[])
             fprintf(stderr, "    Cannot open wave file '%s'\n", decode_test_file);
             exit(2);
         }
-        fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_1], TRUE, 0, put_bit, NULL);
+        fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_1], TRUE, put_bit, NULL);
         test_bps = preset_fsk_specs[modem_under_test_1].baud_rate;
+
+        for (;;)
+        {
+            samples = afReadFrames(inhandle,
+                                   AF_DEFAULT_TRACK,
+                                   caller_model_amp,
+                                   BLOCK_LEN);
+            if (samples < BLOCK_LEN)
+                break;
+            for (i = 0;  i < samples;  i++)
+                power_meter_update(&caller_meter, caller_model_amp[i]);
+            fsk_rx(&caller_rx, caller_model_amp, samples);
+        }
+
+        if (afCloseFile(inhandle) != 0)
+        {
+            fprintf(stderr, "    Cannot close wave file '%s'\n", decode_test_file);
+            exit(2);
+        }
     }
     else
     {
+        printf("Test cutoff level\n");
+        fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_1], TRUE, cutoff_test_put_bit, NULL);
+        fsk_rx_signal_cutoff(&caller_rx, -30.0f);
+        on_at = 0;
+        for (i = -40;  i < -25;  i++)
+        {
+            make_tone_gen_descriptor(&tone_desc,
+                                     1500,
+                                     i,
+                                     0,
+                                     0,
+                                     1,
+                                     0,
+                                     0,
+                                     0,
+                                     TRUE);
+            tone_gen_init(&tone_tx, &tone_desc);
+            for (j = 0;  j < 10;  j++)
+            {
+                samples = tone_gen(&tone_tx, caller_model_amp, 160);
+                fsk_rx(&caller_rx, caller_model_amp, samples);
+            }
+            if (cutoff_test_carrier)
+               break;
+        }
+        on_at = i;
+        off_at = 0;
+        for (  ;  i > -40;  i--)
+        {
+            make_tone_gen_descriptor(&tone_desc,
+                                     1500,
+                                     i,
+                                     0,
+                                     0,
+                                     1,
+                                     0,
+                                     0,
+                                     0,
+                                     TRUE);
+            tone_gen_init(&tone_tx, &tone_desc);
+            for (j = 0;  j < 10;  j++)
+            {
+                samples = tone_gen(&tone_tx, caller_model_amp, 160);
+                fsk_rx(&caller_rx, caller_model_amp, samples);
+            }
+            if (!cutoff_test_carrier)
+                break;
+        }
+        off_at = i;
+        printf("Carrier on at %d, off at %d\n", on_at, off_at);
+        if (on_at < -29  ||  on_at > -26  
+            ||
+            off_at < -35  ||  off_at > -31)
+        {
+            printf("Tests failed.\n");
+            exit(2);
+        }
+                
         printf("Test with BERT\n");
+        test_bps = preset_fsk_specs[modem_under_test_1].baud_rate;
         if (modem_under_test_1 >= 0)
         {
             fsk_tx_init(&caller_tx, &preset_fsk_specs[modem_under_test_1], (get_bit_func_t) bert_get_bit, &caller_bert);
-            fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, 0, (put_bit_func_t) bert_put_bit, &answerer_bert);
+            fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, (put_bit_func_t) bert_put_bit, &answerer_bert);
         }
         if (modem_under_test_2 >= 0)
         {
             fsk_tx_init(&answerer_tx, &preset_fsk_specs[modem_under_test_2], (get_bit_func_t) bert_get_bit, &answerer_bert);
-            fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, 0, (put_bit_func_t) bert_put_bit, &caller_bert);
+            fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, (put_bit_func_t) bert_put_bit, &caller_bert);
         }
         test_bps = preset_fsk_specs[modem_under_test_1].baud_rate;
 
@@ -285,40 +417,8 @@ int main(int argc, char *argv[])
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
         }
-    }
-    if (log_audio)
-    {
-        if ((outhandle = afOpenFile(OUTPUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
-        {
-            fprintf(stderr, "    Cannot create wave file '%s'\n", OUTPUT_FILE_NAME);
-            exit(2);
-        }
-    }
 
-    if (modem_under_test_1 >= 0)
-        printf("Modem channel 1 is '%s'\n", preset_fsk_specs[modem_under_test_1].name);
-    if (modem_under_test_2 >= 0)
-        printf("Modem channel 2 is '%s'\n", preset_fsk_specs[modem_under_test_2].name);
-    memset(caller_amp, 0, sizeof(*caller_amp));
-    memset(answerer_amp, 0, sizeof(*answerer_amp));
-    memset(caller_model_amp, 0, sizeof(*caller_model_amp));
-    memset(answerer_model_amp, 0, sizeof(*answerer_model_amp));
-    power_meter_init(&caller_meter, 7);
-    power_meter_init(&answerer_meter, 7);
-    for (;;)
-    {
-        if (decode_test_file)
-        {
-            samples = afReadFrames(inhandle,
-                                   AF_DEFAULT_TRACK,
-                                   caller_model_amp,
-                                   BLOCK_LEN);
-            if (samples < BLOCK_LEN)
-                break;
-            for (i = 0;  i < samples;  i++)
-                power_meter_update(&caller_meter, caller_model_amp[i]);
-        }
-        else
+        for (;;)
         {
             samples = fsk_tx(&caller_tx, caller_amp, BLOCK_LEN);
             for (i = 0;  i < samples;  i++)
@@ -332,101 +432,102 @@ int main(int argc, char *argv[])
                                  answerer_model_amp,
                                  answerer_amp,
                                  samples);
-        }
-        //printf("Powers %10.5fdBm0 %10.5fdBm0\n", power_meter_current_dbm0(&caller_meter), power_meter_current_dbm0(&answerer_meter));
 
-        fsk_rx(&answerer_rx, caller_model_amp, samples);
-        for (i = 0;  i < samples;  i++)
-            out_amp[2*i] = caller_model_amp[i];
-        for (  ;  i < BLOCK_LEN;  i++)
-            out_amp[2*i] = 0;
+            //printf("Powers %10.5fdBm0 %10.5fdBm0\n", power_meter_current_dbm0(&caller_meter), power_meter_current_dbm0(&answerer_meter));
 
-        fsk_rx(&caller_rx, answerer_model_amp, samples);
-        for (i = 0;  i < samples;  i++)
-            out_amp[2*i + 1] = answerer_model_amp[i];
-        for (  ;  i < BLOCK_LEN;  i++)
-            out_amp[2*i + 1] = 0;
+            fsk_rx(&answerer_rx, caller_model_amp, samples);
+            for (i = 0;  i < samples;  i++)
+                out_amp[2*i] = caller_model_amp[i];
+            for (  ;  i < BLOCK_LEN;  i++)
+                out_amp[2*i] = 0;
+
+            fsk_rx(&caller_rx, answerer_model_amp, samples);
+            for (i = 0;  i < samples;  i++)
+                out_amp[2*i + 1] = answerer_model_amp[i];
+            for (  ;  i < BLOCK_LEN;  i++)
+                out_amp[2*i + 1] = 0;
         
-        if (log_audio)
-        {
-            outframes = afWriteFrames(outhandle,
-                                      AF_DEFAULT_TRACK,
-                                      out_amp,
-                                      BLOCK_LEN);
-            if (outframes != BLOCK_LEN)
-            {
-                fprintf(stderr, "    Error writing wave file\n");
-                exit(2);
-            }
-        }
-
-        if (samples < BLOCK_LEN)
-        {
-            bert_result(&caller_bert, &bert_results);
-            fprintf(stderr, "%ddB AWGN, %d bits, %d bad bits, %d resyncs\n", noise_level, bert_results.total_bits, bert_results.bad_bits, bert_results.resyncs);
-            if (!noise_sweep)
-            {
-                if (bert_results.total_bits != bits_per_test - 43
-                    ||
-                    bert_results.bad_bits != 0
-                    ||
-                    bert_results.resyncs != 0)
-                {
-                    printf("Tests failed.\n");
-                    exit(2);
-                }
-            }
-            bert_result(&answerer_bert, &bert_results);
-            fprintf(stderr, "%ddB AWGN, %d bits, %d bad bits, %d resyncs\n", noise_level, bert_results.total_bits, bert_results.bad_bits, bert_results.resyncs);
-            if (!noise_sweep)
-            {
-                if (bert_results.total_bits != bits_per_test - 43
-                    ||
-                    bert_results.bad_bits != 0
-                    ||
-                    bert_results.resyncs != 0)
-                {
-                    printf("Tests failed.\n");
-                    exit(2);
-                }
-                break;
-            }
-
-            /* Put a little silence between the chunks in the file. */
-            memset(out_amp, 0, sizeof(out_amp));
             if (log_audio)
             {
-                for (i = 0;  i < 200;  i++)
+                outframes = afWriteFrames(outhandle,
+                                          AF_DEFAULT_TRACK,
+                                          out_amp,
+                                          BLOCK_LEN);
+                if (outframes != BLOCK_LEN)
                 {
-                    outframes = afWriteFrames(outhandle,
-                                              AF_DEFAULT_TRACK,
-                                              out_amp,
-                                              BLOCK_LEN);
+                    fprintf(stderr, "    Error writing wave file\n");
+                    exit(2);
                 }
             }
-            if (modem_under_test_1 >= 0)
-            {
-                fsk_tx_init(&caller_tx, &preset_fsk_specs[modem_under_test_1], (get_bit_func_t) bert_get_bit, &caller_bert);
-                fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, 0, (put_bit_func_t) bert_put_bit, &answerer_bert);
-            }
-            if (modem_under_test_2 >= 0)
-            {
-                fsk_tx_init(&answerer_tx, &preset_fsk_specs[modem_under_test_2], (get_bit_func_t) bert_get_bit, &answerer_bert);
-                fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, 0, (put_bit_func_t) bert_put_bit, &caller_bert);
-            }
-            noise_level++;
-            if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec, 0)) == NULL)
-            {
-                fprintf(stderr, "    Failed to create line model\n");
-                exit(2);
-            }
-            bert_init(&caller_bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
-            bert_set_report(&caller_bert, 100000, reporter, (void *) (intptr_t) 1);
-            bert_init(&answerer_bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
-            bert_set_report(&answerer_bert, 100000, reporter, (void *) (intptr_t) 2);
-        }
-    }
 
+            if (samples < BLOCK_LEN)
+            {
+                bert_result(&caller_bert, &bert_results);
+                fprintf(stderr, "%ddB AWGN, %d bits, %d bad bits, %d resyncs\n", noise_level, bert_results.total_bits, bert_results.bad_bits, bert_results.resyncs);
+                if (!noise_sweep)
+                {
+                    if (bert_results.total_bits != bits_per_test - 43
+                        ||
+                        bert_results.bad_bits != 0
+                        ||
+                        bert_results.resyncs != 0)
+                    {
+                        printf("Tests failed.\n");
+                        exit(2);
+                    }
+                }
+                bert_result(&answerer_bert, &bert_results);
+                fprintf(stderr, "%ddB AWGN, %d bits, %d bad bits, %d resyncs\n", noise_level, bert_results.total_bits, bert_results.bad_bits, bert_results.resyncs);
+                if (!noise_sweep)
+                {
+                    if (bert_results.total_bits != bits_per_test - 43
+                        ||
+                        bert_results.bad_bits != 0
+                        ||
+                        bert_results.resyncs != 0)
+                    {
+                        printf("Tests failed.\n");
+                        exit(2);
+                    }
+                    break;
+                }
+    
+                /* Put a little silence between the chunks in the file. */
+                memset(out_amp, 0, sizeof(out_amp));
+                if (log_audio)
+                {
+                    for (i = 0;  i < 200;  i++)
+                    {
+                        outframes = afWriteFrames(outhandle,
+                                                  AF_DEFAULT_TRACK,
+                                                  out_amp,
+                                                  BLOCK_LEN);
+                    }
+                }
+                if (modem_under_test_1 >= 0)
+                {
+                    fsk_tx_init(&caller_tx, &preset_fsk_specs[modem_under_test_1], (get_bit_func_t) bert_get_bit, &caller_bert);
+                    fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, (put_bit_func_t) bert_put_bit, &answerer_bert);
+                }
+                if (modem_under_test_2 >= 0)
+                {
+                    fsk_tx_init(&answerer_tx, &preset_fsk_specs[modem_under_test_2], (get_bit_func_t) bert_get_bit, &answerer_bert);
+                    fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, (put_bit_func_t) bert_put_bit, &caller_bert);
+                }
+                noise_level++;
+                if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec, 0)) == NULL)
+                {
+                    fprintf(stderr, "    Failed to create line model\n");
+                    exit(2);
+                }
+                bert_init(&caller_bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
+                bert_set_report(&caller_bert, 100000, reporter, (void *) (intptr_t) 1);
+                bert_init(&answerer_bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
+                bert_set_report(&answerer_bert, 100000, reporter, (void *) (intptr_t) 2);
+            }
+        }
+        printf("Tests passed.\n");
+    }
     if (log_audio)
     {
         if (afCloseFile(outhandle) != 0)
@@ -436,18 +537,6 @@ int main(int argc, char *argv[])
         }
         afFreeFileSetup(filesetup);
     }
-    if (decode_test_file)
-    {
-        if (afCloseFile(inhandle) != 0)
-        {
-            fprintf(stderr, "    Cannot close wave file '%s'\n", decode_test_file);
-            exit(2);
-        }
-    }
-    else
-    {
-        printf("Tests passed.\n");
-    }
     return  0;
 }
 /*- End of function --------------------------------------------------------*/
