@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: noise_tests.c,v 1.8 2006/10/24 13:22:03 steveu Exp $
+ * $Id: noise_tests.c,v 1.9 2006/11/19 14:07:27 steveu Exp $
  */
 
 /*! \page noise_tests_page Noise generator tests
@@ -37,7 +37,12 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <audiofile.h>
 #include <tiffio.h>
 
