@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: super_tone_rx_tests.c,v 1.15 2006/10/24 13:45:29 steveu Exp $
+ * $Id: super_tone_rx_tests.c,v 1.20 2007/06/28 13:10:59 steveu Exp $
  */
 
 /*! \file */
@@ -44,11 +44,22 @@
 #include <time.h>
 #include <inttypes.h>
 #include <sys/socket.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 
+#if defined(HAVE_LIBXML_XMLMEMORY_H)
 #include <libxml/xmlmemory.h>
+#endif
+#if defined(HAVE_LIBXML_PARSER_H)
 #include <libxml/parser.h>
+#endif
+#if defined(HAVE_LIBXML_XINCLUDE_H)
 #include <libxml/xinclude.h>
+#endif
 
 #include <audiofile.h>
 #include <tiffio.h>
@@ -56,10 +67,13 @@
 #include "spandsp.h"
 
 #define IN_FILE_NAME    "super_tone.wav"
+
+#define MITEL_DIR       "../itutests/mitel/"
 #define BELLCORE_DIR	"../itutests/bellcore/"
 
 const char *bellcore_files[] =
 {
+    MITEL_DIR    "mitel-cm7291-talkoff.wav",
     BELLCORE_DIR "tr-tsy-00763-1.wav",
     BELLCORE_DIR "tr-tsy-00763-2.wav",
     BELLCORE_DIR "tr-tsy-00763-3.wav",
@@ -82,6 +96,7 @@ super_tone_tx_step_t *nutone_tree = NULL;
 super_tone_tx_step_t *congestiontone_tree = NULL;
 super_tone_tx_step_t *waitingtone_tree = NULL;
 
+#if defined(HAVE_LIBXML2)
 static int parse_tone(super_tone_rx_descriptor_t *desc, int tone_id, super_tone_tx_step_t **tree, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
     xmlChar *x;
@@ -322,6 +337,7 @@ static void get_tone_set(super_tone_rx_descriptor_t *desc, const char *tone_file
     xmlFreeDoc(doc);
 }
 /*- End of function --------------------------------------------------------*/
+#endif
 
 static void super_tone_rx_fill_descriptor(super_tone_rx_descriptor_t *desc)
 {
@@ -338,7 +354,7 @@ static void super_tone_rx_fill_descriptor(super_tone_rx_descriptor_t *desc)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void wakeup(void *data, int code)
+static void wakeup(void *data, int code, int level)
 {
     if (code >= 0)
         printf("Current tone is %d '%s' '%s'\n", code, (tone_names[code])  ?  tone_names[code]  :  "???", (char *) data);
@@ -389,7 +405,9 @@ int main(int argc, char *argv[])
         exit(2);
     }
     super_tone_rx_make_descriptor(&desc);
+#if defined(HAVE_LIBXML2)
     get_tone_set(&desc, "../spandsp/global-tones.xml", (argc > 1)  ?  argv[1]  :  "hk");
+#endif
     super_tone_rx_fill_descriptor(&desc);
     if ((super = super_tone_rx_init(NULL, &desc, wakeup, (void *) "test")) == NULL)
     {
@@ -416,6 +434,7 @@ int main(int argc, char *argv[])
         exit(2);
     }
 #if 0
+    /* Test for voice immunity */
     for (j = 0;  bellcore_files[j][0];  j++)
     {
         if ((inhandle = afOpenFile(bellcore_files[j], "r", 0)) == AF_NULL_FILEHANDLE)
