@@ -22,12 +22,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t38_gateway_tests.c,v 1.36 2006/10/24 13:45:29 steveu Exp $
+ * $Id: t38_gateway_tests.c,v 1.51 2007/04/13 12:12:07 steveu Exp $
  */
 
 /*! \file */
 
-/*! \page t38_gateway_tests_page T.38 gateway tests tests
+/*! \page t38_gateway_tests_page T.38 gateway tests
 \section t38_gateway_tests_page_sec_1 What does it do?
 These tests exercise the path
 
@@ -38,19 +38,29 @@ These tests exercise the path
 #include <config.h>
 #endif
 
+#if defined(HAVE_FL_FL_H)  &&  defined(HAVE_FL_FL_CARTESIAN_H)  &&  defined(HAVE_FL_FL_AUDIO_METER_H)
+#define ENABLE_GUI
+#endif
+
 #include <inttypes.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <errno.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#if !defined(__USE_MISC)
 #define __USE_MISC
+#endif
 #include <arpa/inet.h>
 #include <sys/select.h>
 #include <sys/time.h>
@@ -58,8 +68,11 @@ These tests exercise the path
 #include <tiffio.h>
 
 #include "spandsp.h"
+#include "spandsp-sim.h"
 
-#include "ip_network_model.h"
+#if defined(ENABLE_GUI)
+#include "media_monitor.h"
+#endif
 
 #define SAMPLES_PER_CHUNK       160
 
@@ -76,12 +89,16 @@ t38_gateway_state_t t38_state_a;
 t38_gateway_state_t t38_state_b;
 fax_state_t fax_state_b;
 
-ip_network_model_state_t *path_a_to_b;
-ip_network_model_state_t *path_b_to_a;
+g1050_state_t *path_a_to_b;
+g1050_state_t *path_b_to_a;
+
+double when = 0.0;
 
 int done[2] = {FALSE, FALSE};
 int succeeded[2] = {FALSE, FALSE};
 
+int simulate_incrementing_repeats = FALSE;
+
 static void phase_b_handler(t30_state_t *s, void *user_data, int result)
 {
     int i;
@@ -145,26 +162,58 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
 
 static int tx_packet_handler_a(t38_core_state_t *s, void *user_data, const uint8_t *buf, int len, int count)
 {
-    t38_gateway_state_t *t;
+    t38_terminal_state_t *t;
+    int i;
+    static int subst_seq = 0;
 
     /* This routine queues messages between two instances of T.38 processing */
-    t = (t38_gateway_state_t *) user_data;
-    span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+    t = (t38_terminal_state_t *) user_data;
+    if (simulate_incrementing_repeats)
+    {
+        for (i = 0;  i < count;  i++)
+        {
+            span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d\n", subst_seq, len);
 
-    ip_network_model_send(path_a_to_b, s->tx_seq_no, count, buf, len);
+            g1050_put(path_a_to_b, buf, len, subst_seq, when);
+            subst_seq = (subst_seq + 1) & 0xFFFF;
+        }
+    }
+    else
+    {
+        span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+
+        for (i = 0;  i < count;  i++)
+            g1050_put(path_a_to_b, buf, len, s->tx_seq_no, when);
+    }
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
 
 static int tx_packet_handler_b(t38_core_state_t *s, void *user_data, const uint8_t *buf, int len, int count)
 {
-    t38_gateway_state_t *t;
+    t38_terminal_state_t *t;
+    int i;
+    static int subst_seq = 0;
 
     /* This routine queues messages between two instances of T.38 processing */
-    t = (t38_gateway_state_t *) user_data;
-    span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+    t = (t38_terminal_state_t *) user_data;
+    if (simulate_incrementing_repeats)
+    {
+        for (i = 0;  i < count;  i++)
+        {
+            span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d\n", subst_seq, len);
 
-    ip_network_model_send(path_b_to_a, s->tx_seq_no, count, buf, len);
+            g1050_put(path_b_to_a, buf, len, subst_seq, when);
+            subst_seq = (subst_seq + 1) & 0xFFFF;
+        }
+    }
+    else
+    {
+        span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+
+        for (i = 0;  i < count;  i++)
+            g1050_put(path_b_to_a, buf, len, s->tx_seq_no, when);
+    }
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -188,15 +237,28 @@ int main(int argc, char *argv[])
     AFfilesetup filesetup;
     AFfilehandle wave_handle;
     int use_ecm;
+    int use_tep;
+    int use_transmit_on_idle;
     int t38_version;
     const char *input_file_name;
     int i;
     int seq_no;
-    
+    int model_no;
+    int speed_pattern_no;
+    double tx_when;
+    double rx_when;
+    int use_gui;
+
     log_audio = FALSE;
     use_ecm = FALSE;
     t38_version = 1;
     input_file_name = INPUT_FILE_NAME;
+    simulate_incrementing_repeats = FALSE;
+    model_no = 0;
+    speed_pattern_no = 1;
+    use_gui = FALSE;
+    use_tep = FALSE;
+    use_transmit_on_idle = TRUE;
     for (i = 1;  i < argc;  i++)
     {
         if (strcmp(argv[i], "-e") == 0)
@@ -204,10 +266,19 @@ int main(int argc, char *argv[])
             use_ecm = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-g") == 0)
+        {
+            use_gui = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-i") == 0)
         {
-            i++;
-            input_file_name = argv[i];
+            input_file_name = argv[++i];
+            continue;
+        }
+        if (strcmp(argv[i], "-I") == 0)
+        {
+            simulate_incrementing_repeats = TRUE;
             continue;
         }
         if (strcmp(argv[i], "-l") == 0)
@@ -215,14 +286,32 @@ int main(int argc, char *argv[])
             log_audio = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-m") == 0)
+        {
+            model_no = argv[++i][0] - 'A' + 1;
+            continue;
+        }
+        if (strcmp(argv[i], "-s") == 0)
+        {
+            speed_pattern_no = atoi(argv[++i]);
+            continue;
+        }
+        if (strcmp(argv[i], "-t") == 0)
+        {
+            use_tep = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-v") == 0)
         {
-            i++;
-            t38_version = atoi(argv[i]);
+            t38_version = atoi(argv[++i]);
             continue;
         }
     }
 
+    printf("Using T.38 version %d\n", t38_version);
+    if (use_ecm)
+        printf("Using ECM\n");
+
     filesetup = AF_NULL_FILESETUP;
     wave_handle = AF_NULL_FILEHANDLE;
     if (log_audio)
@@ -244,23 +333,32 @@ int main(int argc, char *argv[])
     }
     memset(silence, 0, sizeof(silence));
  
-    if ((path_a_to_b = ip_network_model_init(800, 2000, 0)) == NULL)
+    srand48(0x1234567);
+    if ((path_a_to_b = g1050_init(model_no, speed_pattern_no, 100, 33)) == NULL)
     {
         fprintf(stderr, "Failed to start IP network path model\n");
         exit(2);
     }
-    if ((path_b_to_a = ip_network_model_init(800, 2000, 0)) == NULL)
+    if ((path_b_to_a = g1050_init(model_no, speed_pattern_no, 100, 33)) == NULL)
     {
         fprintf(stderr, "Failed to start IP network path model\n");
         exit(2);
     }
 
-    fax_init(&fax_state_a, TRUE);
+    if (fax_init(&fax_state_a, TRUE) == NULL)
+    {
+        fprintf(stderr, "Cannot start FAX\n");
+        exit(2);
+    }
+    fax_set_transmit_on_idle(&fax_state_a, use_transmit_on_idle);
+    fax_set_tep_mode(&fax_state_a, use_tep);
+    //t30_set_supported_modems(&(fax_state_a.t30_state), T30_SUPPORT_V27TER | T30_SUPPORT_V29);
     t30_set_local_ident(&fax_state_a.t30_state, "11111111");
     t30_set_tx_file(&fax_state_a.t30_state, input_file_name, -1, -1);
     t30_set_phase_b_handler(&fax_state_a.t30_state, phase_b_handler, (void *) (intptr_t) 'A');
     t30_set_phase_d_handler(&fax_state_a.t30_state, phase_d_handler, (void *) (intptr_t) 'A');
     t30_set_phase_e_handler(&fax_state_a.t30_state, phase_e_handler, (void *) (intptr_t) 'A');
+    t30_set_local_nsf(&fax_state_a.t30_state, (const uint8_t *) "\x50\x00\x00\x00Spandsp\x00", 12);
     t30_set_ecm_capability(&fax_state_a.t30_state, use_ecm);
     if (use_ecm)
         t30_set_supported_compressions(&fax_state_a.t30_state, T30_SUPPORT_T4_1D_COMPRESSION | T30_SUPPORT_T4_2D_COMPRESSION | T30_SUPPORT_T6_COMPRESSION);
@@ -275,8 +373,11 @@ int main(int argc, char *argv[])
         fprintf(stderr, "Cannot start the T.38 channel\n");
         exit(2);
     }
+    t38_gateway_set_transmit_on_idle(&t38_state_a, use_transmit_on_idle);
+    //t38_gateway_set_supported_modems(&t38_state_a, T30_SUPPORT_V27TER | T30_SUPPORT_V29);
+    //t38_gateway_set_nsx_suppression(&t38_state_a, FALSE);
     t38_set_t38_version(&t38_state_a.t38, t38_version);
-    t38_gateway_ecm_control(&t38_state_a, use_ecm);
+    t38_gateway_set_ecm_capability(&t38_state_a, use_ecm);
     span_log_set_level(&t38_state_a.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
     span_log_set_tag(&t38_state_a.logging, "T.38-A");
     span_log_set_level(&t38_state_a.t38.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
@@ -288,20 +389,30 @@ int main(int argc, char *argv[])
         fprintf(stderr, "Cannot start the T.38 channel\n");
         exit(2);
     }
+    t38_gateway_set_transmit_on_idle(&t38_state_b, use_transmit_on_idle);
+    //t38_gateway_set_supported_modems(&t38_state_b, T30_SUPPORT_V27TER | T30_SUPPORT_V29);
+    //t38_gateway_set_nsx_suppression(&t38_state_b, FALSE);
     t38_set_t38_version(&t38_state_b.t38, t38_version);
-    t38_gateway_ecm_control(&t38_state_b, use_ecm);
+    t38_gateway_set_ecm_capability(&t38_state_b, use_ecm);
     span_log_set_level(&t38_state_b.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
     span_log_set_tag(&t38_state_b.logging, "T.38-B");
     span_log_set_level(&t38_state_b.t38.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
     span_log_set_tag(&t38_state_b.t38.logging, "T.38-B");
     memset(t38_amp_b, 0, sizeof(t38_amp_b));
 
-    fax_init(&fax_state_b, FALSE);
+    if (fax_init(&fax_state_b, FALSE) == NULL)
+    {
+        fprintf(stderr, "Cannot start FAX\n");
+        exit(2);
+    }
+    fax_set_transmit_on_idle(&fax_state_b, use_transmit_on_idle);
+    fax_set_tep_mode(&fax_state_b, use_tep);
     t30_set_local_ident(&fax_state_b.t30_state, "22222222");
     t30_set_rx_file(&fax_state_b.t30_state, OUTPUT_FILE_NAME, -1);
     t30_set_phase_b_handler(&fax_state_b.t30_state, phase_b_handler, (void *) (intptr_t) 'B');
     t30_set_phase_d_handler(&fax_state_b.t30_state, phase_d_handler, (void *) (intptr_t) 'B');
     t30_set_phase_e_handler(&fax_state_b.t30_state, phase_e_handler, (void *) (intptr_t) 'B');
+    t30_set_local_nsf(&fax_state_b.t30_state, (const uint8_t *) "\x50\x00\x00\x00Spandsp\x00", 12);
     t30_set_ecm_capability(&fax_state_b.t30_state, use_ecm);
     if (use_ecm)
         t30_set_supported_compressions(&fax_state_b.t30_state, T30_SUPPORT_T4_1D_COMPRESSION | T30_SUPPORT_T4_2D_COMPRESSION | T30_SUPPORT_T6_COMPRESSION);
@@ -311,6 +422,10 @@ int main(int argc, char *argv[])
     span_log_set_tag(&fax_state_b.t30_state.logging, "FAX-B ");
     memset(t30_amp_b, 0, sizeof(t30_amp_b));
 
+#if defined(ENABLE_GUI)
+    if (use_gui)
+        start_media_monitor();
+#endif
     for (;;)
     {
         span_log_bump_samples(&fax_state_a.logging, SAMPLES_PER_CHUNK);
@@ -324,13 +439,16 @@ int main(int argc, char *argv[])
         memset(out_amp, 0, sizeof(out_amp));
 
         t30_len_a = fax_tx(&fax_state_a, t30_amp_a, SAMPLES_PER_CHUNK);
-        /* The receive side always expects a full block of samples, but the
-           transmit side may not be sending any when it doesn't need to. We
-           may need to pad with some silence. */
-        if (t30_len_a < SAMPLES_PER_CHUNK)
+        if (!use_transmit_on_idle)
         {
-            memset(t30_amp_a + t30_len_a, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t30_len_a));
-            t30_len_a = SAMPLES_PER_CHUNK;
+            /* The receive side always expects a full block of samples, but the
+               transmit side may not be sending any when it doesn't need to. We
+               may need to pad with some silence. */
+            if (t30_len_a < SAMPLES_PER_CHUNK)
+            {
+                memset(t30_amp_a + t30_len_a, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t30_len_a));
+                t30_len_a = SAMPLES_PER_CHUNK;
+            }
         }
         if (log_audio)
         {
@@ -341,10 +459,13 @@ int main(int argc, char *argv[])
             break;
 
         t38_len_a = t38_gateway_tx(&t38_state_a, t38_amp_a, SAMPLES_PER_CHUNK);
-        if (t38_len_a < SAMPLES_PER_CHUNK)
+        if (!use_transmit_on_idle)
         {
-            memset(t38_amp_a + t38_len_a, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t38_len_a));
-            t38_len_a = SAMPLES_PER_CHUNK;
+            if (t38_len_a < SAMPLES_PER_CHUNK)
+            {
+                memset(t38_amp_a + t38_len_a, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t38_len_a));
+                t38_len_a = SAMPLES_PER_CHUNK;
+            }
         }
         if (log_audio)
         {
@@ -355,13 +476,16 @@ int main(int argc, char *argv[])
             break;
 
         t30_len_b = fax_tx(&fax_state_b, t30_amp_b, SAMPLES_PER_CHUNK);
-        /* The receive side always expects a full block of samples, but the
-           transmit side may not be sending any when it doesn't need to. We
-           may need to pad with some silence. */
-        if (t30_len_b < SAMPLES_PER_CHUNK)
+        if (!use_transmit_on_idle)
         {
-            memset(t30_amp_b + t30_len_b, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t30_len_b));
-            t30_len_b = SAMPLES_PER_CHUNK;
+            /* The receive side always expects a full block of samples, but the
+               transmit side may not be sending any when it doesn't need to. We
+               may need to pad with some silence. */
+            if (t30_len_b < SAMPLES_PER_CHUNK)
+            {
+                memset(t30_amp_b + t30_len_b, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t30_len_b));
+                t30_len_b = SAMPLES_PER_CHUNK;
+            }
         }
         if (log_audio)
         {
@@ -372,10 +496,13 @@ int main(int argc, char *argv[])
             break;
 
         t38_len_b = t38_gateway_tx(&t38_state_b, t38_amp_b, SAMPLES_PER_CHUNK);
-        if (t38_len_b < SAMPLES_PER_CHUNK)
+        if (!use_transmit_on_idle)
         {
-            memset(t38_amp_b + t38_len_b, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t38_len_b));
-            t38_len_b = SAMPLES_PER_CHUNK;
+            if (t38_len_b < SAMPLES_PER_CHUNK)
+            {
+                memset(t38_amp_b + t38_len_b, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - t38_len_b));
+                t38_len_b = SAMPLES_PER_CHUNK;
+            }
         }
         if (log_audio)
         {
@@ -385,11 +512,24 @@ int main(int argc, char *argv[])
         if (fax_rx(&fax_state_b, t38_amp_b, SAMPLES_PER_CHUNK))
             break;
 
-        while ((msg_len = ip_network_model_get(path_a_to_b, SAMPLES_PER_CHUNK, msg, 1024, &seq_no)) >= 0)
-            t38_core_rx_ifp_packet(&t38_state_b.t38, seq_no, msg, msg_len);
-        while ((msg_len = ip_network_model_get(path_b_to_a, SAMPLES_PER_CHUNK, msg, 1024, &seq_no)) >= 0)
-            t38_core_rx_ifp_packet(&t38_state_a.t38, seq_no, msg, msg_len);
+        when += 0.02;
 
+        while ((msg_len = g1050_get(path_a_to_b, msg, 1024, when, &seq_no, &tx_when, &rx_when)) >= 0)
+        {
+#if defined(ENABLE_GUI)
+            if (use_gui)
+                media_monitor_rx(seq_no, tx_when, rx_when);
+#endif
+            t38_core_rx_ifp_packet(&t38_state_b.t38, msg, msg_len, seq_no);
+        }
+        while ((msg_len = g1050_get(path_b_to_a, msg, 1024, when, &seq_no, &tx_when, &rx_when)) >= 0)
+        {
+#if defined(ENABLE_GUI)
+            if (use_gui)
+                media_monitor_rx(seq_no, tx_when, rx_when);
+#endif
+            t38_core_rx_ifp_packet(&t38_state_a.t38, msg, msg_len, seq_no);
+        }
         if (log_audio)
         {
             outframes = afWriteFrames(wave_handle, AF_DEFAULT_TRACK, out_amp, SAMPLES_PER_CHUNK);
@@ -399,6 +539,10 @@ int main(int argc, char *argv[])
 
         if (done[0]  &&  done[1])
             break;
+#if defined(ENABLE_GUI)
+        if (use_gui)
+            media_monitor_update_display();
+#endif
     }
     if (log_audio)
     {
