@@ -1,4 +1,3 @@
-#define ENABLE_V17
 //#define ADD_MAINS_INTERFERENCE
 /*
  * SpanDSP - a series of DSP components for telephony
@@ -24,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v17_tests.c,v 1.66 2007/04/03 12:59:33 steveu Exp $
+ * $Id: v17_tests.c,v 1.69 2007/06/29 12:56:28 steveu Exp $
  */
 
 /*! \page v17_tests_page V.17 modem tests
@@ -377,17 +376,17 @@ int main(int argc, char *argv[])
         }
         if ((x = afGetFrameSize(inhandle, AF_DEFAULT_TRACK, 1)) != 2.0)
         {
-            printf("    Unexpected frame size in speech file '%s'\n", decode_test_file);
+            printf("    Unexpected frame size in speech file '%s' (%f)\n", decode_test_file, x);
             exit(2);
         }
         if ((x = afGetRate(inhandle, AF_DEFAULT_TRACK)) != (float) SAMPLE_RATE)
         {
-            printf("    Unexpected sample rate in speech file '%s'\n", decode_test_file);
+            printf("    Unexpected sample rate in speech file '%s' (%f)\n", decode_test_file, x);
             exit(2);
         }
         if ((x = afGetChannels(inhandle, AF_DEFAULT_TRACK)) != 1.0)
         {
-            printf("    Unexpected number of channels in speech file '%s'\n", decode_test_file);
+            printf("    Unexpected number of channels in speech file '%s' (%f)\n", decode_test_file, x);
             exit(2);
         }
     }
@@ -509,14 +508,7 @@ int main(int argc, char *argv[])
 #endif
         v17_rx(&rx, amp, samples);
     }
-    if (decode_test_file)
-    {
-#if defined(ENABLE_GUI)
-        if (use_gui)
-            qam_wait_to_end(qam_monitor);
-#endif
-    }
-    else
+    if (!decode_test_file)
     {
         bert_result(&bert, &bert_results);
         fprintf(stderr, "At completion:\n");
@@ -524,6 +516,10 @@ int main(int argc, char *argv[])
         fprintf(stderr, "Last report  %ddBm0, %d bits, %d bad bits, %d resyncs\n", signal_level, latest_results.total_bits, latest_results.bad_bits, latest_results.resyncs);
         one_way_line_model_release(line_model);
     }
+#if defined(ENABLE_GUI)
+    if (use_gui)
+        qam_wait_to_end(qam_monitor);
+#endif
     if (log_audio)
     {
         if (afCloseFile(outhandle))
