@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tests.c,v 1.73 2007/04/03 12:59:33 steveu Exp $
+ * $Id: v27ter_tests.c,v 1.75 2007/06/29 12:56:28 steveu Exp $
  */
 
 /*! \page v27ter_tests_page V.27ter modem tests
@@ -378,17 +378,17 @@ int main(int argc, char *argv[])
         }
         if ((x = afGetFrameSize(inhandle, AF_DEFAULT_TRACK, 1)) != 2.0)
         {
-            printf("    Unexpected frame size in speech file '%s'\n", decode_test_file);
+            printf("    Unexpected frame size in speech file '%s' (%f)\n", decode_test_file, x);
             exit(2);
         }
         if ((x = afGetRate(inhandle, AF_DEFAULT_TRACK)) != (float) SAMPLE_RATE)
         {
-            printf("    Unexpected sample rate in speech file '%s'\n", decode_test_file);
+            printf("    Unexpected sample rate in speech file '%s' (%f)\n", decode_test_file, x);
             exit(2);
         }
         if ((x = afGetChannels(inhandle, AF_DEFAULT_TRACK)) != 1.0)
         {
-            printf("    Unexpected number of channels in speech file '%s'\n", decode_test_file);
+            printf("    Unexpected number of channels in speech file '%s' (%f)\n", decode_test_file, x);
             exit(2);
         }
     }
@@ -514,14 +514,7 @@ int main(int argc, char *argv[])
         if (decode_test_file == NULL  &&  block%500 == 0)
             printf("Noise level is %d\n", noise_level);
     }
-    if (decode_test_file)
-    {
-#if defined(ENABLE_GUI)
-        if (use_gui)
-            qam_wait_to_end(qam_monitor);
-#endif
-    }
-    else
+    if (!decode_test_file)
     {
         bert_result(&bert, &bert_results);
         fprintf(stderr, "At completion:\n");
@@ -536,6 +529,10 @@ int main(int argc, char *argv[])
 
         printf("Tests passed.\n");
     }
+#if defined(ENABLE_GUI)
+    if (use_gui)
+        qam_wait_to_end(qam_monitor);
+#endif
     if (log_audio)
     {
         if (afCloseFile(outhandle))
