@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42bis_tests.c,v 1.15 2006/10/24 13:45:30 steveu Exp $
+ * $Id: v42bis_tests.c,v 1.18 2006/11/19 14:07:28 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. */
@@ -47,8 +47,12 @@ of this file should exactly match the original file.
 #include <errno.h>
 #include <fcntl.h>
 #include <ctype.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <tiffio.h>
 
@@ -87,70 +91,82 @@ int main(int argc, char *argv[])
     int in_fd;
     int v42bis_fd;
     int out_fd;
+    int do_compression;
+    int do_decompression;
+    time_t now;
 
+    do_compression = TRUE;
+    do_decompression = TRUE;
     if (argc < 2)
     {
         fprintf(stderr, "Usage: %s <file>\n", argv[0]);
         exit(2);
     }
-    if ((in_fd = open(argv[1], O_RDONLY)) < 0)
+    if (do_compression)
     {
-        fprintf(stderr, "Error opening file '%s'.\n", argv[1]);
-        exit(2);
-    }
-    if ((v42bis_fd = open(COMPRESSED_FILE_NAME, O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0)
-    {
-        fprintf(stderr, "Error opening file '%s'.\n", COMPRESSED_FILE_NAME);
-        exit(2);
-    }
-
-    v42bis_init(&state_a, 3, 512, 6, frame_handler, (void *) (intptr_t) v42bis_fd, 512, data_handler, NULL, 512);
-    in_octets_to_date = 0;
-    out_octets_to_date = 0;
-    while ((len = read(in_fd, buf, 1024)) > 0)
-    {
-        if (v42bis_compress(&state_a, buf, len))
+        if ((in_fd = open(argv[1], O_RDONLY)) < 0)
         {
-            fprintf(stderr, "Bad return code from compression\n");
+            fprintf(stderr, "Error opening file '%s'.\n", argv[1]);
             exit(2);
         }
-        in_octets_to_date += len;
-    }
-    v42bis_compress_flush(&state_a);
-    printf("%d bytes compressed to %d bytes\n", in_octets_to_date, out_octets_to_date);
-    close(in_fd);
-    close(v42bis_fd);
-
-    /* Now open the files for the decompression. */
-    if ((v42bis_fd = open(COMPRESSED_FILE_NAME, O_RDONLY)) < 0)
-    {
-        fprintf(stderr, "Error opening file '%s'.\n", COMPRESSED_FILE_NAME);
-        exit(2);
-    }
-    if ((out_fd = open(OUTPUT_FILE_NAME, O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0)
-    {
-        fprintf(stderr, "Error opening file '%s'.\n", OUTPUT_FILE_NAME);
-        exit(2);
-    }
-
-    v42bis_init(&state_b, 3, 512, 6, frame_handler, (void *) (intptr_t) v42bis_fd, 512, data_handler, (void *) (intptr_t) out_fd, 512);
-
-    in_octets_to_date = 0;
-    out_octets_to_date = 0;
-    while ((len = read(v42bis_fd, buf, 1024)) > 0)
-    {
-        if (v42bis_decompress(&state_b, buf, len))
+        if ((v42bis_fd = open(COMPRESSED_FILE_NAME, O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0)
         {
-            fprintf(stderr, "Bad return code from decompression\n");
+            fprintf(stderr, "Error opening file '%s'.\n", COMPRESSED_FILE_NAME);
             exit(2);
         }
-        in_octets_to_date += len;
-    }
-    v42bis_decompress_flush(&state_b);
-    printf("%d bytes decompressed to %d bytes\n", in_octets_to_date, out_octets_to_date);
-    close(v42bis_fd);
-    close(out_fd);
 
+        time(&now);
+        v42bis_init(&state_a, 3, 512, 6, frame_handler, (void *) (intptr_t) v42bis_fd, 512, data_handler, NULL, 512);
+        v42bis_compression_control(&state_a, V42BIS_COMPRESSION_MODE_ALWAYS);
+        in_octets_to_date = 0;
+        out_octets_to_date = 0;
+        while ((len = read(in_fd, buf, 1024)) > 0)
+        {
+            if (v42bis_compress(&state_a, buf, len))
+            {
+                fprintf(stderr, "Bad return code from compression\n");
+                exit(2);
+            }
+            in_octets_to_date += len;
+        }
+        v42bis_compress_flush(&state_a);
+        printf("%d bytes compressed to %d bytes in %lds\n", in_octets_to_date, out_octets_to_date, time(NULL) - now);
+        close(in_fd);
+        close(v42bis_fd);
+    }
+
+    if (do_decompression)
+    {
+        /* Now open the files for the decompression. */
+        if ((v42bis_fd = open(COMPRESSED_FILE_NAME, O_RDONLY)) < 0)
+        {
+            fprintf(stderr, "Error opening file '%s'.\n", COMPRESSED_FILE_NAME);
+            exit(2);
+        }
+        if ((out_fd = open(OUTPUT_FILE_NAME, O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0)
+        {
+            fprintf(stderr, "Error opening file '%s'.\n", OUTPUT_FILE_NAME);
+            exit(2);
+        }
+    
+        time(&now);
+        v42bis_init(&state_b, 3, 512, 6, frame_handler, (void *) (intptr_t) v42bis_fd, 512, data_handler, (void *) (intptr_t) out_fd, 512);
+        in_octets_to_date = 0;
+        out_octets_to_date = 0;
+        while ((len = read(v42bis_fd, buf, 1024)) > 0)
+        {
+            if (v42bis_decompress(&state_b, buf, len))
+            {
+                fprintf(stderr, "Bad return code from decompression\n");
+                exit(2);
+            }
+            in_octets_to_date += len;
+        }
+        v42bis_decompress_flush(&state_b);
+        printf("%d bytes decompressed to %d bytes in %lds\n", in_octets_to_date, out_octets_to_date, time(NULL) - now);
+        close(v42bis_fd);
+        close(out_fd);
+    }
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
